/**
 * Copyright (C) 2020-2022 Dušan Vejnović <vaadin@dussan.org>
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.dussan.vaadin.dvirtual.client;

import java.util.Objects;

import org.apache.commons.lang3.ArrayUtils;
import org.apache.commons.lang3.StringUtils;
import org.dussan.vaadin.dvirtual.database.Database;
import org.dussan.vaadin.dvirtual.database.data.UserSessionData;
import org.dussan.vaadin.dvirtual.i18n.Translation;
import org.dussan.vaadin.dvirtual.info.ClientProperties;
import org.dussan.vaadin.dvirtual.runners.ClosingApplicationRunner;
import org.dussan.vaadin.dvirtual.runners.VirtualClientRunner;
import org.dussan.vaadin.dvirtual.ui.VirtualUI.VirtualUiAction;
import org.dussan.vaadin.dvirtual.utils.CanvasImageUtil;
import org.dussan.vaadin.dvirtual.utils.VirtualUtil;
import org.dussan.vaadin.dvirtual.utils.dialog.DialogConnectingUtil;
import org.dussan.vaadin.dvirtual.xpra.client.Client;
import org.dussan.vaadin.dvirtual.xpra.connector.Connector;
import org.dussan.vaadin.dvirtual.xpra.packets.base.WindowType;
import org.dussan.vaadin.dvirtual.xpra.window.Cursor;
import org.dussan.vaadin.dvirtual.xpra.window.Window;

import com.vaadin.flow.component.UI;
import com.vaadin.flow.component.UIDetachedException;
import com.vaadin.flow.component.html.Div;

public abstract class VirtualClient
    extends Client
{

    private static final long serialVersionUID = -6456049864286958418L;

    private static final int UNDEFINED_WINDOW = -1;

    private boolean closingApplicationDialogVisible = true;

    private boolean oldRunningApplicationDetached = true;

    private boolean windowClosedByUser = false;

    private transient Connector connector = null;

    private Div canvasDiv = null;

    private VirtualUtil virtualUtil = null;

    private transient VirtualClientRunner virtualClientRunner = null;

    public enum VirtualClientAction
    {
        CLIPBOARD_DATA,
        CLOSE_ALL_WINDOWS,
        CLOSE_APPLICATION,
        CLOSE_WINDOW,
        CREATE_NEW_OVERRIDE_REDIRECT,
        CREATE_NEW_WINDOW,
        DRAW_CANVAS,
        DRAW_CURSOR,
        DRAW_WINDOW_ICON,
        MOVE_RESIZE_WINDOW,
        NEW_WINDOW_FOCUS,
        NEW_WINDOW_TITLE,
        RECREATE_NEW_CHILD_WINDOW,
        RECREATE_NEW_WINDOW,
        RESIZE_WINDOW,
        START_CHILD_APPLICATION,
        UPDATE_WINDOWS_FOCUS
    }

    protected VirtualClient( Div canvasDiv, ClientProperties clientProperties, VirtualUtil virtualUtil )
    {
        super( clientProperties, virtualUtil );
        this.canvasDiv = canvasDiv;
        this.virtualUtil = virtualUtil;
    }

    public abstract void virtualUiAction( Integer id, VirtualUiAction action, Object... data );

    private void clipboardData( UI ui, Object... data )
    {
        if ( ArrayUtils.isNotEmpty( data ) && data.length == 1 )
        {
            ui.getPage()
              .executeJs( Command.JS.CLIPBOARD_DATA, Objects.toString( data[0], StringUtils.EMPTY ) );
        }
    }

    private void closeAllWindows( UI ui )
    {
        if ( isDisconnectedByClientRequest() )
        {
            ui.getPage()
              .executeJs( Command.JS.CLOSE_ALL_WINDOWS, canvasDiv.getElement() );
            setWindowClosedByUser( true );
        }
        else if ( isDisconnectedBySecurityError() || isDisconnectedByServerDetach() || isDisconnectedByServerError() )
        {
            // close application due to server detach/error
            ui.getPage()
              .executeJs( Command.JS.CLOSE_ALL_WINDOWS, canvasDiv.getElement() );
            setWindowClosedByUser( true );
            virtualUiAction( UNDEFINED_WINDOW, VirtualUiAction.CLOSE_APPLICATION );
        }
    }

    private void closeApplication()
    {
        if ( isClosingApplicationDialogVisible() && virtualUtil.isRunningApplicationInClosingProcess() )
        {
            DialogConnectingUtil dialogConnectingUtil = new DialogConnectingUtil( virtualUtil.getTranslation( Translation.APPLICATION_STOP ), StringUtils.EMPTY );
            dialogConnectingUtil.setButtonsVisible( false );
            dialogConnectingUtil.open();
            ClosingApplicationRunner closingApplicationRunner = new ClosingApplicationRunner( virtualUtil, connector, dialogConnectingUtil );
            virtualUtil.startRunnerService( closingApplicationRunner );
        }
        setClosingApplicationDialogVisible( true );
    }

    private void closeWindow( UI ui, Integer id, Object... data )
    {
        if ( ArrayUtils.isNotEmpty( data ) && data.length == 1 )
        {
            if ( isDisconnectedByServerError() || isDisconnectedByServerShutdown() )
            {
                // close application due to server error
                setWindowClosedByUser( true );
            }

            if ( Objects.nonNull( id ) )
            {
                if ( !isOldRunningApplicationDetached() )
                {
                    ui.getPage()
                      .executeJs( Command.JS.REMOVE_OLD_WINDOWS, canvasDiv.getElement() );
                }
                else
                {
                    boolean isNormalWindow = (boolean) data[0];
                    ui.getPage()
                      .executeJs( Command.JS.CLOSE_WINDOW, canvasDiv.getElement(), id );
                    if ( isNormalWindow )
                    {
                        virtualUiAction( id, VirtualUiAction.CLOSE_WINDOW );
                    }
                }
                setOldRunningApplicationDetached( true );
            }
        }
    }

    private void createNewWindow( UI ui, Integer id )
    {
        if ( Objects.nonNull( id ) && Objects.nonNull( getWindow( id ) ) )
        {
            Window window = getWindow( id );
            WindowType windowType = window.getWindowType();
            window.setTitle( isNormalWindow( window ) ? StringUtils.defaultIfBlank( window.getTitle(), virtualUtil.getTranslation( Translation.APPLICATION_TITLE_EMPTY ) ) : window.getTitle() );

            if ( !isOldRunningApplicationDetached() )
            {
                ui.getPage()
                  .executeJs( Command.JS.REMOVE_OLD_WINDOWS, canvasDiv.getElement() );
            }
            ui.getPage()
              .executeJs( Command.JS.CREATE_NEW_WINDOW, canvasDiv.getElement(), id, window.getParentId(), window.getTitle(), window.getX(), window.getY(), window.getWidth(), window.getHeight(),
                          windowType.toString(), window.isOverrideRedirect(), window.isClipboardEnabled(), window.isAutoCenterEnabled() );

            if ( window.isNotOverrideRedirect() )
            {
                virtualClientRunner.resetWaitIntervals();
                switch ( windowType )
                {
                    case DEFAULT:
                    case DIALOG:
                    case NORMAL:
                        virtualUiAction( id, VirtualUiAction.CREATE_NEW_WINDOW );
                        break;
                    default:
                        // nothing to do
                        break;
                }
            }
            setOldRunningApplicationDetached( true );
        }
    }

    private void drawCanvas( UI ui, Integer id, Object... data )
    {
        if ( Objects.nonNull( id ) && ArrayUtils.isNotEmpty( data ) && data.length == 1 )
        {
            ui.getUI()
              .ifPresent( u -> u.access( () -> {
                  u.getPage()
                   .executeJs( Command.JS.DRAW_CANVAS, canvasDiv.getElement(), id, Objects.toString( data[0], StringUtils.EMPTY ) );
                  u.push();
              } ) );
        }
    }

    private void drawCursor( UI ui, Integer id )
    {
        if ( Objects.nonNull( id ) && Objects.nonNull( getCursor() ) )
        {
            Cursor cursor = getCursor();
            ui.getPage()
              .executeJs( Command.JS.DRAW_CURSOR, canvasDiv.getElement(), id, cursor.getX(), cursor.getY(), cursor.getImage(), cursor.getPictureEncodingType() );
        }
    }

    private void drawWindowIcon( UI ui, Integer id )
    {
        if ( Objects.nonNull( id ) && Objects.nonNull( getWindow( id ) ) )
        {
            Window window = getWindow( id );
            if ( isNotSplashWindow( window ) )
            {
                ui.getPage()
                  .executeJs( Command.JS.DRAW_WINDOW_ICON, canvasDiv.getElement(), id, CanvasImageUtil.toBase64( window.getIcon() ) );
                virtualUiAction( id, VirtualUiAction.DRAW_WINDOW_ICON, window.getTitle(), window.getIcon(), window.isMaximized() );
            }
        }
    }

    private void moveResizeWindow( UI ui, Integer id )
    {
        if ( Objects.nonNull( id ) && Objects.nonNull( getWindow( id ) ) && !isForceUpdateEnabled() )
        {
            Window window = getWindow( id );
            if ( window.isVisible() || isNotNormalWindow( window ) )
            {
                ui.getPage()
                  .executeJs( Command.JS.MOVE_RESIZE_WINDOW, canvasDiv.getElement(), id, window.getX(), window.getY(), window.getWidth(), window.getHeight(), window.isMaximized() );
                if ( isNormalWindow( window ) )
                {
                    if ( !window.isMaximized() )
                    {
                        window.setRestoredWindowDimension( window.getX(), window.getY(), window.getWidth(), window.getHeight() );
                    }
                    virtualUiAction( id, VirtualUiAction.NEW_WINDOW_FOCUS, window.isMaximized() );
                }
            }
        }
    }

    private void newWindowFocus( UI ui, Integer id )
    {
        if ( Objects.nonNull( id ) && Objects.nonNull( getWindow( id ) ) )
        {
            ui.getPage()
              .executeJs( Command.JS.NEW_WINDOW_FOCUS, canvasDiv.getElement(), id );
            virtualUiAction( id, VirtualUiAction.NEW_WINDOW_FOCUS, getWindow( id ).isMaximized() );
        }
    }

    private void newWindowTitle( UI ui, Integer id )
    {
        if ( Objects.nonNull( id ) && Objects.nonNull( getWindow( id ) ) )
        {
            Window window = getWindow( id );
            window.setTitle( isNormalWindow( window ) ? StringUtils.defaultIfBlank( window.getTitle(), virtualUtil.getTranslation( Translation.APPLICATION_TITLE_EMPTY ) ) : window.getTitle() );
            ui.getPage()
              .executeJs( Command.JS.NEW_WINDOW_TITLE, canvasDiv.getElement(), id, window.getTitle() );
            virtualUiAction( id, VirtualUiAction.NEW_WINDOW_TITLE, window.getTitle(), window.getIcon(), window.isMaximized() );
        }
    }

    private void recreateNewChildWindow( Integer id )
    {
        if ( Objects.nonNull( id ) && id == UNDEFINED_WINDOW )
        {
            virtualClientRunner.resetWaitIntervals();
            virtualUiAction( id, VirtualUiAction.RECREATE_NEW_WINDOW );
        }
    }

    private void recreateNewMainWindow( Integer id )
    {
        if ( Objects.nonNull( id ) && id == UNDEFINED_WINDOW )
        {
            virtualUiAction( id, VirtualUiAction.RECREATE_NEW_WINDOW );
        }
    }

    private void resizeWindow( UI ui, Integer id, Object... data )
    {
        if ( Objects.nonNull( id ) && !isForceUpdateEnabled() && ArrayUtils.isNotEmpty( data ) && data.length == 5 )
        {
            boolean isMaximized = (boolean) data[4];
            int height = (int) data[3];
            int width = (int) data[2];
            int x = (int) data[0];
            int y = (int) data[1];
            ui.getPage()
              .executeJs( Command.JS.RESIZE_WINDOW, canvasDiv.getElement(), id, x, y, width, height, isMaximized );
        }
    }

    private void startChildrenApplication( Integer id )
    {
        if ( Objects.nonNull( virtualClientRunner ) && id == UNDEFINED_WINDOW )
        {
            virtualClientRunner.resetWaitIntervals();
            virtualUtil.setChildApplicationCanStarted( true );
            virtualUiAction( id, VirtualUiAction.START_CHILD_APPLICATION );
        }
    }

    private void updateWindowFocus( Integer id )
    {
        if ( Objects.nonNull( id ) && id == UNDEFINED_WINDOW )
        {
            virtualUiAction( id, VirtualUiAction.UPDATE_WINDOWS_FOCUS );
        }
    }

    protected void windowCreatedProcessor( boolean windowCreated, String errorMessage )
    {
        // noting to do: this method will be overwritten in the classes for testing applications
    }

    public Connector getConnector()
    {
        return connector;
    }

    public boolean isClosingApplicationDialogVisible()
    {
        return closingApplicationDialogVisible;
    }

    public void setClosingApplicationDialogVisible( boolean closingApplicationDialogVisible )
    {
        this.closingApplicationDialogVisible = closingApplicationDialogVisible;
    }

    public boolean isOldRunningApplicationDetached()
    {
        return oldRunningApplicationDetached;
    }

    public void setOldRunningApplicationDetached( boolean oldRunningApplicationDetached )
    {
        this.oldRunningApplicationDetached = oldRunningApplicationDetached;
    }

    public boolean isWindowClosedByUser()
    {
        return windowClosedByUser;
    }

    public boolean isNotWindowClosedByUser()
    {
        return !windowClosedByUser;
    }

    public void setWindowClosedByUser( boolean windowClosedByUser )
    {
        this.windowClosedByUser = windowClosedByUser;
    }

    public void connect( String host, int xpraPort, Database database, UserSessionData userSessionData, DialogConnectingUtil dialogConnectingUtil )
    {
        connector = new Connector( this, host, xpraPort, dialogConnectingUtil, virtualUtil );
        connector.setUserSessionData( database, userSessionData );
        virtualClientRunner = new VirtualClientRunner( connector, canvasDiv )
        {
            @Override
            public void windowCreated( boolean windowCreated )
            {
                if ( windowCreated )
                {
                    if ( virtualUtil.isLogOutInProcess() )
                    {
                        virtualUiAction( UNDEFINED_WINDOW, VirtualUiAction.CLOSE_APPLICATION );
                    }
                    else if ( virtualUtil.haveToStartChildApplication() && virtualUtil.canNotChildApplicationStarted() )
                    {
                        virtualClientAction( UNDEFINED_WINDOW, VirtualClientAction.START_CHILD_APPLICATION );
                    }
                }
            }
        };
        virtualClientRunner.setVirtualUtil( virtualUtil );
        virtualClientRunner.setServerData( database, userSessionData );
        virtualUtil.startRunnerService( virtualClientRunner );
    }

    public void resetReconnectCounter()
    {
        if ( Objects.nonNull( connector ) )
        {
            connector.resetReconnectCounter();
        }
    }

    public synchronized void detachServer()
    {
        if ( Objects.nonNull( connector ) )
        {
            setClosingApplicationDialogVisible( false );
            connector.disconnect();
            connector = null;
        }
    }

    public synchronized void disconnect()
    {
        if ( Objects.nonNull( connector ) && connector.isRunning() )
        {
            connector.disconnect();
        }
    }

    public synchronized void stopServer()
    {
        if ( Objects.nonNull( connector ) )
        {
            connector.disconnect();
            connector.stopServer();
            connector = null;
        }
    }

    @Override
    public void virtualClientAction( Integer id, VirtualClientAction action, Object... data )
    {
        canvasDiv.getUI()
                 .ifPresent( ui -> {
                     try
                     {
                         if ( Objects.nonNull( ui ) )
                         {
                             ui.access( () -> {
                                 switch ( action )
                                 {
                                     case CLIPBOARD_DATA:
                                         clipboardData( ui, data );
                                         break;
                                     case CLOSE_ALL_WINDOWS:
                                         closeAllWindows( ui );
                                         break;
                                     case CLOSE_APPLICATION:
                                         closeApplication();
                                         break;
                                     case CLOSE_WINDOW:
                                         closeWindow( ui, id, data );
                                         break;
                                     case CREATE_NEW_OVERRIDE_REDIRECT:
                                     case CREATE_NEW_WINDOW:
                                         createNewWindow( ui, id );
                                         break;
                                     case DRAW_CANVAS:
                                         drawCanvas( ui, id, data );
                                         break;
                                     case DRAW_CURSOR:
                                         drawCursor( ui, id );
                                         break;
                                     case DRAW_WINDOW_ICON:
                                         drawWindowIcon( ui, id );
                                         break;
                                     case MOVE_RESIZE_WINDOW:
                                         moveResizeWindow( ui, id );
                                         break;
                                     case NEW_WINDOW_FOCUS:
                                         newWindowFocus( ui, id );
                                         break;
                                     case NEW_WINDOW_TITLE:
                                         newWindowTitle( ui, id );
                                         break;
                                     case RECREATE_NEW_CHILD_WINDOW:
                                         recreateNewChildWindow( id );
                                         break;
                                     case RECREATE_NEW_WINDOW:
                                         recreateNewMainWindow( id );
                                         break;
                                     case RESIZE_WINDOW:
                                         resizeWindow( ui, id, data );
                                         break;
                                     case START_CHILD_APPLICATION:
                                         startChildrenApplication( id );
                                         break;
                                     case UPDATE_WINDOWS_FOCUS:
                                         updateWindowFocus( id );
                                         break;
                                     default:
                                         break;
                                 }
                                 ui.push();
                             } );
                         }
                     }
                     catch ( UIDetachedException exception )
                     {
                         // nothing to do
                     }
                 } );
    }

}
