/**
 * Copyright (C) 2020-2022 Dušan Vejnović <vaadin@dussan.org>
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.dussan.vaadin.dvirtual.database.data;

import java.io.Serializable;

import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.Table;
import javax.persistence.UniqueConstraint;

import org.apache.commons.lang3.StringUtils;

@Entity( name = "UserData" )
@Table( name = "dv_user", uniqueConstraints = { @UniqueConstraint( columnNames = { "user_name" } ) } )
public class UserData
    implements Serializable
{

    private static final long serialVersionUID = -9013171862490432460L;

    public static final int DEFAULT_MAX_APPLICATIONS = 2;

    public static final int DEFAULT_MAX_FILE_SIZE = 10;

    public static final long NEW_USER = 0;

    public static final long UNDEFINED_USER = -1;

    public static final Boolean DEFAULT_ADMINISTRATOR = Boolean.FALSE;

    public static final Boolean DEFAULT_BANDWIDTH_LIMIT = Boolean.FALSE;

    public static final Boolean DEFAULT_DOWNLOAD = Boolean.FALSE;

    public static final Boolean DEFAULT_ENABLED = Boolean.TRUE;

    public static final Boolean DEFAULT_FULL_SCREEN = Boolean.TRUE;

    public static final Boolean DEFAULT_SCREENSHOT = Boolean.FALSE;

    public static final Boolean DEFAULT_UPLOAD = Boolean.FALSE;

    public static final String NEW_DATA = StringUtils.EMPTY;

    private static final int BYTE_FACOR = 1000000;

    @Id
    @GeneratedValue( strategy = GenerationType.AUTO )
    private Long id;

    @Column( name = "user_name", length = 100, nullable = false )
    private String userName;

    @Column( name = "full_name", length = 100, nullable = true )
    private String fullName;

    @Column( name = "max_applications", nullable = false )
    private Integer maxApplications;

    @Column( name = "max_file_size", nullable = false )
    private Integer maxFileSize;

    @Column( name = "administrator", nullable = false )
    private Boolean administrator;

    @Column( name = "upload", nullable = false )
    private Boolean upload;

    @Column( name = "download", nullable = false )
    private Boolean download;

    @Column( name = "screenshot", nullable = false )
    private Boolean screenshot;

    @Column( name = "full_screen", nullable = false )
    private Boolean fullScreen;

    @Column( name = "enabled", nullable = false )
    private Boolean enabled;

    public UserData()
    {
        setAdministrator( DEFAULT_ADMINISTRATOR );
        setDownload( DEFAULT_DOWNLOAD );
        setFullScreen( DEFAULT_FULL_SCREEN );
        setEnabled( DEFAULT_ENABLED );
        setMaxApplications( DEFAULT_MAX_APPLICATIONS );
        setMaxFileSize( DEFAULT_MAX_FILE_SIZE );
        setScreenshot( DEFAULT_SCREENSHOT );
        setUpload( DEFAULT_UPLOAD );
    }

    public UserData( long id, String userName, String fullName, int maxApplications, int maxFileSize, boolean administrator, boolean screenshot, boolean upload, boolean download, boolean fullScreen,
                     boolean enabled )
    {
        setId( id );
        setUserName( userName );
        setFullName( fullName );
        setMaxApplications( maxApplications );
        setMaxFileSize( maxFileSize );
        setAdministrator( administrator );
        setScreenshot( screenshot );
        setUpload( upload );
        setDownload( download );
        setFullScreen( fullScreen );
        setEnabled( enabled );
    }

    public Long getId()
    {
        return id;
    }

    public void setId( Long id )
    {
        this.id = id;
    }

    public String getUserName()
    {
        return userName;
    }

    public void setUserName( String userName )
    {
        this.userName = StringUtils.stripToEmpty( userName );
    }

    public String getFullName()
    {
        return fullName;
    }

    public void setFullName( String fullName )
    {
        this.fullName = StringUtils.stripToEmpty( fullName );
    }

    public Integer getMaxApplications()
    {
        return maxApplications;
    }

    public void setMaxApplications( Integer maxApplications )
    {
        this.maxApplications = maxApplications;
    }

    public Integer getMaxFileSize()
    {
        return maxFileSize;
    }

    public Integer getMaxFileSizeInBytes()
    {
        return ( BYTE_FACOR * maxFileSize );
    }

    public void setMaxFileSize( Integer maxFileSize )
    {
        this.maxFileSize = maxFileSize;
    }

    public Boolean isAdministrator()
    {
        return administrator;
    }

    public void setAdministrator( Boolean administrator )
    {
        this.administrator = administrator;
    }

    public Boolean isScreenshot()
    {
        return screenshot;
    }

    public void setScreenshot( Boolean screenshot )
    {
        this.screenshot = screenshot;
    }

    public Boolean isUpload()
    {
        return upload;
    }

    public void setUpload( Boolean upload )
    {
        this.upload = upload;
    }

    public Boolean isDownload()
    {
        return download;
    }

    public void setDownload( Boolean download )
    {
        this.download = download;
    }

    public Boolean isFullScreen()
    {
        return fullScreen;
    }

    public void setFullScreen( Boolean fullScreen )
    {
        this.fullScreen = fullScreen;
    }

    public Boolean isEnabled()
    {
        return enabled;
    }

    public void setEnabled( Boolean enabled )
    {
        this.enabled = enabled;
    }

    @Override
    public int hashCode()
    {
        final int prime = 31;
        int result = 1;
        result = prime * result + ( ( administrator == null ) ? 0 : administrator.hashCode() );
        result = prime * result + ( ( download == null ) ? 0 : download.hashCode() );
        result = prime * result + ( ( enabled == null ) ? 0 : enabled.hashCode() );
        result = prime * result + ( ( fullName == null ) ? 0 : fullName.hashCode() );
        result = prime * result + ( ( fullScreen == null ) ? 0 : fullScreen.hashCode() );
        result = prime * result + ( ( id == null ) ? 0 : id.hashCode() );
        result = prime * result + ( ( maxApplications == null ) ? 0 : maxApplications.hashCode() );
        result = prime * result + ( ( maxFileSize == null ) ? 0 : maxFileSize.hashCode() );
        result = prime * result + ( ( screenshot == null ) ? 0 : screenshot.hashCode() );
        result = prime * result + ( ( upload == null ) ? 0 : upload.hashCode() );
        result = prime * result + ( ( userName == null ) ? 0 : userName.hashCode() );
        return result;
    }

    @Override
    public boolean equals( Object obj )
    {
        if ( this == obj )
        {
            return true;
        }
        if ( obj == null )
        {
            return false;
        }
        if ( !( obj instanceof UserData ) )
        {
            return false;
        }
        UserData other = (UserData) obj;
        if ( administrator == null )
        {
            if ( other.administrator != null )
            {
                return false;
            }
        }
        else if ( !administrator.equals( other.administrator ) )
        {
            return false;
        }
        if ( download == null )
        {
            if ( other.download != null )
            {
                return false;
            }
        }
        else if ( !download.equals( other.download ) )
        {
            return false;
        }
        if ( enabled == null )
        {
            if ( other.enabled != null )
            {
                return false;
            }
        }
        else if ( !enabled.equals( other.enabled ) )
        {
            return false;
        }
        if ( fullName == null )
        {
            if ( other.fullName != null )
            {
                return false;
            }
        }
        else if ( !fullName.equals( other.fullName ) )
        {
            return false;
        }
        if ( fullScreen == null )
        {
            if ( other.fullScreen != null )
            {
                return false;
            }
        }
        else if ( !fullScreen.equals( other.fullScreen ) )
        {
            return false;
        }
        if ( id == null )
        {
            if ( other.id != null )
            {
                return false;
            }
        }
        else if ( !id.equals( other.id ) )
        {
            return false;
        }
        if ( maxApplications == null )
        {
            if ( other.maxApplications != null )
            {
                return false;
            }
        }
        else if ( !maxApplications.equals( other.maxApplications ) )
        {
            return false;
        }
        if ( maxFileSize == null )
        {
            if ( other.maxFileSize != null )
            {
                return false;
            }
        }
        else if ( !maxFileSize.equals( other.maxFileSize ) )
        {
            return false;
        }
        if ( screenshot == null )
        {
            if ( other.screenshot != null )
            {
                return false;
            }
        }
        else if ( !screenshot.equals( other.screenshot ) )
        {
            return false;
        }
        if ( upload == null )
        {
            if ( other.upload != null )
            {
                return false;
            }
        }
        else if ( !upload.equals( other.upload ) )
        {
            return false;
        }
        if ( userName == null )
        {
            if ( other.userName != null )
            {
                return false;
            }
        }
        else if ( !userName.equals( other.userName ) )
        {
            return false;
        }
        return true;
    }

}
