/**
 * Copyright (C) 2020-2022 Dušan Vejnović <vaadin@dussan.org>
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.dussan.vaadin.dvirtual.info;

import java.io.Serializable;
import java.util.HashMap;
import java.util.Map;
import java.util.Objects;

import org.apache.commons.collections4.MapUtils;

public class KeyboardCharName
    implements Serializable
{

    private static final long serialVersionUID = -1065506852106493001L;

    private Map<String, String> charName = new HashMap<>();

    private Map<String, Integer> keysymUnicode = new HashMap<>();

    public KeyboardCharName()
    {
        keysymUnicode.put( "space", 0x0020 );
        keysymUnicode.put( "exclam", 0x0021 );
        keysymUnicode.put( "quotedbl", 0x0022 );
        keysymUnicode.put( "numbersign", 0x0023 );
        keysymUnicode.put( "dollar", 0x0024 );
        keysymUnicode.put( "percent", 0x0025 );
        keysymUnicode.put( "ampersand", 0x0026 );
        keysymUnicode.put( "apostrophe", 0x0027 );
        keysymUnicode.put( "parenleft", 0x0028 );
        keysymUnicode.put( "parenright", 0x0029 );
        keysymUnicode.put( "asterisk", 0x002A );
        keysymUnicode.put( "plus", 0x002B );
        keysymUnicode.put( "comma", 0x002C );
        keysymUnicode.put( "minus", 0x002D );
        keysymUnicode.put( "period", 0x002E );
        keysymUnicode.put( "slash", 0x002F );
        keysymUnicode.put( "0", 0x0030 );
        keysymUnicode.put( "1", 0x0031 );
        keysymUnicode.put( "2", 0x0032 );
        keysymUnicode.put( "3", 0x0033 );
        keysymUnicode.put( "4", 0x0034 );
        keysymUnicode.put( "5", 0x0035 );
        keysymUnicode.put( "6", 0x0036 );
        keysymUnicode.put( "7", 0x0037 );
        keysymUnicode.put( "8", 0x0038 );
        keysymUnicode.put( "9", 0x0039 );
        keysymUnicode.put( "colon", 0x003A );
        keysymUnicode.put( "semicolon", 0x003B );
        keysymUnicode.put( "less", 0x003C );
        keysymUnicode.put( "equal", 0x003D );
        keysymUnicode.put( "greater", 0x003E );
        keysymUnicode.put( "question", 0x003F );
        keysymUnicode.put( "at", 0x0040 );
        keysymUnicode.put( "A", 0x0041 );
        keysymUnicode.put( "B", 0x0042 );
        keysymUnicode.put( "C", 0x0043 );
        keysymUnicode.put( "D", 0x0044 );
        keysymUnicode.put( "E", 0x0045 );
        keysymUnicode.put( "F", 0x0046 );
        keysymUnicode.put( "G", 0x0047 );
        keysymUnicode.put( "H", 0x0048 );
        keysymUnicode.put( "I", 0x0049 );
        keysymUnicode.put( "J", 0x004A );
        keysymUnicode.put( "K", 0x004B );
        keysymUnicode.put( "L", 0x004C );
        keysymUnicode.put( "M", 0x004D );
        keysymUnicode.put( "N", 0x004E );
        keysymUnicode.put( "O", 0x004F );
        keysymUnicode.put( "P", 0x0050 );
        keysymUnicode.put( "Q", 0x0051 );
        keysymUnicode.put( "R", 0x0052 );
        keysymUnicode.put( "S", 0x0053 );
        keysymUnicode.put( "T", 0x0054 );
        keysymUnicode.put( "U", 0x0055 );
        keysymUnicode.put( "V", 0x0056 );
        keysymUnicode.put( "W", 0x0057 );
        keysymUnicode.put( "X", 0x0058 );
        keysymUnicode.put( "Y", 0x0059 );
        keysymUnicode.put( "Z", 0x005A );
        keysymUnicode.put( "bracketleft", 0x005B );
        keysymUnicode.put( "backslash", 0x005C );
        keysymUnicode.put( "bracketright", 0x005D );
        keysymUnicode.put( "asciicircum", 0x005E );
        keysymUnicode.put( "underscore", 0x005F );
        keysymUnicode.put( "grave", 0x0060 );
        keysymUnicode.put( "a", 0x0061 );
        keysymUnicode.put( "b", 0x0062 );
        keysymUnicode.put( "c", 0x0063 );
        keysymUnicode.put( "d", 0x0064 );
        keysymUnicode.put( "e", 0x0065 );
        keysymUnicode.put( "f", 0x0066 );
        keysymUnicode.put( "g", 0x0067 );
        keysymUnicode.put( "h", 0x0068 );
        keysymUnicode.put( "i", 0x0069 );
        keysymUnicode.put( "j", 0x006A );
        keysymUnicode.put( "k", 0x006B );
        keysymUnicode.put( "l", 0x006C );
        keysymUnicode.put( "m", 0x006D );
        keysymUnicode.put( "n", 0x006E );
        keysymUnicode.put( "o", 0x006F );
        keysymUnicode.put( "p", 0x0070 );
        keysymUnicode.put( "q", 0x0071 );
        keysymUnicode.put( "r", 0x0072 );
        keysymUnicode.put( "s", 0x0073 );
        keysymUnicode.put( "t", 0x0074 );
        keysymUnicode.put( "u", 0x0075 );
        keysymUnicode.put( "v", 0x0076 );
        keysymUnicode.put( "w", 0x0077 );
        keysymUnicode.put( "x", 0x0078 );
        keysymUnicode.put( "y", 0x0079 );
        keysymUnicode.put( "z", 0x007A );
        keysymUnicode.put( "braceleft", 0x007B );
        keysymUnicode.put( "bar", 0x007C );
        keysymUnicode.put( "braceright", 0x007D );
        keysymUnicode.put( "asciitilde", 0x007E );
        keysymUnicode.put( "nobreakspace", 0x00A0 );
        keysymUnicode.put( "exclamdown", 0x00A1 );
        keysymUnicode.put( "cent", 0x00A2 );
        keysymUnicode.put( "sterling", 0x00A3 );
        keysymUnicode.put( "currency", 0x00A4 );
        keysymUnicode.put( "yen", 0x00A5 );
        keysymUnicode.put( "brokenbar", 0x00A6 );
        keysymUnicode.put( "section", 0x00A7 );
        keysymUnicode.put( "diaeresis", 0x00A8 );
        keysymUnicode.put( "copyright", 0x00A9 );
        keysymUnicode.put( "ordfeminine", 0x00AA );
        keysymUnicode.put( "guillemotleft", 0x00AB );
        keysymUnicode.put( "notsign", 0x00AC );
        keysymUnicode.put( "hyphen", 0x00AD );
        keysymUnicode.put( "registered", 0x00AE );
        keysymUnicode.put( "macron", 0x00AF );
        keysymUnicode.put( "degree", 0x00B0 );
        keysymUnicode.put( "plusminus", 0x00B1 );
        keysymUnicode.put( "twosuperior", 0x00B2 );
        keysymUnicode.put( "threesuperior", 0x00B3 );
        keysymUnicode.put( "acute", 0x00B4 );
        keysymUnicode.put( "mu", 0x00B5 );
        keysymUnicode.put( "paragraph", 0x00B6 );
        keysymUnicode.put( "periodcentered", 0x00B7 );
        keysymUnicode.put( "cedilla", 0x00B8 );
        keysymUnicode.put( "onesuperior", 0x00B9 );
        keysymUnicode.put( "masculine", 0x00BA );
        keysymUnicode.put( "guillemotright", 0x00BB );
        keysymUnicode.put( "onequarter", 0x00BC );
        keysymUnicode.put( "onehalf", 0x00BD );
        keysymUnicode.put( "threequarters", 0x00BE );
        keysymUnicode.put( "questiondown", 0x00BF );
        keysymUnicode.put( "Agrave", 0x00C0 );
        keysymUnicode.put( "Aacute", 0x00C1 );
        keysymUnicode.put( "Acircumflex", 0x00C2 );
        keysymUnicode.put( "Atilde", 0x00C3 );
        keysymUnicode.put( "Adiaeresis", 0x00C4 );
        keysymUnicode.put( "Aring", 0x00C5 );
        keysymUnicode.put( "AE", 0x00C6 );
        keysymUnicode.put( "Ccedilla", 0x00C7 );
        keysymUnicode.put( "Egrave", 0x00C8 );
        keysymUnicode.put( "Eacute", 0x00C9 );
        keysymUnicode.put( "Ecircumflex", 0x00CA );
        keysymUnicode.put( "Ediaeresis", 0x00CB );
        keysymUnicode.put( "Igrave", 0x00CC );
        keysymUnicode.put( "Iacute", 0x00CD );
        keysymUnicode.put( "Icircumflex", 0x00CE );
        keysymUnicode.put( "Idiaeresis", 0x00CF );
        keysymUnicode.put( "ETH", 0x00D0 );
        keysymUnicode.put( "Ntilde", 0x00D1 );
        keysymUnicode.put( "Ograve", 0x00D2 );
        keysymUnicode.put( "Oacute", 0x00D3 );
        keysymUnicode.put( "Ocircumflex", 0x00D4 );
        keysymUnicode.put( "Otilde", 0x00D5 );
        keysymUnicode.put( "Odiaeresis", 0x00D6 );
        keysymUnicode.put( "multiply", 0x00D7 );
        keysymUnicode.put( "Ooblique", 0x00D8 );
        keysymUnicode.put( "Oslash", 0x00D8 );
        keysymUnicode.put( "Ugrave", 0x00D9 );
        keysymUnicode.put( "Uacute", 0x00DA );
        keysymUnicode.put( "Ucircumflex", 0x00DB );
        keysymUnicode.put( "Udiaeresis", 0x00DC );
        keysymUnicode.put( "Yacute", 0x00DD );
        keysymUnicode.put( "THORN", 0x00DE );
        keysymUnicode.put( "ssharp", 0x00DF );
        keysymUnicode.put( "agrave", 0x00E0 );
        keysymUnicode.put( "aacute", 0x00E1 );
        keysymUnicode.put( "acircumflex", 0x00E2 );
        keysymUnicode.put( "atilde", 0x00E3 );
        keysymUnicode.put( "adiaeresis", 0x00E4 );
        keysymUnicode.put( "aring", 0x00E5 );
        keysymUnicode.put( "ae", 0x00E6 );
        keysymUnicode.put( "ccedilla", 0x00E7 );
        keysymUnicode.put( "egrave", 0x00E8 );
        keysymUnicode.put( "eacute", 0x00E9 );
        keysymUnicode.put( "ecircumflex", 0x00EA );
        keysymUnicode.put( "ediaeresis", 0x00EB );
        keysymUnicode.put( "igrave", 0x00EC );
        keysymUnicode.put( "iacute", 0x00ED );
        keysymUnicode.put( "icircumflex", 0x00EE );
        keysymUnicode.put( "idiaeresis", 0x00EF );
        keysymUnicode.put( "eth", 0x00F0 );
        keysymUnicode.put( "ntilde", 0x00F1 );
        keysymUnicode.put( "ograve", 0x00F2 );
        keysymUnicode.put( "oacute", 0x00F3 );
        keysymUnicode.put( "ocircumflex", 0x00F4 );
        keysymUnicode.put( "otilde", 0x00F5 );
        keysymUnicode.put( "odiaeresis", 0x00F6 );
        keysymUnicode.put( "division", 0x00F7 );
        keysymUnicode.put( "oslash", 0x00F8 );
        keysymUnicode.put( "ooblique", 0x00F8 );
        keysymUnicode.put( "ugrave", 0x00F9 );
        keysymUnicode.put( "uacute", 0x00FA );
        keysymUnicode.put( "ucircumflex", 0x00FB );
        keysymUnicode.put( "udiaeresis", 0x00FC );
        keysymUnicode.put( "yacute", 0x00FD );
        keysymUnicode.put( "thorn", 0x00FE );
        keysymUnicode.put( "ydiaeresis", 0x00FF );
        keysymUnicode.put( "Aogonek", 0x0104 );
        keysymUnicode.put( "breve", 0x02D8 );
        keysymUnicode.put( "Lstroke", 0x0141 );
        keysymUnicode.put( "Lcaron", 0x013D );
        keysymUnicode.put( "Sacute", 0x015A );
        keysymUnicode.put( "Scaron", 0x0160 );
        keysymUnicode.put( "Scedilla", 0x015E );
        keysymUnicode.put( "Tcaron", 0x0164 );
        keysymUnicode.put( "Zacute", 0x0179 );
        keysymUnicode.put( "Zcaron", 0x017D );
        keysymUnicode.put( "Zabovedot", 0x017B );
        keysymUnicode.put( "aogonek", 0x0105 );
        keysymUnicode.put( "ogonek", 0x02DB );
        keysymUnicode.put( "lstroke", 0x0142 );
        keysymUnicode.put( "lcaron", 0x013E );
        keysymUnicode.put( "sacute", 0x015B );
        keysymUnicode.put( "caron", 0x02C7 );
        keysymUnicode.put( "scaron", 0x0161 );
        keysymUnicode.put( "scedilla", 0x015F );
        keysymUnicode.put( "tcaron", 0x0165 );
        keysymUnicode.put( "zacute", 0x017A );
        keysymUnicode.put( "doubleacute", 0x02DD );
        keysymUnicode.put( "zcaron", 0x017E );
        keysymUnicode.put( "zabovedot", 0x017C );
        keysymUnicode.put( "Racute", 0x0154 );
        keysymUnicode.put( "Abreve", 0x0102 );
        keysymUnicode.put( "Lacute", 0x0139 );
        keysymUnicode.put( "Cacute", 0x0106 );
        keysymUnicode.put( "Ccaron", 0x010C );
        keysymUnicode.put( "Eogonek", 0x0118 );
        keysymUnicode.put( "Ecaron", 0x011A );
        keysymUnicode.put( "Dcaron", 0x010E );
        keysymUnicode.put( "Dstroke", 0x0110 );
        keysymUnicode.put( "Nacute", 0x0143 );
        keysymUnicode.put( "Ncaron", 0x0147 );
        keysymUnicode.put( "Odoubleacute", 0x0150 );
        keysymUnicode.put( "Rcaron", 0x0158 );
        keysymUnicode.put( "Uring", 0x016E );
        keysymUnicode.put( "Udoubleacute", 0x0170 );
        keysymUnicode.put( "Tcedilla", 0x0162 );
        keysymUnicode.put( "racute", 0x0155 );
        keysymUnicode.put( "abreve", 0x0103 );
        keysymUnicode.put( "lacute", 0x013A );
        keysymUnicode.put( "cacute", 0x0107 );
        keysymUnicode.put( "ccaron", 0x010D );
        keysymUnicode.put( "eogonek", 0x0119 );
        keysymUnicode.put( "ecaron", 0x011B );
        keysymUnicode.put( "dcaron", 0x010F );
        keysymUnicode.put( "dstroke", 0x0111 );
        keysymUnicode.put( "nacute", 0x0144 );
        keysymUnicode.put( "ncaron", 0x0148 );
        keysymUnicode.put( "odoubleacute", 0x0151 );
        keysymUnicode.put( "rcaron", 0x0159 );
        keysymUnicode.put( "uring", 0x016F );
        keysymUnicode.put( "udoubleacute", 0x0171 );
        keysymUnicode.put( "tcedilla", 0x0163 );
        keysymUnicode.put( "abovedot", 0x02D9 );
        keysymUnicode.put( "Hstroke", 0x0126 );
        keysymUnicode.put( "Hcircumflex", 0x0124 );
        keysymUnicode.put( "Iabovedot", 0x0130 );
        keysymUnicode.put( "Gbreve", 0x011E );
        keysymUnicode.put( "Jcircumflex", 0x0134 );
        keysymUnicode.put( "hstroke", 0x0127 );
        keysymUnicode.put( "hcircumflex", 0x0125 );
        keysymUnicode.put( "idotless", 0x0131 );
        keysymUnicode.put( "gbreve", 0x011F );
        keysymUnicode.put( "jcircumflex", 0x0135 );
        keysymUnicode.put( "Cabovedot", 0x010A );
        keysymUnicode.put( "Ccircumflex", 0x0108 );
        keysymUnicode.put( "Gabovedot", 0x0120 );
        keysymUnicode.put( "Gcircumflex", 0x011C );
        keysymUnicode.put( "Ubreve", 0x016C );
        keysymUnicode.put( "Scircumflex", 0x015C );
        keysymUnicode.put( "cabovedot", 0x010B );
        keysymUnicode.put( "ccircumflex", 0x0109 );
        keysymUnicode.put( "gabovedot", 0x0121 );
        keysymUnicode.put( "gcircumflex", 0x011D );
        keysymUnicode.put( "ubreve", 0x016D );
        keysymUnicode.put( "scircumflex", 0x015D );
        keysymUnicode.put( "kra", 0x0138 );
        keysymUnicode.put( "Rcedilla", 0x0156 );
        keysymUnicode.put( "Itilde", 0x0128 );
        keysymUnicode.put( "Lcedilla", 0x013B );
        keysymUnicode.put( "Emacron", 0x0112 );
        keysymUnicode.put( "Gcedilla", 0x0122 );
        keysymUnicode.put( "Tslash", 0x0166 );
        keysymUnicode.put( "rcedilla", 0x0157 );
        keysymUnicode.put( "itilde", 0x0129 );
        keysymUnicode.put( "lcedilla", 0x013C );
        keysymUnicode.put( "emacron", 0x0113 );
        keysymUnicode.put( "gcedilla", 0x0123 );
        keysymUnicode.put( "tslash", 0x0167 );
        keysymUnicode.put( "ENG", 0x014A );
        keysymUnicode.put( "eng", 0x014B );
        keysymUnicode.put( "Amacron", 0x0100 );
        keysymUnicode.put( "Iogonek", 0x012E );
        keysymUnicode.put( "Eabovedot", 0x0116 );
        keysymUnicode.put( "Imacron", 0x012A );
        keysymUnicode.put( "Ncedilla", 0x0145 );
        keysymUnicode.put( "Omacron", 0x014C );
        keysymUnicode.put( "Kcedilla", 0x0136 );
        keysymUnicode.put( "Uogonek", 0x0172 );
        keysymUnicode.put( "Utilde", 0x0168 );
        keysymUnicode.put( "Umacron", 0x016A );
        keysymUnicode.put( "amacron", 0x0101 );
        keysymUnicode.put( "iogonek", 0x012F );
        keysymUnicode.put( "eabovedot", 0x0117 );
        keysymUnicode.put( "imacron", 0x012B );
        keysymUnicode.put( "ncedilla", 0x0146 );
        keysymUnicode.put( "omacron", 0x014D );
        keysymUnicode.put( "kcedilla", 0x0137 );
        keysymUnicode.put( "uogonek", 0x0173 );
        keysymUnicode.put( "utilde", 0x0169 );
        keysymUnicode.put( "umacron", 0x016B );
        keysymUnicode.put( "Wcircumflex", 0x0174 );
        keysymUnicode.put( "wcircumflex", 0x0175 );
        keysymUnicode.put( "Ycircumflex", 0x0176 );
        keysymUnicode.put( "ycircumflex", 0x0177 );
        keysymUnicode.put( "Babovedot", 0x1E02 );
        keysymUnicode.put( "babovedot", 0x1E03 );
        keysymUnicode.put( "Dabovedot", 0x1E0A );
        keysymUnicode.put( "dabovedot", 0x1E0B );
        keysymUnicode.put( "Fabovedot", 0x1E1E );
        keysymUnicode.put( "fabovedot", 0x1E1F );
        keysymUnicode.put( "Mabovedot", 0x1E40 );
        keysymUnicode.put( "mabovedot", 0x1E41 );
        keysymUnicode.put( "Pabovedot", 0x1E56 );
        keysymUnicode.put( "pabovedot", 0x1E57 );
        keysymUnicode.put( "Sabovedot", 0x1E60 );
        keysymUnicode.put( "sabovedot", 0x1E61 );
        keysymUnicode.put( "Tabovedot", 0x1E6A );
        keysymUnicode.put( "tabovedot", 0x1E6B );
        keysymUnicode.put( "Wgrave", 0x1E80 );
        keysymUnicode.put( "wgrave", 0x1E81 );
        keysymUnicode.put( "Wacute", 0x1E82 );
        keysymUnicode.put( "wacute", 0x1E83 );
        keysymUnicode.put( "Wdiaeresis", 0x1E84 );
        keysymUnicode.put( "wdiaeresis", 0x1E85 );
        keysymUnicode.put( "Ygrave", 0x1EF2 );
        keysymUnicode.put( "ygrave", 0x1EF3 );
        keysymUnicode.put( "OE", 0x0152 );
        keysymUnicode.put( "oe", 0x0153 );
        keysymUnicode.put( "Ydiaeresis", 0x0178 );
        keysymUnicode.put( "overline", 0x203E );
        keysymUnicode.put( "kana_fullstop", 0x3002 );
        keysymUnicode.put( "kana_openingbracket", 0x300C );
        keysymUnicode.put( "kana_closingbracket", 0x300D );
        keysymUnicode.put( "kana_comma", 0x3001 );
        keysymUnicode.put( "kana_conjunctive", 0x30FB );
        keysymUnicode.put( "kana_WO", 0x30F2 );
        keysymUnicode.put( "kana_a", 0x30A1 );
        keysymUnicode.put( "kana_i", 0x30A3 );
        keysymUnicode.put( "kana_u", 0x30A5 );
        keysymUnicode.put( "kana_e", 0x30A7 );
        keysymUnicode.put( "kana_o", 0x30A9 );
        keysymUnicode.put( "kana_ya", 0x30E3 );
        keysymUnicode.put( "kana_yu", 0x30E5 );
        keysymUnicode.put( "kana_yo", 0x30E7 );
        keysymUnicode.put( "kana_tsu", 0x30C3 );
        keysymUnicode.put( "prolongedsound", 0x30FC );
        keysymUnicode.put( "kana_A", 0x30A2 );
        keysymUnicode.put( "kana_I", 0x30A4 );
        keysymUnicode.put( "kana_U", 0x30A6 );
        keysymUnicode.put( "kana_E", 0x30A8 );
        keysymUnicode.put( "kana_O", 0x30AA );
        keysymUnicode.put( "kana_KA", 0x30AB );
        keysymUnicode.put( "kana_KI", 0x30AD );
        keysymUnicode.put( "kana_KU", 0x30AF );
        keysymUnicode.put( "kana_KE", 0x30B1 );
        keysymUnicode.put( "kana_KO", 0x30B3 );
        keysymUnicode.put( "kana_SA", 0x30B5 );
        keysymUnicode.put( "kana_SHI", 0x30B7 );
        keysymUnicode.put( "kana_SU", 0x30B9 );
        keysymUnicode.put( "kana_SE", 0x30BB );
        keysymUnicode.put( "kana_SO", 0x30BD );
        keysymUnicode.put( "kana_TA", 0x30BF );
        keysymUnicode.put( "kana_CHI", 0x30C1 );
        keysymUnicode.put( "kana_TSU", 0x30C4 );
        keysymUnicode.put( "kana_TE", 0x30C6 );
        keysymUnicode.put( "kana_TO", 0x30C8 );
        keysymUnicode.put( "kana_NA", 0x30CA );
        keysymUnicode.put( "kana_NI", 0x30CB );
        keysymUnicode.put( "kana_NU", 0x30CC );
        keysymUnicode.put( "kana_NE", 0x30CD );
        keysymUnicode.put( "kana_NO", 0x30CE );
        keysymUnicode.put( "kana_HA", 0x30CF );
        keysymUnicode.put( "kana_HI", 0x30D2 );
        keysymUnicode.put( "kana_FU", 0x30D5 );
        keysymUnicode.put( "kana_HE", 0x30D8 );
        keysymUnicode.put( "kana_HO", 0x30DB );
        keysymUnicode.put( "kana_MA", 0x30DE );
        keysymUnicode.put( "kana_MI", 0x30DF );
        keysymUnicode.put( "kana_MU", 0x30E0 );
        keysymUnicode.put( "kana_ME", 0x30E1 );
        keysymUnicode.put( "kana_MO", 0x30E2 );
        keysymUnicode.put( "kana_YA", 0x30E4 );
        keysymUnicode.put( "kana_YU", 0x30E6 );
        keysymUnicode.put( "kana_YO", 0x30E8 );
        keysymUnicode.put( "kana_RA", 0x30E9 );
        keysymUnicode.put( "kana_RI", 0x30EA );
        keysymUnicode.put( "kana_RU", 0x30EB );
        keysymUnicode.put( "kana_RE", 0x30EC );
        keysymUnicode.put( "kana_RO", 0x30ED );
        keysymUnicode.put( "kana_WA", 0x30EF );
        keysymUnicode.put( "kana_N", 0x30F3 );
        keysymUnicode.put( "voicedsound", 0x309B );
        keysymUnicode.put( "semivoicedsound", 0x309C );
        keysymUnicode.put( "Farsi_0", 0x06F0 );
        keysymUnicode.put( "Farsi_1", 0x06F1 );
        keysymUnicode.put( "Farsi_2", 0x06F2 );
        keysymUnicode.put( "Farsi_3", 0x06F3 );
        keysymUnicode.put( "Farsi_4", 0x06F4 );
        keysymUnicode.put( "Farsi_5", 0x06F5 );
        keysymUnicode.put( "Farsi_6", 0x06F6 );
        keysymUnicode.put( "Farsi_7", 0x06F7 );
        keysymUnicode.put( "Farsi_8", 0x06F8 );
        keysymUnicode.put( "Farsi_9", 0x06F9 );
        keysymUnicode.put( "Arabic_percent", 0x066A );
        keysymUnicode.put( "Arabic_superscript_alef", 0x0670 );
        keysymUnicode.put( "Arabic_tteh", 0x0679 );
        keysymUnicode.put( "Arabic_peh", 0x067E );
        keysymUnicode.put( "Arabic_tcheh", 0x0686 );
        keysymUnicode.put( "Arabic_ddal", 0x0688 );
        keysymUnicode.put( "Arabic_rreh", 0x0691 );
        keysymUnicode.put( "Arabic_comma", 0x060C );
        keysymUnicode.put( "Arabic_fullstop", 0x06D4 );
        keysymUnicode.put( "Arabic_0", 0x0660 );
        keysymUnicode.put( "Arabic_1", 0x0661 );
        keysymUnicode.put( "Arabic_2", 0x0662 );
        keysymUnicode.put( "Arabic_3", 0x0663 );
        keysymUnicode.put( "Arabic_4", 0x0664 );
        keysymUnicode.put( "Arabic_5", 0x0665 );
        keysymUnicode.put( "Arabic_6", 0x0666 );
        keysymUnicode.put( "Arabic_7", 0x0667 );
        keysymUnicode.put( "Arabic_8", 0x0668 );
        keysymUnicode.put( "Arabic_9", 0x0669 );
        keysymUnicode.put( "Arabic_semicolon", 0x061B );
        keysymUnicode.put( "Arabic_question_mark", 0x061F );
        keysymUnicode.put( "Arabic_hamza", 0x0621 );
        keysymUnicode.put( "Arabic_maddaonalef", 0x0622 );
        keysymUnicode.put( "Arabic_hamzaonalef", 0x0623 );
        keysymUnicode.put( "Arabic_hamzaonwaw", 0x0624 );
        keysymUnicode.put( "Arabic_hamzaunderalef", 0x0625 );
        keysymUnicode.put( "Arabic_hamzaonyeh", 0x0626 );
        keysymUnicode.put( "Arabic_alef", 0x0627 );
        keysymUnicode.put( "Arabic_beh", 0x0628 );
        keysymUnicode.put( "Arabic_tehmarbuta", 0x0629 );
        keysymUnicode.put( "Arabic_teh", 0x062A );
        keysymUnicode.put( "Arabic_theh", 0x062B );
        keysymUnicode.put( "Arabic_jeem", 0x062C );
        keysymUnicode.put( "Arabic_hah", 0x062D );
        keysymUnicode.put( "Arabic_khah", 0x062E );
        keysymUnicode.put( "Arabic_dal", 0x062F );
        keysymUnicode.put( "Arabic_thal", 0x0630 );
        keysymUnicode.put( "Arabic_ra", 0x0631 );
        keysymUnicode.put( "Arabic_zain", 0x0632 );
        keysymUnicode.put( "Arabic_seen", 0x0633 );
        keysymUnicode.put( "Arabic_sheen", 0x0634 );
        keysymUnicode.put( "Arabic_sad", 0x0635 );
        keysymUnicode.put( "Arabic_dad", 0x0636 );
        keysymUnicode.put( "Arabic_tah", 0x0637 );
        keysymUnicode.put( "Arabic_zah", 0x0638 );
        keysymUnicode.put( "Arabic_ain", 0x0639 );
        keysymUnicode.put( "Arabic_ghain", 0x063A );
        keysymUnicode.put( "Arabic_tatweel", 0x0640 );
        keysymUnicode.put( "Arabic_feh", 0x0641 );
        keysymUnicode.put( "Arabic_qaf", 0x0642 );
        keysymUnicode.put( "Arabic_kaf", 0x0643 );
        keysymUnicode.put( "Arabic_lam", 0x0644 );
        keysymUnicode.put( "Arabic_meem", 0x0645 );
        keysymUnicode.put( "Arabic_noon", 0x0646 );
        keysymUnicode.put( "Arabic_ha", 0x0647 );
        keysymUnicode.put( "Arabic_waw", 0x0648 );
        keysymUnicode.put( "Arabic_alefmaksura", 0x0649 );
        keysymUnicode.put( "Arabic_yeh", 0x064A );
        keysymUnicode.put( "Arabic_fathatan", 0x064B );
        keysymUnicode.put( "Arabic_dammatan", 0x064C );
        keysymUnicode.put( "Arabic_kasratan", 0x064D );
        keysymUnicode.put( "Arabic_fatha", 0x064E );
        keysymUnicode.put( "Arabic_damma", 0x064F );
        keysymUnicode.put( "Arabic_kasra", 0x0650 );
        keysymUnicode.put( "Arabic_shadda", 0x0651 );
        keysymUnicode.put( "Arabic_sukun", 0x0652 );
        keysymUnicode.put( "Arabic_madda_above", 0x0653 );
        keysymUnicode.put( "Arabic_hamza_above", 0x0654 );
        keysymUnicode.put( "Arabic_hamza_below", 0x0655 );
        keysymUnicode.put( "Arabic_jeh", 0x0698 );
        keysymUnicode.put( "Arabic_veh", 0x06A4 );
        keysymUnicode.put( "Arabic_keheh", 0x06A9 );
        keysymUnicode.put( "Arabic_gaf", 0x06AF );
        keysymUnicode.put( "Arabic_noon_ghunna", 0x06BA );
        keysymUnicode.put( "Arabic_heh_doachashmee", 0x06BE );
        keysymUnicode.put( "Farsi_yeh", 0x06CC );
        keysymUnicode.put( "Arabic_farsi_yeh", 0x06CC );
        keysymUnicode.put( "Arabic_yeh_baree", 0x06D2 );
        keysymUnicode.put( "Arabic_heh_goal", 0x06C1 );
        keysymUnicode.put( "Cyrillic_GHE_bar", 0x0492 );
        keysymUnicode.put( "Cyrillic_ghe_bar", 0x0493 );
        keysymUnicode.put( "Cyrillic_ZHE_descender", 0x0496 );
        keysymUnicode.put( "Cyrillic_zhe_descender", 0x0497 );
        keysymUnicode.put( "Cyrillic_KA_descender", 0x049A );
        keysymUnicode.put( "Cyrillic_ka_descender", 0x049B );
        keysymUnicode.put( "Cyrillic_KA_vertstroke", 0x049C );
        keysymUnicode.put( "Cyrillic_ka_vertstroke", 0x049D );
        keysymUnicode.put( "Cyrillic_EN_descender", 0x04A2 );
        keysymUnicode.put( "Cyrillic_en_descender", 0x04A3 );
        keysymUnicode.put( "Cyrillic_U_straight", 0x04AE );
        keysymUnicode.put( "Cyrillic_u_straight", 0x04AF );
        keysymUnicode.put( "Cyrillic_U_straight_bar", 0x04B0 );
        keysymUnicode.put( "Cyrillic_u_straight_bar", 0x04B1 );
        keysymUnicode.put( "Cyrillic_HA_descender", 0x04B2 );
        keysymUnicode.put( "Cyrillic_ha_descender", 0x04B3 );
        keysymUnicode.put( "Cyrillic_CHE_descender", 0x04B6 );
        keysymUnicode.put( "Cyrillic_che_descender", 0x04B7 );
        keysymUnicode.put( "Cyrillic_CHE_vertstroke", 0x04B8 );
        keysymUnicode.put( "Cyrillic_che_vertstroke", 0x04B9 );
        keysymUnicode.put( "Cyrillic_SHHA", 0x04BA );
        keysymUnicode.put( "Cyrillic_shha", 0x04BB );
        keysymUnicode.put( "Cyrillic_SCHWA", 0x04D8 );
        keysymUnicode.put( "Cyrillic_schwa", 0x04D9 );
        keysymUnicode.put( "Cyrillic_I_macron", 0x04E2 );
        keysymUnicode.put( "Cyrillic_i_macron", 0x04E3 );
        keysymUnicode.put( "Cyrillic_O_bar", 0x04E8 );
        keysymUnicode.put( "Cyrillic_o_bar", 0x04E9 );
        keysymUnicode.put( "Cyrillic_U_macron", 0x04EE );
        keysymUnicode.put( "Cyrillic_u_macron", 0x04EF );
        keysymUnicode.put( "Serbian_dje", 0x0452 );
        keysymUnicode.put( "Macedonia_gje", 0x0453 );
        keysymUnicode.put( "Cyrillic_io", 0x0451 );
        keysymUnicode.put( "Ukrainian_ie", 0x0454 );
        keysymUnicode.put( "Macedonia_dse", 0x0455 );
        keysymUnicode.put( "Ukrainian_i", 0x0456 );
        keysymUnicode.put( "Ukrainian_yi", 0x0457 );
        keysymUnicode.put( "Cyrillic_je", 0x0458 );
        keysymUnicode.put( "Cyrillic_lje", 0x0459 );
        keysymUnicode.put( "Cyrillic_nje", 0x045A );
        keysymUnicode.put( "Serbian_tshe", 0x045B );
        keysymUnicode.put( "Macedonia_kje", 0x045C );
        keysymUnicode.put( "Ukrainian_ghe_with_upturn", 0x0491 );
        keysymUnicode.put( "Byelorussian_shortu", 0x045E );
        keysymUnicode.put( "Cyrillic_dzhe", 0x045F );
        keysymUnicode.put( "numerosign", 0x2116 );
        keysymUnicode.put( "Serbian_DJE", 0x0402 );
        keysymUnicode.put( "Macedonia_GJE", 0x0403 );
        keysymUnicode.put( "Cyrillic_IO", 0x0401 );
        keysymUnicode.put( "Ukrainian_IE", 0x0404 );
        keysymUnicode.put( "Macedonia_DSE", 0x0405 );
        keysymUnicode.put( "Ukrainian_I", 0x0406 );
        keysymUnicode.put( "Ukrainian_YI", 0x0407 );
        keysymUnicode.put( "Cyrillic_JE", 0x0408 );
        keysymUnicode.put( "Cyrillic_LJE", 0x0409 );
        keysymUnicode.put( "Cyrillic_NJE", 0x040A );
        keysymUnicode.put( "Serbian_TSHE", 0x040B );
        keysymUnicode.put( "Macedonia_KJE", 0x040C );
        keysymUnicode.put( "Ukrainian_GHE_WITH_UPTURN", 0x0490 );
        keysymUnicode.put( "Byelorussian_SHORTU", 0x040E );
        keysymUnicode.put( "Cyrillic_DZHE", 0x040F );
        keysymUnicode.put( "Cyrillic_yu", 0x044E );
        keysymUnicode.put( "Cyrillic_a", 0x0430 );
        keysymUnicode.put( "Cyrillic_be", 0x0431 );
        keysymUnicode.put( "Cyrillic_tse", 0x0446 );
        keysymUnicode.put( "Cyrillic_de", 0x0434 );
        keysymUnicode.put( "Cyrillic_ie", 0x0435 );
        keysymUnicode.put( "Cyrillic_ef", 0x0444 );
        keysymUnicode.put( "Cyrillic_ghe", 0x0433 );
        keysymUnicode.put( "Cyrillic_ha", 0x0445 );
        keysymUnicode.put( "Cyrillic_i", 0x0438 );
        keysymUnicode.put( "Cyrillic_shorti", 0x0439 );
        keysymUnicode.put( "Cyrillic_ka", 0x043A );
        keysymUnicode.put( "Cyrillic_el", 0x043B );
        keysymUnicode.put( "Cyrillic_em", 0x043C );
        keysymUnicode.put( "Cyrillic_en", 0x043D );
        keysymUnicode.put( "Cyrillic_o", 0x043E );
        keysymUnicode.put( "Cyrillic_pe", 0x043F );
        keysymUnicode.put( "Cyrillic_ya", 0x044F );
        keysymUnicode.put( "Cyrillic_er", 0x0440 );
        keysymUnicode.put( "Cyrillic_es", 0x0441 );
        keysymUnicode.put( "Cyrillic_te", 0x0442 );
        keysymUnicode.put( "Cyrillic_u", 0x0443 );
        keysymUnicode.put( "Cyrillic_zhe", 0x0436 );
        keysymUnicode.put( "Cyrillic_ve", 0x0432 );
        keysymUnicode.put( "Cyrillic_softsign", 0x044C );
        keysymUnicode.put( "Cyrillic_yeru", 0x044B );
        keysymUnicode.put( "Cyrillic_ze", 0x0437 );
        keysymUnicode.put( "Cyrillic_sha", 0x0448 );
        keysymUnicode.put( "Cyrillic_e", 0x044D );
        keysymUnicode.put( "Cyrillic_shcha", 0x0449 );
        keysymUnicode.put( "Cyrillic_che", 0x0447 );
        keysymUnicode.put( "Cyrillic_hardsign", 0x044A );
        keysymUnicode.put( "Cyrillic_YU", 0x042E );
        keysymUnicode.put( "Cyrillic_A", 0x0410 );
        keysymUnicode.put( "Cyrillic_BE", 0x0411 );
        keysymUnicode.put( "Cyrillic_TSE", 0x0426 );
        keysymUnicode.put( "Cyrillic_DE", 0x0414 );
        keysymUnicode.put( "Cyrillic_IE", 0x0415 );
        keysymUnicode.put( "Cyrillic_EF", 0x0424 );
        keysymUnicode.put( "Cyrillic_GHE", 0x0413 );
        keysymUnicode.put( "Cyrillic_HA", 0x0425 );
        keysymUnicode.put( "Cyrillic_I", 0x0418 );
        keysymUnicode.put( "Cyrillic_SHORTI", 0x0419 );
        keysymUnicode.put( "Cyrillic_KA", 0x041A );
        keysymUnicode.put( "Cyrillic_EL", 0x041B );
        keysymUnicode.put( "Cyrillic_EM", 0x041C );
        keysymUnicode.put( "Cyrillic_EN", 0x041D );
        keysymUnicode.put( "Cyrillic_O", 0x041E );
        keysymUnicode.put( "Cyrillic_PE", 0x041F );
        keysymUnicode.put( "Cyrillic_YA", 0x042F );
        keysymUnicode.put( "Cyrillic_ER", 0x0420 );
        keysymUnicode.put( "Cyrillic_ES", 0x0421 );
        keysymUnicode.put( "Cyrillic_TE", 0x0422 );
        keysymUnicode.put( "Cyrillic_U", 0x0423 );
        keysymUnicode.put( "Cyrillic_ZHE", 0x0416 );
        keysymUnicode.put( "Cyrillic_VE", 0x0412 );
        keysymUnicode.put( "Cyrillic_SOFTSIGN", 0x042C );
        keysymUnicode.put( "Cyrillic_YERU", 0x042B );
        keysymUnicode.put( "Cyrillic_ZE", 0x0417 );
        keysymUnicode.put( "Cyrillic_SHA", 0x0428 );
        keysymUnicode.put( "Cyrillic_E", 0x042D );
        keysymUnicode.put( "Cyrillic_SHCHA", 0x0429 );
        keysymUnicode.put( "Cyrillic_CHE", 0x0427 );
        keysymUnicode.put( "Cyrillic_HARDSIGN", 0x042A );
        keysymUnicode.put( "Greek_ALPHAaccent", 0x0386 );
        keysymUnicode.put( "Greek_EPSILONaccent", 0x0388 );
        keysymUnicode.put( "Greek_ETAaccent", 0x0389 );
        keysymUnicode.put( "Greek_IOTAaccent", 0x038A );
        keysymUnicode.put( "Greek_IOTAdieresis", 0x03AA );
        keysymUnicode.put( "Greek_OMICRONaccent", 0x038C );
        keysymUnicode.put( "Greek_UPSILONaccent", 0x038E );
        keysymUnicode.put( "Greek_UPSILONdieresis", 0x03AB );
        keysymUnicode.put( "Greek_OMEGAaccent", 0x038F );
        keysymUnicode.put( "Greek_accentdieresis", 0x0385 );
        keysymUnicode.put( "Greek_horizbar", 0x2015 );
        keysymUnicode.put( "Greek_alphaaccent", 0x03AC );
        keysymUnicode.put( "Greek_epsilonaccent", 0x03AD );
        keysymUnicode.put( "Greek_etaaccent", 0x03AE );
        keysymUnicode.put( "Greek_iotaaccent", 0x03AF );
        keysymUnicode.put( "Greek_iotadieresis", 0x03CA );
        keysymUnicode.put( "Greek_iotaaccentdieresis", 0x0390 );
        keysymUnicode.put( "Greek_omicronaccent", 0x03CC );
        keysymUnicode.put( "Greek_upsilonaccent", 0x03CD );
        keysymUnicode.put( "Greek_upsilondieresis", 0x03CB );
        keysymUnicode.put( "Greek_upsilonaccentdieresis", 0x03B0 );
        keysymUnicode.put( "Greek_omegaaccent", 0x03CE );
        keysymUnicode.put( "Greek_ALPHA", 0x0391 );
        keysymUnicode.put( "Greek_BETA", 0x0392 );
        keysymUnicode.put( "Greek_GAMMA", 0x0393 );
        keysymUnicode.put( "Greek_DELTA", 0x0394 );
        keysymUnicode.put( "Greek_EPSILON", 0x0395 );
        keysymUnicode.put( "Greek_ZETA", 0x0396 );
        keysymUnicode.put( "Greek_ETA", 0x0397 );
        keysymUnicode.put( "Greek_THETA", 0x0398 );
        keysymUnicode.put( "Greek_IOTA", 0x0399 );
        keysymUnicode.put( "Greek_KAPPA", 0x039A );
        keysymUnicode.put( "Greek_LAMDA", 0x039B );
        keysymUnicode.put( "Greek_LAMBDA", 0x039B );
        keysymUnicode.put( "Greek_MU", 0x039C );
        keysymUnicode.put( "Greek_NU", 0x039D );
        keysymUnicode.put( "Greek_XI", 0x039E );
        keysymUnicode.put( "Greek_OMICRON", 0x039F );
        keysymUnicode.put( "Greek_PI", 0x03A0 );
        keysymUnicode.put( "Greek_RHO", 0x03A1 );
        keysymUnicode.put( "Greek_SIGMA", 0x03A3 );
        keysymUnicode.put( "Greek_TAU", 0x03A4 );
        keysymUnicode.put( "Greek_UPSILON", 0x03A5 );
        keysymUnicode.put( "Greek_PHI", 0x03A6 );
        keysymUnicode.put( "Greek_CHI", 0x03A7 );
        keysymUnicode.put( "Greek_PSI", 0x03A8 );
        keysymUnicode.put( "Greek_OMEGA", 0x03A9 );
        keysymUnicode.put( "Greek_alpha", 0x03B1 );
        keysymUnicode.put( "Greek_beta", 0x03B2 );
        keysymUnicode.put( "Greek_gamma", 0x03B3 );
        keysymUnicode.put( "Greek_delta", 0x03B4 );
        keysymUnicode.put( "Greek_epsilon", 0x03B5 );
        keysymUnicode.put( "Greek_zeta", 0x03B6 );
        keysymUnicode.put( "Greek_eta", 0x03B7 );
        keysymUnicode.put( "Greek_theta", 0x03B8 );
        keysymUnicode.put( "Greek_iota", 0x03B9 );
        keysymUnicode.put( "Greek_kappa", 0x03BA );
        keysymUnicode.put( "Greek_lamda", 0x03BB );
        keysymUnicode.put( "Greek_lambda", 0x03BB );
        keysymUnicode.put( "Greek_mu", 0x03BC );
        keysymUnicode.put( "Greek_nu", 0x03BD );
        keysymUnicode.put( "Greek_xi", 0x03BE );
        keysymUnicode.put( "Greek_omicron", 0x03BF );
        keysymUnicode.put( "Greek_pi", 0x03C0 );
        keysymUnicode.put( "Greek_rho", 0x03C1 );
        keysymUnicode.put( "Greek_sigma", 0x03C3 );
        keysymUnicode.put( "Greek_finalsmallsigma", 0x03C2 );
        keysymUnicode.put( "Greek_tau", 0x03C4 );
        keysymUnicode.put( "Greek_upsilon", 0x03C5 );
        keysymUnicode.put( "Greek_phi", 0x03C6 );
        keysymUnicode.put( "Greek_chi", 0x03C7 );
        keysymUnicode.put( "Greek_psi", 0x03C8 );
        keysymUnicode.put( "Greek_omega", 0x03C9 );
        keysymUnicode.put( "leftradical", 0x23B7 );
        keysymUnicode.put( "topintegral", 0x2320 );
        keysymUnicode.put( "botintegral", 0x2321 );
        keysymUnicode.put( "topleftsqbracket", 0x23A1 );
        keysymUnicode.put( "botleftsqbracket", 0x23A3 );
        keysymUnicode.put( "toprightsqbracket", 0x23A4 );
        keysymUnicode.put( "botrightsqbracket", 0x23A6 );
        keysymUnicode.put( "topleftparens", 0x239B );
        keysymUnicode.put( "botleftparens", 0x239D );
        keysymUnicode.put( "toprightparens", 0x239E );
        keysymUnicode.put( "botrightparens", 0x23A0 );
        keysymUnicode.put( "leftmiddlecurlybrace", 0x23A8 );
        keysymUnicode.put( "rightmiddlecurlybrace", 0x23AC );
        keysymUnicode.put( "lessthanequal", 0x2264 );
        keysymUnicode.put( "notequal", 0x2260 );
        keysymUnicode.put( "greaterthanequal", 0x2265 );
        keysymUnicode.put( "integral", 0x222B );
        keysymUnicode.put( "therefore", 0x2234 );
        keysymUnicode.put( "variation", 0x221D );
        keysymUnicode.put( "infinity", 0x221E );
        keysymUnicode.put( "nabla", 0x2207 );
        keysymUnicode.put( "approximate", 0x223C );
        keysymUnicode.put( "similarequal", 0x2243 );
        keysymUnicode.put( "ifonlyif", 0x21D4 );
        keysymUnicode.put( "implies", 0x21D2 );
        keysymUnicode.put( "identical", 0x2261 );
        keysymUnicode.put( "radical", 0x221A );
        keysymUnicode.put( "includedin", 0x2282 );
        keysymUnicode.put( "includes", 0x2283 );
        keysymUnicode.put( "intersection", 0x2229 );
        keysymUnicode.put( "union", 0x222A );
        keysymUnicode.put( "logicaland", 0x2227 );
        keysymUnicode.put( "logicalor", 0x2228 );
        keysymUnicode.put( "partialderivative", 0x2202 );
        keysymUnicode.put( "function", 0x0192 );
        keysymUnicode.put( "leftarrow", 0x2190 );
        keysymUnicode.put( "uparrow", 0x2191 );
        keysymUnicode.put( "rightarrow", 0x2192 );
        keysymUnicode.put( "downarrow", 0x2193 );
        keysymUnicode.put( "soliddiamond", 0x25C6 );
        keysymUnicode.put( "checkerboard", 0x2592 );
        keysymUnicode.put( "ht", 0x2409 );
        keysymUnicode.put( "ff", 0x240C );
        keysymUnicode.put( "cr", 0x240D );
        keysymUnicode.put( "lf", 0x240A );
        keysymUnicode.put( "nl", 0x2424 );
        keysymUnicode.put( "vt", 0x240B );
        keysymUnicode.put( "lowrightcorner", 0x2518 );
        keysymUnicode.put( "uprightcorner", 0x2510 );
        keysymUnicode.put( "upleftcorner", 0x250C );
        keysymUnicode.put( "lowleftcorner", 0x2514 );
        keysymUnicode.put( "crossinglines", 0x253C );
        keysymUnicode.put( "horizlinescan1", 0x23BA );
        keysymUnicode.put( "horizlinescan3", 0x23BB );
        keysymUnicode.put( "horizlinescan5", 0x2500 );
        keysymUnicode.put( "horizlinescan7", 0x23BC );
        keysymUnicode.put( "horizlinescan9", 0x23BD );
        keysymUnicode.put( "leftt", 0x251C );
        keysymUnicode.put( "rightt", 0x2524 );
        keysymUnicode.put( "bott", 0x2534 );
        keysymUnicode.put( "topt", 0x252C );
        keysymUnicode.put( "vertbar", 0x2502 );
        keysymUnicode.put( "emspace", 0x2003 );
        keysymUnicode.put( "enspace", 0x2002 );
        keysymUnicode.put( "em3space", 0x2004 );
        keysymUnicode.put( "em4space", 0x2005 );
        keysymUnicode.put( "digitspace", 0x2007 );
        keysymUnicode.put( "punctspace", 0x2008 );
        keysymUnicode.put( "thinspace", 0x2009 );
        keysymUnicode.put( "hairspace", 0x200A );
        keysymUnicode.put( "emdash", 0x2014 );
        keysymUnicode.put( "endash", 0x2013 );
        keysymUnicode.put( "ellipsis", 0x2026 );
        keysymUnicode.put( "doubbaselinedot", 0x2025 );
        keysymUnicode.put( "onethird", 0x2153 );
        keysymUnicode.put( "twothirds", 0x2154 );
        keysymUnicode.put( "onefifth", 0x2155 );
        keysymUnicode.put( "twofifths", 0x2156 );
        keysymUnicode.put( "threefifths", 0x2157 );
        keysymUnicode.put( "fourfifths", 0x2158 );
        keysymUnicode.put( "onesixth", 0x2159 );
        keysymUnicode.put( "fivesixths", 0x215A );
        keysymUnicode.put( "careof", 0x2105 );
        keysymUnicode.put( "figdash", 0x2012 );
        keysymUnicode.put( "oneeighth", 0x215B );
        keysymUnicode.put( "threeeighths", 0x215C );
        keysymUnicode.put( "fiveeighths", 0x215D );
        keysymUnicode.put( "seveneighths", 0x215E );
        keysymUnicode.put( "trademark", 0x2122 );
        keysymUnicode.put( "leftsinglequotemark", 0x2018 );
        keysymUnicode.put( "rightsinglequotemark", 0x2019 );
        keysymUnicode.put( "leftdoublequotemark", 0x201C );
        keysymUnicode.put( "rightdoublequotemark", 0x201D );
        keysymUnicode.put( "prescription", 0x211E );
        keysymUnicode.put( "permille", 0x2030 );
        keysymUnicode.put( "minutes", 0x2032 );
        keysymUnicode.put( "seconds", 0x2033 );
        keysymUnicode.put( "latincross", 0x271D );
        keysymUnicode.put( "club", 0x2663 );
        keysymUnicode.put( "diamond", 0x2666 );
        keysymUnicode.put( "heart", 0x2665 );
        keysymUnicode.put( "maltesecross", 0x2720 );
        keysymUnicode.put( "dagger", 0x2020 );
        keysymUnicode.put( "doubledagger", 0x2021 );
        keysymUnicode.put( "checkmark", 0x2713 );
        keysymUnicode.put( "ballotcross", 0x2717 );
        keysymUnicode.put( "musicalsharp", 0x266F );
        keysymUnicode.put( "musicalflat", 0x266D );
        keysymUnicode.put( "malesymbol", 0x2642 );
        keysymUnicode.put( "femalesymbol", 0x2640 );
        keysymUnicode.put( "telephone", 0x260E );
        keysymUnicode.put( "telephonerecorder", 0x2315 );
        keysymUnicode.put( "phonographcopyright", 0x2117 );
        keysymUnicode.put( "caret", 0x2038 );
        keysymUnicode.put( "singlelowquotemark", 0x201A );
        keysymUnicode.put( "doublelowquotemark", 0x201E );
        keysymUnicode.put( "downtack", 0x22A4 );
        keysymUnicode.put( "downstile", 0x230A );
        keysymUnicode.put( "jot", 0x2218 );
        keysymUnicode.put( "quad", 0x2395 );
        keysymUnicode.put( "uptack", 0x22A5 );
        keysymUnicode.put( "circle", 0x25CB );
        keysymUnicode.put( "upstile", 0x2308 );
        keysymUnicode.put( "lefttack", 0x22A3 );
        keysymUnicode.put( "righttack", 0x22A2 );
        keysymUnicode.put( "hebrew_doublelowline", 0x2017 );
        keysymUnicode.put( "hebrew_aleph", 0x05D0 );
        keysymUnicode.put( "hebrew_bet", 0x05D1 );
        keysymUnicode.put( "hebrew_gimel", 0x05D2 );
        keysymUnicode.put( "hebrew_dalet", 0x05D3 );
        keysymUnicode.put( "hebrew_he", 0x05D4 );
        keysymUnicode.put( "hebrew_waw", 0x05D5 );
        keysymUnicode.put( "hebrew_zain", 0x05D6 );
        keysymUnicode.put( "hebrew_chet", 0x05D7 );
        keysymUnicode.put( "hebrew_tet", 0x05D8 );
        keysymUnicode.put( "hebrew_yod", 0x05D9 );
        keysymUnicode.put( "hebrew_finalkaph", 0x05DA );
        keysymUnicode.put( "hebrew_kaph", 0x05DB );
        keysymUnicode.put( "hebrew_lamed", 0x05DC );
        keysymUnicode.put( "hebrew_finalmem", 0x05DD );
        keysymUnicode.put( "hebrew_mem", 0x05DE );
        keysymUnicode.put( "hebrew_finalnun", 0x05DF );
        keysymUnicode.put( "hebrew_nun", 0x05E0 );
        keysymUnicode.put( "hebrew_samech", 0x05E1 );
        keysymUnicode.put( "hebrew_ayin", 0x05E2 );
        keysymUnicode.put( "hebrew_finalpe", 0x05E3 );
        keysymUnicode.put( "hebrew_pe", 0x05E4 );
        keysymUnicode.put( "hebrew_finalzade", 0x05E5 );
        keysymUnicode.put( "hebrew_zade", 0x05E6 );
        keysymUnicode.put( "hebrew_qoph", 0x05E7 );
        keysymUnicode.put( "hebrew_resh", 0x05E8 );
        keysymUnicode.put( "hebrew_shin", 0x05E9 );
        keysymUnicode.put( "hebrew_taw", 0x05EA );
        keysymUnicode.put( "Thai_kokai", 0x0E01 );
        keysymUnicode.put( "Thai_khokhai", 0x0E02 );
        keysymUnicode.put( "Thai_khokhuat", 0x0E03 );
        keysymUnicode.put( "Thai_khokhwai", 0x0E04 );
        keysymUnicode.put( "Thai_khokhon", 0x0E05 );
        keysymUnicode.put( "Thai_khorakhang", 0x0E06 );
        keysymUnicode.put( "Thai_ngongu", 0x0E07 );
        keysymUnicode.put( "Thai_chochan", 0x0E08 );
        keysymUnicode.put( "Thai_choching", 0x0E09 );
        keysymUnicode.put( "Thai_chochang", 0x0E0A );
        keysymUnicode.put( "Thai_soso", 0x0E0B );
        keysymUnicode.put( "Thai_chochoe", 0x0E0C );
        keysymUnicode.put( "Thai_yoying", 0x0E0D );
        keysymUnicode.put( "Thai_dochada", 0x0E0E );
        keysymUnicode.put( "Thai_topatak", 0x0E0F );
        keysymUnicode.put( "Thai_thothan", 0x0E10 );
        keysymUnicode.put( "Thai_thonangmontho", 0x0E11 );
        keysymUnicode.put( "Thai_thophuthao", 0x0E12 );
        keysymUnicode.put( "Thai_nonen", 0x0E13 );
        keysymUnicode.put( "Thai_dodek", 0x0E14 );
        keysymUnicode.put( "Thai_totao", 0x0E15 );
        keysymUnicode.put( "Thai_thothung", 0x0E16 );
        keysymUnicode.put( "Thai_thothahan", 0x0E17 );
        keysymUnicode.put( "Thai_thothong", 0x0E18 );
        keysymUnicode.put( "Thai_nonu", 0x0E19 );
        keysymUnicode.put( "Thai_bobaimai", 0x0E1A );
        keysymUnicode.put( "Thai_popla", 0x0E1B );
        keysymUnicode.put( "Thai_phophung", 0x0E1C );
        keysymUnicode.put( "Thai_fofa", 0x0E1D );
        keysymUnicode.put( "Thai_phophan", 0x0E1E );
        keysymUnicode.put( "Thai_fofan", 0x0E1F );
        keysymUnicode.put( "Thai_phosamphao", 0x0E20 );
        keysymUnicode.put( "Thai_moma", 0x0E21 );
        keysymUnicode.put( "Thai_yoyak", 0x0E22 );
        keysymUnicode.put( "Thai_rorua", 0x0E23 );
        keysymUnicode.put( "Thai_ru", 0x0E24 );
        keysymUnicode.put( "Thai_loling", 0x0E25 );
        keysymUnicode.put( "Thai_lu", 0x0E26 );
        keysymUnicode.put( "Thai_wowaen", 0x0E27 );
        keysymUnicode.put( "Thai_sosala", 0x0E28 );
        keysymUnicode.put( "Thai_sorusi", 0x0E29 );
        keysymUnicode.put( "Thai_sosua", 0x0E2A );
        keysymUnicode.put( "Thai_hohip", 0x0E2B );
        keysymUnicode.put( "Thai_lochula", 0x0E2C );
        keysymUnicode.put( "Thai_oang", 0x0E2D );
        keysymUnicode.put( "Thai_honokhuk", 0x0E2E );
        keysymUnicode.put( "Thai_paiyannoi", 0x0E2F );
        keysymUnicode.put( "Thai_saraa", 0x0E30 );
        keysymUnicode.put( "Thai_maihanakat", 0x0E31 );
        keysymUnicode.put( "Thai_saraaa", 0x0E32 );
        keysymUnicode.put( "Thai_saraam", 0x0E33 );
        keysymUnicode.put( "Thai_sarai", 0x0E34 );
        keysymUnicode.put( "Thai_saraii", 0x0E35 );
        keysymUnicode.put( "Thai_saraue", 0x0E36 );
        keysymUnicode.put( "Thai_sarauee", 0x0E37 );
        keysymUnicode.put( "Thai_sarau", 0x0E38 );
        keysymUnicode.put( "Thai_sarauu", 0x0E39 );
        keysymUnicode.put( "Thai_phinthu", 0x0E3A );
        keysymUnicode.put( "Thai_baht", 0x0E3F );
        keysymUnicode.put( "Thai_sarae", 0x0E40 );
        keysymUnicode.put( "Thai_saraae", 0x0E41 );
        keysymUnicode.put( "Thai_sarao", 0x0E42 );
        keysymUnicode.put( "Thai_saraaimaimuan", 0x0E43 );
        keysymUnicode.put( "Thai_saraaimaimalai", 0x0E44 );
        keysymUnicode.put( "Thai_lakkhangyao", 0x0E45 );
        keysymUnicode.put( "Thai_maiyamok", 0x0E46 );
        keysymUnicode.put( "Thai_maitaikhu", 0x0E47 );
        keysymUnicode.put( "Thai_maiek", 0x0E48 );
        keysymUnicode.put( "Thai_maitho", 0x0E49 );
        keysymUnicode.put( "Thai_maitri", 0x0E4A );
        keysymUnicode.put( "Thai_maichattawa", 0x0E4B );
        keysymUnicode.put( "Thai_thanthakhat", 0x0E4C );
        keysymUnicode.put( "Thai_nikhahit", 0x0E4D );
        keysymUnicode.put( "Thai_leksun", 0x0E50 );
        keysymUnicode.put( "Thai_leknung", 0x0E51 );
        keysymUnicode.put( "Thai_leksong", 0x0E52 );
        keysymUnicode.put( "Thai_leksam", 0x0E53 );
        keysymUnicode.put( "Thai_leksi", 0x0E54 );
        keysymUnicode.put( "Thai_lekha", 0x0E55 );
        keysymUnicode.put( "Thai_lekhok", 0x0E56 );
        keysymUnicode.put( "Thai_lekchet", 0x0E57 );
        keysymUnicode.put( "Thai_lekpaet", 0x0E58 );
        keysymUnicode.put( "Thai_lekkao", 0x0E59 );
        keysymUnicode.put( "Armenian_ligature_ew", 0x0587 );
        keysymUnicode.put( "Armenian_full_stop", 0x0589 );
        keysymUnicode.put( "Armenian_verjaket", 0x0589 );
        keysymUnicode.put( "Armenian_separation_mark", 0x055D );
        keysymUnicode.put( "Armenian_but", 0x055D );
        keysymUnicode.put( "Armenian_hyphen", 0x058A );
        keysymUnicode.put( "Armenian_yentamna", 0x058A );
        keysymUnicode.put( "Armenian_exclam", 0x055C );
        keysymUnicode.put( "Armenian_amanak", 0x055C );
        keysymUnicode.put( "Armenian_accent", 0x055B );
        keysymUnicode.put( "Armenian_shesht", 0x055B );
        keysymUnicode.put( "Armenian_question", 0x055E );
        keysymUnicode.put( "Armenian_paruyk", 0x055E );
        keysymUnicode.put( "Armenian_AYB", 0x0531 );
        keysymUnicode.put( "Armenian_ayb", 0x0561 );
        keysymUnicode.put( "Armenian_BEN", 0x0532 );
        keysymUnicode.put( "Armenian_ben", 0x0562 );
        keysymUnicode.put( "Armenian_GIM", 0x0533 );
        keysymUnicode.put( "Armenian_gim", 0x0563 );
        keysymUnicode.put( "Armenian_DA", 0x0534 );
        keysymUnicode.put( "Armenian_da", 0x0564 );
        keysymUnicode.put( "Armenian_YECH", 0x0535 );
        keysymUnicode.put( "Armenian_yech", 0x0565 );
        keysymUnicode.put( "Armenian_ZA", 0x0536 );
        keysymUnicode.put( "Armenian_za", 0x0566 );
        keysymUnicode.put( "Armenian_E", 0x0537 );
        keysymUnicode.put( "Armenian_e", 0x0567 );
        keysymUnicode.put( "Armenian_AT", 0x0538 );
        keysymUnicode.put( "Armenian_at", 0x0568 );
        keysymUnicode.put( "Armenian_TO", 0x0539 );
        keysymUnicode.put( "Armenian_to", 0x0569 );
        keysymUnicode.put( "Armenian_ZHE", 0x053A );
        keysymUnicode.put( "Armenian_zhe", 0x056A );
        keysymUnicode.put( "Armenian_INI", 0x053B );
        keysymUnicode.put( "Armenian_ini", 0x056B );
        keysymUnicode.put( "Armenian_LYUN", 0x053C );
        keysymUnicode.put( "Armenian_lyun", 0x056C );
        keysymUnicode.put( "Armenian_KHE", 0x053D );
        keysymUnicode.put( "Armenian_khe", 0x056D );
        keysymUnicode.put( "Armenian_TSA", 0x053E );
        keysymUnicode.put( "Armenian_tsa", 0x056E );
        keysymUnicode.put( "Armenian_KEN", 0x053F );
        keysymUnicode.put( "Armenian_ken", 0x056F );
        keysymUnicode.put( "Armenian_HO", 0x0540 );
        keysymUnicode.put( "Armenian_ho", 0x0570 );
        keysymUnicode.put( "Armenian_DZA", 0x0541 );
        keysymUnicode.put( "Armenian_dza", 0x0571 );
        keysymUnicode.put( "Armenian_GHAT", 0x0542 );
        keysymUnicode.put( "Armenian_ghat", 0x0572 );
        keysymUnicode.put( "Armenian_TCHE", 0x0543 );
        keysymUnicode.put( "Armenian_tche", 0x0573 );
        keysymUnicode.put( "Armenian_MEN", 0x0544 );
        keysymUnicode.put( "Armenian_men", 0x0574 );
        keysymUnicode.put( "Armenian_HI", 0x0545 );
        keysymUnicode.put( "Armenian_hi", 0x0575 );
        keysymUnicode.put( "Armenian_NU", 0x0546 );
        keysymUnicode.put( "Armenian_nu", 0x0576 );
        keysymUnicode.put( "Armenian_SHA", 0x0547 );
        keysymUnicode.put( "Armenian_sha", 0x0577 );
        keysymUnicode.put( "Armenian_VO", 0x0548 );
        keysymUnicode.put( "Armenian_vo", 0x0578 );
        keysymUnicode.put( "Armenian_CHA", 0x0549 );
        keysymUnicode.put( "Armenian_cha", 0x0579 );
        keysymUnicode.put( "Armenian_PE", 0x054A );
        keysymUnicode.put( "Armenian_pe", 0x057A );
        keysymUnicode.put( "Armenian_JE", 0x054B );
        keysymUnicode.put( "Armenian_je", 0x057B );
        keysymUnicode.put( "Armenian_RA", 0x054C );
        keysymUnicode.put( "Armenian_ra", 0x057C );
        keysymUnicode.put( "Armenian_SE", 0x054D );
        keysymUnicode.put( "Armenian_se", 0x057D );
        keysymUnicode.put( "Armenian_VEV", 0x054E );
        keysymUnicode.put( "Armenian_vev", 0x057E );
        keysymUnicode.put( "Armenian_TYUN", 0x054F );
        keysymUnicode.put( "Armenian_tyun", 0x057F );
        keysymUnicode.put( "Armenian_RE", 0x0550 );
        keysymUnicode.put( "Armenian_re", 0x0580 );
        keysymUnicode.put( "Armenian_TSO", 0x0551 );
        keysymUnicode.put( "Armenian_tso", 0x0581 );
        keysymUnicode.put( "Armenian_VYUN", 0x0552 );
        keysymUnicode.put( "Armenian_vyun", 0x0582 );
        keysymUnicode.put( "Armenian_PYUR", 0x0553 );
        keysymUnicode.put( "Armenian_pyur", 0x0583 );
        keysymUnicode.put( "Armenian_KE", 0x0554 );
        keysymUnicode.put( "Armenian_ke", 0x0584 );
        keysymUnicode.put( "Armenian_O", 0x0555 );
        keysymUnicode.put( "Armenian_o", 0x0585 );
        keysymUnicode.put( "Armenian_FE", 0x0556 );
        keysymUnicode.put( "Armenian_fe", 0x0586 );
        keysymUnicode.put( "Armenian_apostrophe", 0x055A );
        keysymUnicode.put( "Georgian_an", 0x10D0 );
        keysymUnicode.put( "Georgian_ban", 0x10D1 );
        keysymUnicode.put( "Georgian_gan", 0x10D2 );
        keysymUnicode.put( "Georgian_don", 0x10D3 );
        keysymUnicode.put( "Georgian_en", 0x10D4 );
        keysymUnicode.put( "Georgian_vin", 0x10D5 );
        keysymUnicode.put( "Georgian_zen", 0x10D6 );
        keysymUnicode.put( "Georgian_tan", 0x10D7 );
        keysymUnicode.put( "Georgian_in", 0x10D8 );
        keysymUnicode.put( "Georgian_kan", 0x10D9 );
        keysymUnicode.put( "Georgian_las", 0x10DA );
        keysymUnicode.put( "Georgian_man", 0x10DB );
        keysymUnicode.put( "Georgian_nar", 0x10DC );
        keysymUnicode.put( "Georgian_on", 0x10DD );
        keysymUnicode.put( "Georgian_par", 0x10DE );
        keysymUnicode.put( "Georgian_zhar", 0x10DF );
        keysymUnicode.put( "Georgian_rae", 0x10E0 );
        keysymUnicode.put( "Georgian_san", 0x10E1 );
        keysymUnicode.put( "Georgian_tar", 0x10E2 );
        keysymUnicode.put( "Georgian_un", 0x10E3 );
        keysymUnicode.put( "Georgian_phar", 0x10E4 );
        keysymUnicode.put( "Georgian_khar", 0x10E5 );
        keysymUnicode.put( "Georgian_ghan", 0x10E6 );
        keysymUnicode.put( "Georgian_qar", 0x10E7 );
        keysymUnicode.put( "Georgian_shin", 0x10E8 );
        keysymUnicode.put( "Georgian_chin", 0x10E9 );
        keysymUnicode.put( "Georgian_can", 0x10EA );
        keysymUnicode.put( "Georgian_jil", 0x10EB );
        keysymUnicode.put( "Georgian_cil", 0x10EC );
        keysymUnicode.put( "Georgian_char", 0x10ED );
        keysymUnicode.put( "Georgian_xan", 0x10EE );
        keysymUnicode.put( "Georgian_jhan", 0x10EF );
        keysymUnicode.put( "Georgian_hae", 0x10F0 );
        keysymUnicode.put( "Georgian_he", 0x10F1 );
        keysymUnicode.put( "Georgian_hie", 0x10F2 );
        keysymUnicode.put( "Georgian_we", 0x10F3 );
        keysymUnicode.put( "Georgian_har", 0x10F4 );
        keysymUnicode.put( "Georgian_hoe", 0x10F5 );
        keysymUnicode.put( "Georgian_fi", 0x10F6 );
        keysymUnicode.put( "Xabovedot", 0x1E8A );
        keysymUnicode.put( "Ibreve", 0x012C );
        keysymUnicode.put( "Zstroke", 0x01B5 );
        keysymUnicode.put( "Gcaron", 0x01E6 );
        keysymUnicode.put( "Ocaron", 0x01D2 );
        keysymUnicode.put( "Obarred", 0x019F );
        keysymUnicode.put( "xabovedot", 0x1E8B );
        keysymUnicode.put( "ibreve", 0x012D );
        keysymUnicode.put( "zstroke", 0x01B6 );
        keysymUnicode.put( "gcaron", 0x01E7 );
        keysymUnicode.put( "ocaron", 0x01D2 );
        keysymUnicode.put( "obarred", 0x0275 );
        keysymUnicode.put( "SCHWA", 0x018F );
        keysymUnicode.put( "schwa", 0x0259 );
        keysymUnicode.put( "EZH", 0x01B7 );
        keysymUnicode.put( "ezh", 0x0292 );
        keysymUnicode.put( "Lbelowdot", 0x1E36 );
        keysymUnicode.put( "lbelowdot", 0x1E37 );
        keysymUnicode.put( "Abelowdot", 0x1EA0 );
        keysymUnicode.put( "abelowdot", 0x1EA1 );
        keysymUnicode.put( "Ahook", 0x1EA2 );
        keysymUnicode.put( "ahook", 0x1EA3 );
        keysymUnicode.put( "Acircumflexacute", 0x1EA4 );
        keysymUnicode.put( "acircumflexacute", 0x1EA5 );
        keysymUnicode.put( "Acircumflexgrave", 0x1EA6 );
        keysymUnicode.put( "acircumflexgrave", 0x1EA7 );
        keysymUnicode.put( "Acircumflexhook", 0x1EA8 );
        keysymUnicode.put( "acircumflexhook", 0x1EA9 );
        keysymUnicode.put( "Acircumflextilde", 0x1EAA );
        keysymUnicode.put( "acircumflextilde", 0x1EAB );
        keysymUnicode.put( "Acircumflexbelowdot", 0x1EAC );
        keysymUnicode.put( "acircumflexbelowdot", 0x1EAD );
        keysymUnicode.put( "Abreveacute", 0x1EAE );
        keysymUnicode.put( "abreveacute", 0x1EAF );
        keysymUnicode.put( "Abrevegrave", 0x1EB0 );
        keysymUnicode.put( "abrevegrave", 0x1EB1 );
        keysymUnicode.put( "Abrevehook", 0x1EB2 );
        keysymUnicode.put( "abrevehook", 0x1EB3 );
        keysymUnicode.put( "Abrevetilde", 0x1EB4 );
        keysymUnicode.put( "abrevetilde", 0x1EB5 );
        keysymUnicode.put( "Abrevebelowdot", 0x1EB6 );
        keysymUnicode.put( "abrevebelowdot", 0x1EB7 );
        keysymUnicode.put( "Ebelowdot", 0x1EB8 );
        keysymUnicode.put( "ebelowdot", 0x1EB9 );
        keysymUnicode.put( "Ehook", 0x1EBA );
        keysymUnicode.put( "ehook", 0x1EBB );
        keysymUnicode.put( "Etilde", 0x1EBC );
        keysymUnicode.put( "etilde", 0x1EBD );
        keysymUnicode.put( "Ecircumflexacute", 0x1EBE );
        keysymUnicode.put( "ecircumflexacute", 0x1EBF );
        keysymUnicode.put( "Ecircumflexgrave", 0x1EC0 );
        keysymUnicode.put( "ecircumflexgrave", 0x1EC1 );
        keysymUnicode.put( "Ecircumflexhook", 0x1EC2 );
        keysymUnicode.put( "ecircumflexhook", 0x1EC3 );
        keysymUnicode.put( "Ecircumflextilde", 0x1EC4 );
        keysymUnicode.put( "ecircumflextilde", 0x1EC5 );
        keysymUnicode.put( "Ecircumflexbelowdot", 0x1EC6 );
        keysymUnicode.put( "ecircumflexbelowdot", 0x1EC7 );
        keysymUnicode.put( "Ihook", 0x1EC8 );
        keysymUnicode.put( "ihook", 0x1EC9 );
        keysymUnicode.put( "Ibelowdot", 0x1ECA );
        keysymUnicode.put( "ibelowdot", 0x1ECB );
        keysymUnicode.put( "Obelowdot", 0x1ECC );
        keysymUnicode.put( "obelowdot", 0x1ECD );
        keysymUnicode.put( "Ohook", 0x1ECE );
        keysymUnicode.put( "ohook", 0x1ECF );
        keysymUnicode.put( "Ocircumflexacute", 0x1ED0 );
        keysymUnicode.put( "ocircumflexacute", 0x1ED1 );
        keysymUnicode.put( "Ocircumflexgrave", 0x1ED2 );
        keysymUnicode.put( "ocircumflexgrave", 0x1ED3 );
        keysymUnicode.put( "Ocircumflexhook", 0x1ED4 );
        keysymUnicode.put( "ocircumflexhook", 0x1ED5 );
        keysymUnicode.put( "Ocircumflextilde", 0x1ED6 );
        keysymUnicode.put( "ocircumflextilde", 0x1ED7 );
        keysymUnicode.put( "Ocircumflexbelowdot", 0x1ED8 );
        keysymUnicode.put( "ocircumflexbelowdot", 0x1ED9 );
        keysymUnicode.put( "Ohornacute", 0x1EDA );
        keysymUnicode.put( "ohornacute", 0x1EDB );
        keysymUnicode.put( "Ohorngrave", 0x1EDC );
        keysymUnicode.put( "ohorngrave", 0x1EDD );
        keysymUnicode.put( "Ohornhook", 0x1EDE );
        keysymUnicode.put( "ohornhook", 0x1EDF );
        keysymUnicode.put( "Ohorntilde", 0x1EE0 );
        keysymUnicode.put( "ohorntilde", 0x1EE1 );
        keysymUnicode.put( "Ohornbelowdot", 0x1EE2 );
        keysymUnicode.put( "ohornbelowdot", 0x1EE3 );
        keysymUnicode.put( "Ubelowdot", 0x1EE4 );
        keysymUnicode.put( "ubelowdot", 0x1EE5 );
        keysymUnicode.put( "Uhook", 0x1EE6 );
        keysymUnicode.put( "uhook", 0x1EE7 );
        keysymUnicode.put( "Uhornacute", 0x1EE8 );
        keysymUnicode.put( "uhornacute", 0x1EE9 );
        keysymUnicode.put( "Uhorngrave", 0x1EEA );
        keysymUnicode.put( "uhorngrave", 0x1EEB );
        keysymUnicode.put( "Uhornhook", 0x1EEC );
        keysymUnicode.put( "uhornhook", 0x1EED );
        keysymUnicode.put( "Uhorntilde", 0x1EEE );
        keysymUnicode.put( "uhorntilde", 0x1EEF );
        keysymUnicode.put( "Uhornbelowdot", 0x1EF0 );
        keysymUnicode.put( "uhornbelowdot", 0x1EF1 );
        keysymUnicode.put( "Ybelowdot", 0x1EF4 );
        keysymUnicode.put( "ybelowdot", 0x1EF5 );
        keysymUnicode.put( "Yhook", 0x1EF6 );
        keysymUnicode.put( "yhook", 0x1EF7 );
        keysymUnicode.put( "Ytilde", 0x1EF8 );
        keysymUnicode.put( "ytilde", 0x1EF9 );
        keysymUnicode.put( "Ohorn", 0x01A0 );
        keysymUnicode.put( "ohorn", 0x01A1 );
        keysymUnicode.put( "Uhorn", 0x01AF );
        keysymUnicode.put( "uhorn", 0x01B0 );
        keysymUnicode.put( "EcuSign", 0x20A0 );
        keysymUnicode.put( "ColonSign", 0x20A1 );
        keysymUnicode.put( "CruzeiroSign", 0x20A2 );
        keysymUnicode.put( "FFrancSign", 0x20A3 );
        keysymUnicode.put( "LiraSign", 0x20A4 );
        keysymUnicode.put( "MillSign", 0x20A5 );
        keysymUnicode.put( "NairaSign", 0x20A6 );
        keysymUnicode.put( "PesetaSign", 0x20A7 );
        keysymUnicode.put( "RupeeSign", 0x20A8 );
        keysymUnicode.put( "WonSign", 0x20A9 );
        keysymUnicode.put( "NewSheqelSign", 0x20AA );
        keysymUnicode.put( "DongSign", 0x20AB );
        keysymUnicode.put( "EuroSign", 0x20AC );
        keysymUnicode.put( "zerosuperior", 0x2070 );
        keysymUnicode.put( "foursuperior", 0x2074 );
        keysymUnicode.put( "fivesuperior", 0x2075 );
        keysymUnicode.put( "sixsuperior", 0x2076 );
        keysymUnicode.put( "sevensuperior", 0x2077 );
        keysymUnicode.put( "eightsuperior", 0x2078 );
        keysymUnicode.put( "ninesuperior", 0x2079 );
        keysymUnicode.put( "zerosubscript", 0x2080 );
        keysymUnicode.put( "onesubscript", 0x2081 );
        keysymUnicode.put( "twosubscript", 0x2082 );
        keysymUnicode.put( "threesubscript", 0x2083 );
        keysymUnicode.put( "foursubscript", 0x2084 );
        keysymUnicode.put( "fivesubscript", 0x2085 );
        keysymUnicode.put( "sixsubscript", 0x2086 );
        keysymUnicode.put( "sevensubscript", 0x2087 );
        keysymUnicode.put( "eightsubscript", 0x2088 );
        keysymUnicode.put( "ninesubscript", 0x2089 );
        keysymUnicode.put( "partdifferential", 0x2202 );
        keysymUnicode.put( "emptyset", 0x2205 );
        keysymUnicode.put( "elementof", 0x2208 );
        keysymUnicode.put( "notelementof", 0x2209 );
        keysymUnicode.put( "containsas", 0x220B );
        keysymUnicode.put( "squareroot", 0x221A );
        keysymUnicode.put( "cuberoot", 0x221B );
        keysymUnicode.put( "fourthroot", 0x221C );
        keysymUnicode.put( "dintegral", 0x222C );
        keysymUnicode.put( "tintegral", 0x222D );
        keysymUnicode.put( "because", 0x2235 );
        keysymUnicode.put( "approxeq", 0x2245 );
        keysymUnicode.put( "notapproxeq", 0x2247 );
        keysymUnicode.put( "notidentical", 0x2262 );
        keysymUnicode.put( "stricteq", 0x2263 );
        keysymUnicode.put( "braille_blank", 0x2800 );
        keysymUnicode.put( "braille_dots_1", 0x2801 );
        keysymUnicode.put( "braille_dots_2", 0x2802 );
        keysymUnicode.put( "braille_dots_12", 0x2803 );
        keysymUnicode.put( "braille_dots_3", 0x2804 );
        keysymUnicode.put( "braille_dots_13", 0x2805 );
        keysymUnicode.put( "braille_dots_23", 0x2806 );
        keysymUnicode.put( "braille_dots_123", 0x2807 );
        keysymUnicode.put( "braille_dots_4", 0x2808 );
        keysymUnicode.put( "braille_dots_14", 0x2809 );
        keysymUnicode.put( "braille_dots_24", 0x280a );
        keysymUnicode.put( "braille_dots_124", 0x280b );
        keysymUnicode.put( "braille_dots_34", 0x280c );
        keysymUnicode.put( "braille_dots_134", 0x280d );
        keysymUnicode.put( "braille_dots_234", 0x280e );
        keysymUnicode.put( "braille_dots_1234", 0x280f );
        keysymUnicode.put( "braille_dots_5", 0x2810 );
        keysymUnicode.put( "braille_dots_15", 0x2811 );
        keysymUnicode.put( "braille_dots_25", 0x2812 );
        keysymUnicode.put( "braille_dots_125", 0x2813 );
        keysymUnicode.put( "braille_dots_35", 0x2814 );
        keysymUnicode.put( "braille_dots_135", 0x2815 );
        keysymUnicode.put( "braille_dots_235", 0x2816 );
        keysymUnicode.put( "braille_dots_1235", 0x2817 );
        keysymUnicode.put( "braille_dots_45", 0x2818 );
        keysymUnicode.put( "braille_dots_145", 0x2819 );
        keysymUnicode.put( "braille_dots_245", 0x281a );
        keysymUnicode.put( "braille_dots_1245", 0x281b );
        keysymUnicode.put( "braille_dots_345", 0x281c );
        keysymUnicode.put( "braille_dots_1345", 0x281d );
        keysymUnicode.put( "braille_dots_2345", 0x281e );
        keysymUnicode.put( "braille_dots_12345", 0x281f );
        keysymUnicode.put( "braille_dots_6", 0x2820 );
        keysymUnicode.put( "braille_dots_16", 0x2821 );
        keysymUnicode.put( "braille_dots_26", 0x2822 );
        keysymUnicode.put( "braille_dots_126", 0x2823 );
        keysymUnicode.put( "braille_dots_36", 0x2824 );
        keysymUnicode.put( "braille_dots_136", 0x2825 );
        keysymUnicode.put( "braille_dots_236", 0x2826 );
        keysymUnicode.put( "braille_dots_1236", 0x2827 );
        keysymUnicode.put( "braille_dots_46", 0x2828 );
        keysymUnicode.put( "braille_dots_146", 0x2829 );
        keysymUnicode.put( "braille_dots_246", 0x282a );
        keysymUnicode.put( "braille_dots_1246", 0x282b );
        keysymUnicode.put( "braille_dots_346", 0x282c );
        keysymUnicode.put( "braille_dots_1346", 0x282d );
        keysymUnicode.put( "braille_dots_2346", 0x282e );
        keysymUnicode.put( "braille_dots_12346", 0x282f );
        keysymUnicode.put( "braille_dots_56", 0x2830 );
        keysymUnicode.put( "braille_dots_156", 0x2831 );
        keysymUnicode.put( "braille_dots_256", 0x2832 );
        keysymUnicode.put( "braille_dots_1256", 0x2833 );
        keysymUnicode.put( "braille_dots_356", 0x2834 );
        keysymUnicode.put( "braille_dots_1356", 0x2835 );
        keysymUnicode.put( "braille_dots_2356", 0x2836 );
        keysymUnicode.put( "braille_dots_12356", 0x2837 );
        keysymUnicode.put( "braille_dots_456", 0x2838 );
        keysymUnicode.put( "braille_dots_1456", 0x2839 );
        keysymUnicode.put( "braille_dots_2456", 0x283a );
        keysymUnicode.put( "braille_dots_12456", 0x283b );
        keysymUnicode.put( "braille_dots_3456", 0x283c );
        keysymUnicode.put( "braille_dots_13456", 0x283d );
        keysymUnicode.put( "braille_dots_23456", 0x283e );
        keysymUnicode.put( "braille_dots_123456", 0x283f );
        keysymUnicode.put( "braille_dots_7", 0x2840 );
        keysymUnicode.put( "braille_dots_17", 0x2841 );
        keysymUnicode.put( "braille_dots_27", 0x2842 );
        keysymUnicode.put( "braille_dots_127", 0x2843 );
        keysymUnicode.put( "braille_dots_37", 0x2844 );
        keysymUnicode.put( "braille_dots_137", 0x2845 );
        keysymUnicode.put( "braille_dots_237", 0x2846 );
        keysymUnicode.put( "braille_dots_1237", 0x2847 );
        keysymUnicode.put( "braille_dots_47", 0x2848 );
        keysymUnicode.put( "braille_dots_147", 0x2849 );
        keysymUnicode.put( "braille_dots_247", 0x284a );
        keysymUnicode.put( "braille_dots_1247", 0x284b );
        keysymUnicode.put( "braille_dots_347", 0x284c );
        keysymUnicode.put( "braille_dots_1347", 0x284d );
        keysymUnicode.put( "braille_dots_2347", 0x284e );
        keysymUnicode.put( "braille_dots_12347", 0x284f );
        keysymUnicode.put( "braille_dots_57", 0x2850 );
        keysymUnicode.put( "braille_dots_157", 0x2851 );
        keysymUnicode.put( "braille_dots_257", 0x2852 );
        keysymUnicode.put( "braille_dots_1257", 0x2853 );
        keysymUnicode.put( "braille_dots_357", 0x2854 );
        keysymUnicode.put( "braille_dots_1357", 0x2855 );
        keysymUnicode.put( "braille_dots_2357", 0x2856 );
        keysymUnicode.put( "braille_dots_12357", 0x2857 );
        keysymUnicode.put( "braille_dots_457", 0x2858 );
        keysymUnicode.put( "braille_dots_1457", 0x2859 );
        keysymUnicode.put( "braille_dots_2457", 0x285a );
        keysymUnicode.put( "braille_dots_12457", 0x285b );
        keysymUnicode.put( "braille_dots_3457", 0x285c );
        keysymUnicode.put( "braille_dots_13457", 0x285d );
        keysymUnicode.put( "braille_dots_23457", 0x285e );
        keysymUnicode.put( "braille_dots_123457", 0x285f );
        keysymUnicode.put( "braille_dots_67", 0x2860 );
        keysymUnicode.put( "braille_dots_167", 0x2861 );
        keysymUnicode.put( "braille_dots_267", 0x2862 );
        keysymUnicode.put( "braille_dots_1267", 0x2863 );
        keysymUnicode.put( "braille_dots_367", 0x2864 );
        keysymUnicode.put( "braille_dots_1367", 0x2865 );
        keysymUnicode.put( "braille_dots_2367", 0x2866 );
        keysymUnicode.put( "braille_dots_12367", 0x2867 );
        keysymUnicode.put( "braille_dots_467", 0x2868 );
        keysymUnicode.put( "braille_dots_1467", 0x2869 );
        keysymUnicode.put( "braille_dots_2467", 0x286a );
        keysymUnicode.put( "braille_dots_12467", 0x286b );
        keysymUnicode.put( "braille_dots_3467", 0x286c );
        keysymUnicode.put( "braille_dots_13467", 0x286d );
        keysymUnicode.put( "braille_dots_23467", 0x286e );
        keysymUnicode.put( "braille_dots_123467", 0x286f );
        keysymUnicode.put( "braille_dots_567", 0x2870 );
        keysymUnicode.put( "braille_dots_1567", 0x2871 );
        keysymUnicode.put( "braille_dots_2567", 0x2872 );
        keysymUnicode.put( "braille_dots_12567", 0x2873 );
        keysymUnicode.put( "braille_dots_3567", 0x2874 );
        keysymUnicode.put( "braille_dots_13567", 0x2875 );
        keysymUnicode.put( "braille_dots_23567", 0x2876 );
        keysymUnicode.put( "braille_dots_123567", 0x2877 );
        keysymUnicode.put( "braille_dots_4567", 0x2878 );
        keysymUnicode.put( "braille_dots_14567", 0x2879 );
        keysymUnicode.put( "braille_dots_24567", 0x287a );
        keysymUnicode.put( "braille_dots_124567", 0x287b );
        keysymUnicode.put( "braille_dots_34567", 0x287c );
        keysymUnicode.put( "braille_dots_134567", 0x287d );
        keysymUnicode.put( "braille_dots_234567", 0x287e );
        keysymUnicode.put( "braille_dots_1234567", 0x287f );
        keysymUnicode.put( "braille_dots_8", 0x2880 );
        keysymUnicode.put( "braille_dots_18", 0x2881 );
        keysymUnicode.put( "braille_dots_28", 0x2882 );
        keysymUnicode.put( "braille_dots_128", 0x2883 );
        keysymUnicode.put( "braille_dots_38", 0x2884 );
        keysymUnicode.put( "braille_dots_138", 0x2885 );
        keysymUnicode.put( "braille_dots_238", 0x2886 );
        keysymUnicode.put( "braille_dots_1238", 0x2887 );
        keysymUnicode.put( "braille_dots_48", 0x2888 );
        keysymUnicode.put( "braille_dots_148", 0x2889 );
        keysymUnicode.put( "braille_dots_248", 0x288a );
        keysymUnicode.put( "braille_dots_1248", 0x288b );
        keysymUnicode.put( "braille_dots_348", 0x288c );
        keysymUnicode.put( "braille_dots_1348", 0x288d );
        keysymUnicode.put( "braille_dots_2348", 0x288e );
        keysymUnicode.put( "braille_dots_12348", 0x288f );
        keysymUnicode.put( "braille_dots_58", 0x2890 );
        keysymUnicode.put( "braille_dots_158", 0x2891 );
        keysymUnicode.put( "braille_dots_258", 0x2892 );
        keysymUnicode.put( "braille_dots_1258", 0x2893 );
        keysymUnicode.put( "braille_dots_358", 0x2894 );
        keysymUnicode.put( "braille_dots_1358", 0x2895 );
        keysymUnicode.put( "braille_dots_2358", 0x2896 );
        keysymUnicode.put( "braille_dots_12358", 0x2897 );
        keysymUnicode.put( "braille_dots_458", 0x2898 );
        keysymUnicode.put( "braille_dots_1458", 0x2899 );
        keysymUnicode.put( "braille_dots_2458", 0x289a );
        keysymUnicode.put( "braille_dots_12458", 0x289b );
        keysymUnicode.put( "braille_dots_3458", 0x289c );
        keysymUnicode.put( "braille_dots_13458", 0x289d );
        keysymUnicode.put( "braille_dots_23458", 0x289e );
        keysymUnicode.put( "braille_dots_123458", 0x289f );
        keysymUnicode.put( "braille_dots_68", 0x28a0 );
        keysymUnicode.put( "braille_dots_168", 0x28a1 );
        keysymUnicode.put( "braille_dots_268", 0x28a2 );
        keysymUnicode.put( "braille_dots_1268", 0x28a3 );
        keysymUnicode.put( "braille_dots_368", 0x28a4 );
        keysymUnicode.put( "braille_dots_1368", 0x28a5 );
        keysymUnicode.put( "braille_dots_2368", 0x28a6 );
        keysymUnicode.put( "braille_dots_12368", 0x28a7 );
        keysymUnicode.put( "braille_dots_468", 0x28a8 );
        keysymUnicode.put( "braille_dots_1468", 0x28a9 );
        keysymUnicode.put( "braille_dots_2468", 0x28aa );
        keysymUnicode.put( "braille_dots_12468", 0x28ab );
        keysymUnicode.put( "braille_dots_3468", 0x28ac );
        keysymUnicode.put( "braille_dots_13468", 0x28ad );
        keysymUnicode.put( "braille_dots_23468", 0x28ae );
        keysymUnicode.put( "braille_dots_123468", 0x28af );
        keysymUnicode.put( "braille_dots_568", 0x28b0 );
        keysymUnicode.put( "braille_dots_1568", 0x28b1 );
        keysymUnicode.put( "braille_dots_2568", 0x28b2 );
        keysymUnicode.put( "braille_dots_12568", 0x28b3 );
        keysymUnicode.put( "braille_dots_3568", 0x28b4 );
        keysymUnicode.put( "braille_dots_13568", 0x28b5 );
        keysymUnicode.put( "braille_dots_23568", 0x28b6 );
        keysymUnicode.put( "braille_dots_123568", 0x28b7 );
        keysymUnicode.put( "braille_dots_4568", 0x28b8 );
        keysymUnicode.put( "braille_dots_14568", 0x28b9 );
        keysymUnicode.put( "braille_dots_24568", 0x28ba );
        keysymUnicode.put( "braille_dots_124568", 0x28bb );
        keysymUnicode.put( "braille_dots_34568", 0x28bc );
        keysymUnicode.put( "braille_dots_134568", 0x28bd );
        keysymUnicode.put( "braille_dots_234568", 0x28be );
        keysymUnicode.put( "braille_dots_1234568", 0x28bf );
        keysymUnicode.put( "braille_dots_78", 0x28c0 );
        keysymUnicode.put( "braille_dots_178", 0x28c1 );
        keysymUnicode.put( "braille_dots_278", 0x28c2 );
        keysymUnicode.put( "braille_dots_1278", 0x28c3 );
        keysymUnicode.put( "braille_dots_378", 0x28c4 );
        keysymUnicode.put( "braille_dots_1378", 0x28c5 );
        keysymUnicode.put( "braille_dots_2378", 0x28c6 );
        keysymUnicode.put( "braille_dots_12378", 0x28c7 );
        keysymUnicode.put( "braille_dots_478", 0x28c8 );
        keysymUnicode.put( "braille_dots_1478", 0x28c9 );
        keysymUnicode.put( "braille_dots_2478", 0x28ca );
        keysymUnicode.put( "braille_dots_12478", 0x28cb );
        keysymUnicode.put( "braille_dots_3478", 0x28cc );
        keysymUnicode.put( "braille_dots_13478", 0x28cd );
        keysymUnicode.put( "braille_dots_23478", 0x28ce );
        keysymUnicode.put( "braille_dots_123478", 0x28cf );
        keysymUnicode.put( "braille_dots_578", 0x28d0 );
        keysymUnicode.put( "braille_dots_1578", 0x28d1 );
        keysymUnicode.put( "braille_dots_2578", 0x28d2 );
        keysymUnicode.put( "braille_dots_12578", 0x28d3 );
        keysymUnicode.put( "braille_dots_3578", 0x28d4 );
        keysymUnicode.put( "braille_dots_13578", 0x28d5 );
        keysymUnicode.put( "braille_dots_23578", 0x28d6 );
        keysymUnicode.put( "braille_dots_123578", 0x28d7 );
        keysymUnicode.put( "braille_dots_4578", 0x28d8 );
        keysymUnicode.put( "braille_dots_14578", 0x28d9 );
        keysymUnicode.put( "braille_dots_24578", 0x28da );
        keysymUnicode.put( "braille_dots_124578", 0x28db );
        keysymUnicode.put( "braille_dots_34578", 0x28dc );
        keysymUnicode.put( "braille_dots_134578", 0x28dd );
        keysymUnicode.put( "braille_dots_234578", 0x28de );
        keysymUnicode.put( "braille_dots_1234578", 0x28df );
        keysymUnicode.put( "braille_dots_678", 0x28e0 );
        keysymUnicode.put( "braille_dots_1678", 0x28e1 );
        keysymUnicode.put( "braille_dots_2678", 0x28e2 );
        keysymUnicode.put( "braille_dots_12678", 0x28e3 );
        keysymUnicode.put( "braille_dots_3678", 0x28e4 );
        keysymUnicode.put( "braille_dots_13678", 0x28e5 );
        keysymUnicode.put( "braille_dots_23678", 0x28e6 );
        keysymUnicode.put( "braille_dots_123678", 0x28e7 );
        keysymUnicode.put( "braille_dots_4678", 0x28e8 );
        keysymUnicode.put( "braille_dots_14678", 0x28e9 );
        keysymUnicode.put( "braille_dots_24678", 0x28ea );
        keysymUnicode.put( "braille_dots_124678", 0x28eb );
        keysymUnicode.put( "braille_dots_34678", 0x28ec );
        keysymUnicode.put( "braille_dots_134678", 0x28ed );
        keysymUnicode.put( "braille_dots_234678", 0x28ee );
        keysymUnicode.put( "braille_dots_1234678", 0x28ef );
        keysymUnicode.put( "braille_dots_5678", 0x28f0 );
        keysymUnicode.put( "braille_dots_15678", 0x28f1 );
        keysymUnicode.put( "braille_dots_25678", 0x28f2 );
        keysymUnicode.put( "braille_dots_125678", 0x28f3 );
        keysymUnicode.put( "braille_dots_35678", 0x28f4 );
        keysymUnicode.put( "braille_dots_135678", 0x28f5 );
        keysymUnicode.put( "braille_dots_235678", 0x28f6 );
        keysymUnicode.put( "braille_dots_1235678", 0x28f7 );
        keysymUnicode.put( "braille_dots_45678", 0x28f8 );
        keysymUnicode.put( "braille_dots_145678", 0x28f9 );
        keysymUnicode.put( "braille_dots_245678", 0x28fa );
        keysymUnicode.put( "braille_dots_1245678", 0x28fb );
        keysymUnicode.put( "braille_dots_345678", 0x28fc );
        keysymUnicode.put( "braille_dots_1345678", 0x28fd );
        keysymUnicode.put( "braille_dots_2345678", 0x28fe );
        keysymUnicode.put( "braille_dots_12345678", 0x28ff );
        keysymUnicode.put( "Sinh_ng", 0x0D82 );
        keysymUnicode.put( "Sinh_h2", 0x0D83 );
        keysymUnicode.put( "Sinh_a", 0x0D85 );
        keysymUnicode.put( "Sinh_aa", 0x0D86 );
        keysymUnicode.put( "Sinh_ae", 0x0D87 );
        keysymUnicode.put( "Sinh_aee", 0x0D88 );
        keysymUnicode.put( "Sinh_i", 0x0D89 );
        keysymUnicode.put( "Sinh_ii", 0x0D8A );
        keysymUnicode.put( "Sinh_u", 0x0D8B );
        keysymUnicode.put( "Sinh_uu", 0x0D8C );
        keysymUnicode.put( "Sinh_ri", 0x0D8D );
        keysymUnicode.put( "Sinh_rii", 0x0D8E );
        keysymUnicode.put( "Sinh_lu", 0x0D8F );
        keysymUnicode.put( "Sinh_luu", 0x0D90 );
        keysymUnicode.put( "Sinh_e", 0x0D91 );
        keysymUnicode.put( "Sinh_ee", 0x0D92 );
        keysymUnicode.put( "Sinh_ai", 0x0D93 );
        keysymUnicode.put( "Sinh_o", 0x0D94 );
        keysymUnicode.put( "Sinh_oo", 0x0D95 );
        keysymUnicode.put( "Sinh_au", 0x0D96 );
        keysymUnicode.put( "Sinh_ka", 0x0D9A );
        keysymUnicode.put( "Sinh_kha", 0x0D9B );
        keysymUnicode.put( "Sinh_ga", 0x0D9C );
        keysymUnicode.put( "Sinh_gha", 0x0D9D );
        keysymUnicode.put( "Sinh_ng2", 0x0D9E );
        keysymUnicode.put( "Sinh_nga", 0x0D9F );
        keysymUnicode.put( "Sinh_ca", 0x0DA0 );
        keysymUnicode.put( "Sinh_cha", 0x0DA1 );
        keysymUnicode.put( "Sinh_ja", 0x0DA2 );
        keysymUnicode.put( "Sinh_jha", 0x0DA3 );
        keysymUnicode.put( "Sinh_nya", 0x0DA4 );
        keysymUnicode.put( "Sinh_jnya", 0x0DA5 );
        keysymUnicode.put( "Sinh_nja", 0x0DA6 );
        keysymUnicode.put( "Sinh_tta", 0x0DA7 );
        keysymUnicode.put( "Sinh_ttha", 0x0DA8 );
        keysymUnicode.put( "Sinh_dda", 0x0DA9 );
        keysymUnicode.put( "Sinh_ddha", 0x0DAA );
        keysymUnicode.put( "Sinh_nna", 0x0DAB );
        keysymUnicode.put( "Sinh_ndda", 0x0DAC );
        keysymUnicode.put( "Sinh_tha", 0x0DAD );
        keysymUnicode.put( "Sinh_thha", 0x0DAE );
        keysymUnicode.put( "Sinh_dha", 0x0DAF );
        keysymUnicode.put( "Sinh_dhha", 0x0DB0 );
        keysymUnicode.put( "Sinh_na", 0x0DB1 );
        keysymUnicode.put( "Sinh_ndha", 0x0DB3 );
        keysymUnicode.put( "Sinh_pa", 0x0DB4 );
        keysymUnicode.put( "Sinh_pha", 0x0DB5 );
        keysymUnicode.put( "Sinh_ba", 0x0DB6 );
        keysymUnicode.put( "Sinh_bha", 0x0DB7 );
        keysymUnicode.put( "Sinh_ma", 0x0DB8 );
        keysymUnicode.put( "Sinh_mba", 0x0DB9 );
        keysymUnicode.put( "Sinh_ya", 0x0DBA );
        keysymUnicode.put( "Sinh_ra", 0x0DBB );
        keysymUnicode.put( "Sinh_la", 0x0DBD );
        keysymUnicode.put( "Sinh_va", 0x0DC0 );
        keysymUnicode.put( "Sinh_sha", 0x0DC1 );
        keysymUnicode.put( "Sinh_ssha", 0x0DC2 );
        keysymUnicode.put( "Sinh_sa", 0x0DC3 );
        keysymUnicode.put( "Sinh_ha", 0x0DC4 );
        keysymUnicode.put( "Sinh_lla", 0x0DC5 );
        keysymUnicode.put( "Sinh_fa", 0x0DC6 );
        keysymUnicode.put( "Sinh_al", 0x0DCA );
        keysymUnicode.put( "Sinh_aa2", 0x0DCF );
        keysymUnicode.put( "Sinh_ae2", 0x0DD0 );
        keysymUnicode.put( "Sinh_aee2", 0x0DD1 );
        keysymUnicode.put( "Sinh_i2", 0x0DD2 );
        keysymUnicode.put( "Sinh_ii2", 0x0DD3 );
        keysymUnicode.put( "Sinh_u2", 0x0DD4 );
        keysymUnicode.put( "Sinh_uu2", 0x0DD6 );
        keysymUnicode.put( "Sinh_ru2", 0x0DD8 );
        keysymUnicode.put( "Sinh_e2", 0x0DD9 );
        keysymUnicode.put( "Sinh_ee2", 0x0DDA );
        keysymUnicode.put( "Sinh_ai2", 0x0DDB );
        keysymUnicode.put( "Sinh_o2", 0x0DDC );
        keysymUnicode.put( "Sinh_oo2", 0x0DDD );
        keysymUnicode.put( "Sinh_au2", 0x0DDE );
        keysymUnicode.put( "Sinh_lu2", 0x0DDF );
        keysymUnicode.put( "Sinh_ruu2", 0x0DF2 );
        keysymUnicode.put( "Sinh_luu2", 0x0DF3 );
        keysymUnicode.put( "Sinh_kunddaliya", 0x0DF4 );

        StringBuilder stringBuilder = new StringBuilder();
        keysymUnicode.forEach( ( key, value ) -> {
            stringBuilder.setLength( 0 );
            stringBuilder.append( Character.toChars( value ) );
            charName.put( stringBuilder.toString(), key );
        } );
        charName.put( " ", "space" );
    }

    public boolean contains( String keyString )
    {
        return Objects.nonNull( MapUtils.getString( charName, keyString ) );
    }

    public String get( String keyString )
    {
        return MapUtils.getString( charName, keyString );
    }

}
