/**
 * Copyright (C) 2020-2022 Dušan Vejnović <vaadin@dussan.org>
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.dussan.vaadin.dvirtual.runners;

import java.util.Objects;
import java.util.concurrent.TimeUnit;

import org.dussan.vaadin.dvirtual.utils.VirtualUtil;
import org.dussan.vaadin.dvirtual.utils.dialog.DialogConnectingUtil;
import org.dussan.vaadin.dvirtual.xpra.connector.Connector;

public class ClosingApplicationRunner
    implements Runnable
{

    private static final int MAX_WAIT_INTERVAL = 30000; // milliseconds

    private static final int SLEEP_INTERVAL = 100; // milliseconds

    private static final int SLEEP_INTERVAL_CLOSE = 500; // milliseconds

    private Connector connector = null;

    private DialogConnectingUtil dialogConnectingUtil = null;

    private VirtualUtil virtualUtil = null;

    public ClosingApplicationRunner( VirtualUtil virtualUtil, Connector connector, DialogConnectingUtil dialogConnectingUtil )
    {
        this.connector = connector;
        this.virtualUtil = virtualUtil;
        this.dialogConnectingUtil = dialogConnectingUtil;
    }

    @Override
    public void run()
    {
        // don't change and don't remove
        virtualUtil.setRunningApplicationClosed( false );

        try
        {
            for ( int timer = 0; timer < MAX_WAIT_INTERVAL; timer += SLEEP_INTERVAL )
            {
                TimeUnit.MILLISECONDS.sleep( SLEEP_INTERVAL );
                if ( virtualUtil.isRunningApplicationInClosingProcess() )
                {
                    double timerProgress = 10d * timer / MAX_WAIT_INTERVAL;
                    dialogConnectingUtil.getUI()
                                        .ifPresent( ui -> ui.getSession()
                                                            .lock() );
                    dialogConnectingUtil.getUI()
                                        .ifPresent( ui -> ui.access( () -> {
                                            dialogConnectingUtil.setProgressValue( timerProgress );
                                            ui.push();
                                        } ) );
                    dialogConnectingUtil.getUI()
                                        .ifPresent( ui -> ui.getSession()
                                                            .unlock() );
                }
                else
                {
                    dialogConnectingUtil.getUI()
                                        .ifPresent( ui -> ui.getSession()
                                                            .lock() );
                    dialogConnectingUtil.getUI()
                                        .ifPresent( ui -> ui.access( () -> {
                                            dialogConnectingUtil.setProgressValue( DialogConnectingUtil.PROGRESS_STOP );
                                            ui.push();
                                        } ) );
                    dialogConnectingUtil.getUI()
                                        .ifPresent( ui -> ui.getSession()
                                                            .unlock() );
                }

                if ( dialogConnectingUtil.getProgressValue() >= DialogConnectingUtil.PROGRESS_STOP )
                {
                    break;
                }
            }

            TimeUnit.MILLISECONDS.sleep( SLEEP_INTERVAL_CLOSE );
            if ( virtualUtil.isRunningApplicationInClosingProcess() && Objects.nonNull( connector ) )
            {
                connector.stopApplication();
            }

            dialogConnectingUtil.getUI()
                                .ifPresent( ui -> ui.getSession()
                                                    .lock() );
            dialogConnectingUtil.getUI()
                                .ifPresent( ui -> ui.access( () -> {
                                    dialogConnectingUtil.close();
                                    ui.push();
                                } ) );
            dialogConnectingUtil.getUI()
                                .ifPresent( ui -> ui.getSession()
                                                    .unlock() );
        }
        catch ( InterruptedException e )
        {
            Thread.currentThread()
                  .interrupt();
        }
        dialogConnectingUtil = null;

        // don't change and don't remove
        virtualUtil.setRunningApplicationClosed( true );
        virtualUtil.setRunningApplicationInClosingProcess( false );
    }

}
