/**
 * Copyright (C) 2020-2022 Dušan Vejnović <vaadin@dussan.org>
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.dussan.vaadin.dvirtual.ui;

import java.io.IOException;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Calendar;
import java.util.Comparator;
import java.util.Date;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.Set;
import java.util.concurrent.TimeUnit;
import java.util.stream.Collectors;

import org.apache.commons.collections4.CollectionUtils;
import org.apache.commons.collections4.IterableUtils;
import org.apache.commons.collections4.PredicateUtils;
import org.apache.commons.lang3.ArrayUtils;
import org.apache.commons.lang3.StringUtils;
import org.dussan.vaadin.dvirtual.client.Command;
import org.dussan.vaadin.dvirtual.client.VirtualClient;
import org.dussan.vaadin.dvirtual.client.VirtualClient.VirtualClientAction;
import org.dussan.vaadin.dvirtual.database.Database;
import org.dussan.vaadin.dvirtual.database.data.ApplicationData;
import org.dussan.vaadin.dvirtual.database.data.IconData;
import org.dussan.vaadin.dvirtual.database.data.ServerData;
import org.dussan.vaadin.dvirtual.database.data.ServersGroupData;
import org.dussan.vaadin.dvirtual.database.data.UserData;
import org.dussan.vaadin.dvirtual.database.data.UserSessionApplicationData;
import org.dussan.vaadin.dvirtual.database.data.UserSessionData;
import org.dussan.vaadin.dvirtual.i18n.Translation;
import org.dussan.vaadin.dvirtual.icons.FeatherIcon;
import org.dussan.vaadin.dvirtual.info.ClientProperties;
import org.dussan.vaadin.dvirtual.info.PoweredByBar;
import org.dussan.vaadin.dvirtual.logout.LogOutUserDialog;
import org.dussan.vaadin.dvirtual.runners.CheckBrowserHealthRunner;
import org.dussan.vaadin.dvirtual.runners.ChildApplicationRunner;
import org.dussan.vaadin.dvirtual.runners.CloseDialogChildAppConnectingUtilRunner;
import org.dussan.vaadin.dvirtual.runners.CloseTestApplicationRunner;
import org.dussan.vaadin.dvirtual.runners.CloseWindowsRunner;
import org.dussan.vaadin.dvirtual.runners.DetachServerRunner;
import org.dussan.vaadin.dvirtual.runners.DrawingStartedRunner;
import org.dussan.vaadin.dvirtual.runners.FileDownloadRunner;
import org.dussan.vaadin.dvirtual.runners.FileUploadRunner;
import org.dussan.vaadin.dvirtual.runners.OpenDialogWindowRunner;
import org.dussan.vaadin.dvirtual.runners.StopServerRunner;
import org.dussan.vaadin.dvirtual.runners.TestApplicationRunner;
import org.dussan.vaadin.dvirtual.ui.applications.ApplicationsUI;
import org.dussan.vaadin.dvirtual.ui.bars.applications.ApplicationsBar;
import org.dussan.vaadin.dvirtual.ui.bars.navigation.NavigationBar;
import org.dussan.vaadin.dvirtual.ui.bars.options.OptionsBar;
import org.dussan.vaadin.dvirtual.ui.bars.running.RunningApplicationsBar;
import org.dussan.vaadin.dvirtual.ui.bars.running.WindowsBar;
import org.dussan.vaadin.dvirtual.ui.bars.running.WindowsBar.WindowsBarAction;
import org.dussan.vaadin.dvirtual.ui.events.KeyboardEventProcessor;
import org.dussan.vaadin.dvirtual.ui.events.MouseEventProcessor;
import org.dussan.vaadin.dvirtual.ui.events.data.KeyboardEventData;
import org.dussan.vaadin.dvirtual.ui.events.data.MouseClickData;
import org.dussan.vaadin.dvirtual.ui.events.data.MouseMoveData;
import org.dussan.vaadin.dvirtual.ui.events.data.MouseWheelData;
import org.dussan.vaadin.dvirtual.ui.file.DownloadFileDialog;
import org.dussan.vaadin.dvirtual.ui.file.UploadFileDialog;
import org.dussan.vaadin.dvirtual.utils.EncryptionUtil;
import org.dussan.vaadin.dvirtual.utils.LoggerUtil;
import org.dussan.vaadin.dvirtual.utils.NotificationUtil;
import org.dussan.vaadin.dvirtual.utils.SshUtil;
import org.dussan.vaadin.dvirtual.utils.VirtualUtil;
import org.dussan.vaadin.dvirtual.utils.dialog.DialogConnectingUtil;
import org.dussan.vaadin.dvirtual.utils.dialog.DialogErrorUtil;
import org.dussan.vaadin.dvirtual.utils.dialog.DialogInfoUtil;
import org.dussan.vaadin.dvirtual.utils.dialog.DialogUserSessionActivationUtil;
import org.dussan.vaadin.dvirtual.xpra.packets.client.ClientButtonActionPacket;
import org.dussan.vaadin.dvirtual.xpra.packets.client.ClientClipboardTokenPacket;
import org.dussan.vaadin.dvirtual.xpra.packets.client.ClientCloseWindowPacket;
import org.dussan.vaadin.dvirtual.xpra.packets.client.ClientConfigureWindowPacket;
import org.dussan.vaadin.dvirtual.xpra.packets.client.ClientDamageSequencePacket;
import org.dussan.vaadin.dvirtual.xpra.packets.client.ClientDesktopSizePacket;
import org.dussan.vaadin.dvirtual.xpra.packets.client.ClientFocusPacket;
import org.dussan.vaadin.dvirtual.xpra.packets.client.ClientKeyActionPacket;
import org.dussan.vaadin.dvirtual.xpra.packets.client.ClientPointerPositionPacket;
import org.dussan.vaadin.dvirtual.xpra.packets.client.ClientResumePacket;
import org.dussan.vaadin.dvirtual.xpra.packets.client.ClientSuspendPacket;
import org.dussan.vaadin.dvirtual.xpra.packets.client.ClientWheelMotionPacket;
import org.dussan.vaadin.dvirtual.xpra.window.Window;
import org.dussan.vaadin.dvirtual.xpra.window.WindowDimension;
import org.slf4j.Logger;

import com.jcraft.jsch.Channel;
import com.jcraft.jsch.ChannelExec;
import com.jcraft.jsch.JSchException;
import com.jcraft.jsch.Session;
import com.vaadin.flow.component.ClientCallable;
import com.vaadin.flow.component.Component;
import com.vaadin.flow.component.DetachEvent;
import com.vaadin.flow.component.UI;
import com.vaadin.flow.component.button.Button;
import com.vaadin.flow.component.html.Anchor;
import com.vaadin.flow.component.html.Div;
import com.vaadin.flow.component.html.Paragraph;
import com.vaadin.flow.component.html.Span;
import com.vaadin.flow.component.icon.Icon;
import com.vaadin.flow.component.icon.VaadinIcon;
import com.vaadin.flow.component.notification.NotificationVariant;
import com.vaadin.flow.component.orderedlayout.HorizontalLayout;
import com.vaadin.flow.component.orderedlayout.VerticalLayout;
import com.vaadin.flow.server.StreamResource;

public abstract class VirtualUI
    extends VerticalLayout
{

    private static final long serialVersionUID = -7850353967338163939L;

    private static final Logger LOGGER = LoggerUtil.getLogger();

    private static final int BYTES_LENGTH = 8192;

    private static final int MAX_SERVERS = 3;

    private static final int MAX_WAIT_INTERVAL = 10000; // milliseconds

    private static final int NO_PARENT_WINDOW = 0;

    private static final int NO_STREAM_VALUE = -1;

    private static final int ONLY_ONE_WINDOW = 1;

    private static final int SLEEP_DIALOG_INTERVAL = 1000; // milliseconds

    private static final int SLEEP_INTERVAL = 100; // milliseconds

    private static final int TEST_SERVERS = 10;

    private static final int UNDEFINED_VALUE = 0;

    private static final int UNDEFINED_WINDOW = -1;

    private static final int ZERO_WINDOW_ID = 0;

    private static final long SETUP_USER_ID = -1;

    private static final String ANCHOR_DOWNLOAD = "download";

    private static final String BROWSER_CLOSED = "The browser is closed, all open instances will be closed.";

    private static final String CLASS_BARS_LAYOUT = "bars-layout";

    private static final String CLASS_NO_BORDER = "no-border";

    private static final String CLASS_NO_BORDER_BOTTOM = "no-border-bottom";

    private static final String CLASS_DVIRTUAL_LAYOUT = "dvirtual-layout";

    private static final String CLASS_TEST_BORDER = "test-border";

    private static final String ERROR_LOGIN = "Wrong user name or password!";

    private static final String MAXIMIZED_NO_INDICATOR = "-%s;";

    private static final String MAXIMIZED_NO_SYMBOL = "-";

    private static final String MAXIMIZED_YES_INDICATOR = "+%s;";

    private static final String MAXIMIZED_YES_SYMBOL = "+";

    private static final String MODIFIERS_SPLITTER = ",";

    private static final String STRING_ARRAY_SPLITTER = ";";

    private static final String STRING_NEW_LINE = "\n";

    private static final String STYLE_CONTEXT_MENU = "oncontextmenu";

    private static final String STYLE_CONTEXT_MENU_VALUE = "return false;";

    private static final String STYLE_DISPLAY = "display";

    private static final String STYLE_DISPLAY_NONE = "none";

    private static final String STYLE_DISPLAY_VALUE = "flex";

    private static final String STYLE_FLEX_DIRECTION = "flex-direction";

    private static final String STYLE_FLEX_DIRECTION_VALUE = "column";

    private static final String STYLE_OVERFLOW = "overflow";

    private static final String STYLE_OVERFLOW_VALUE = "hidden";

    private boolean keyboardAndMouseEventsProcessorsRegistered = false;

    private double applicationAndNavigatiobBarHeight = 0;

    private int windowFocus = UNDEFINED_WINDOW;

    private long sessionTimeout = 0;

    private ApplicationsBar applicationsBar = null;

    private CanvasUI canvasUI = null;

    private transient CheckBrowserHealthRunner checkBrowserHealthRunner = null;

    private ClientProperties clientProperties = null;

    private Database database = null;

    private DialogConnectingUtil dialogConnectingUtil = null;

    private HorizontalLayout barsLayout = null;

    private Icon separatorIcon = null;

    private KeyboardEventProcessor keyboardEventProcessor = null;

    private MouseEventProcessor mouseEventProcessor = null;

    private NavigationBar navigationBar = null;

    private OptionsBar optionsBar = null;

    private PoweredByBar poweredByBar = null;

    private RunningApplicationsBar runningApplicationsBar = null;

    private UserData userData = null;

    private VirtualClient virtualClient = null;

    private VirtualUtil virtualUtil = null;

    private WindowsBar windowsBar = null;

    public enum VirtualUiAction
    {
        ARRANGE_WINDOWS, CLOSE_APPLICATION, CLOSE_WINDOW, CREATE_NEW_WINDOW, DRAW_WINDOW_ICON, NEW_WINDOW_FOCUS, NEW_WINDOW_TITLE, RECREATE_NEW_WINDOW, START_CHILD_APPLICATION, UPDATE_WINDOWS_FOCUS
    }

    protected VirtualUI()
    {
        setMargin( false );
        setPadding( false );
        setSpacing( false );
        setSizeFull();
        addClassName( CLASS_DVIRTUAL_LAYOUT );
        getStyle().set( STYLE_DISPLAY, STYLE_DISPLAY_VALUE )
                  .set( STYLE_FLEX_DIRECTION, STYLE_FLEX_DIRECTION_VALUE )
                  .set( STYLE_OVERFLOW, STYLE_OVERFLOW_VALUE );
        keyboardEventProcessor = new KeyboardEventProcessor();
        mouseEventProcessor = new MouseEventProcessor();
    }

    public abstract void logOut();

    private VirtualUI getThis()
    {
        return this;
    }

    private VirtualClient createNewVirtualClient( long userId, long userSessionId, Div canvasDiv, ClientProperties clientProperties, VirtualUtil virtualUtil )
    {
        return new VirtualClient( canvasDiv, clientProperties, virtualUtil )
        {
            private static final long serialVersionUID = 4302368832385480906L;

            private DialogConnectingUtil dialogChildAppConnectingUtil = null;

            private transient DrawingStartedRunner drawingStartedRunner = null;

            private void checkOptionBarButtons()
            {
                if ( Objects.nonNull( optionsBar ) && Objects.nonNull( windowsBar ) )
                {
                    optionsBar.setDownloadButtonEnabled( windowsBar.isNotEmpty() );
                    optionsBar.setScreenshotEnabled( windowsBar.isNotEmpty() );
                    optionsBar.setUploadButtonEnabled( windowsBar.isNotEmpty() );
                }
            }

            private void stopApplication( boolean closingApplicationDialogVisible )
            {
                // !!! DON'T CHANGE ORDER !!!
                virtualUtil.setRunningApplicationInClosingProcess( true );
                virtualClient.setWindowClosedByUser( false );
                virtualClient.setClosingApplicationDialogVisible( closingApplicationDialogVisible );
                virtualClient.stopServer();
                runningApplicationsBar.removeRunningApplication( userSessionId );
            }

            private void arrangeWindows( Integer id )
            {
                if ( id == UNDEFINED_WINDOW && windowsBar.isNotEmpty() && windowsBar.isNotOnlyOneWindow() )
                {
                    UserSessionData userSessionData = database.getUserSession( userSessionId );
                    if ( Objects.nonNull( userSessionData ) )
                    {
                        String windowsStatus = StringUtils.defaultString( userSessionData.getStatus(), StringUtils.EMPTY );
                        windowsStatus = StringUtils.replaceChars( windowsStatus, MAXIMIZED_YES_SYMBOL, MAXIMIZED_NO_SYMBOL );
                        userSessionData.setStatus( windowsStatus );
                        userSessionData.setUpdated( new Date() );
                        database.updateUserSession( userSessionData );
                        virtualUtil.setWindowsFocus( windowsStatus );
                        windowsStatus = StringUtils.remove( windowsStatus, MAXIMIZED_NO_SYMBOL );
                        virtualClient.getWindows()
                                     .stream()
                                     .filter( window -> this.isNormalWindow( window ) )
                                     .forEach( window -> window.setMaximized( false ) );
                        UI.getCurrent()
                          .getPage()
                          .executeJs( Command.JS.ARRANGE_WINDOWS, canvasDiv.getElement(), windowsStatus );

                        // sometimes the windows are not arranged correctly, so fix them
                        try
                        {
                            TimeUnit.MILLISECONDS.sleep( SLEEP_INTERVAL );
                            UI.getCurrent()
                              .getPage()
                              .executeJs( Command.JS.ARRANGE_WINDOWS, canvasDiv.getElement(), windowsStatus );
                        }
                        catch ( InterruptedException e )
                        {
                            Thread.currentThread()
                                  .interrupt();
                        }
                    }
                }
            }

            private void closeApplication( Integer id )
            {
                if ( id == UNDEFINED_WINDOW && ( virtualUtil.isLogOutInProcess() || virtualClient.isDisconnectedBySecurityError() || virtualClient.isDisconnectedByServerDetach()
                    || virtualClient.isDisconnectedByServerError() ) )
                {
                    if ( virtualUtil.isNotLogOutInProcess() )
                    {
                        if ( virtualClient.isDisconnectedBySecurityError() || virtualClient.isDisconnectedByServerError() )
                        {
                            new DialogInfoUtil( virtualUtil.getTranslation( Translation.APPLICATION_RUN_ERROR_INSTANCE ), virtualUtil.getTranslation( Translation.BUTTON_CLOSE ) ).open();
                            if ( runningApplicationsBar.isOnlyOneRunningApplication() )
                            {
                                windowsBar.closeAllWindows();
                            }
                            runningApplicationsBar.closeRunningApplication();
                            database.removeUserSession( userSessionId );
                            database.removeUserSessionApplication( userSessionId );
                        }
                        else if ( virtualClient.isDisconnectedByServerDetach() )
                        {
                            new DialogInfoUtil( virtualUtil.getTranslation( Translation.APPLICATION_INSTANCE_MOVE ), virtualUtil.getTranslation( Translation.BUTTON_CLOSE ) ).open();
                            if ( runningApplicationsBar.isOnlyOneRunningApplication() )
                            {
                                windowsBar.closeAllWindows();
                            }
                            runningApplicationsBar.closeRunningApplication();
                        }
                    }
                    else
                    {
                        runningApplicationsBar.closeRunningApplication();
                        database.removeUserSession( userSessionId );
                        database.removeUserSessionApplication( userSessionId );
                    }
                }
            }

            private void closeWindow( Integer id )
            {
                // update user session status
                UserSessionData userSessionData = database.getUserSession( userSessionId );
                if ( Objects.nonNull( userSessionData ) )
                {
                    String noMaximized = String.format( MAXIMIZED_NO_INDICATOR, id );
                    String yesMaximized = String.format( MAXIMIZED_YES_INDICATOR, id );
                    String windowsStatus = StringUtils.defaultString( userSessionData.getStatus(), StringUtils.EMPTY );
                    windowsStatus = StringUtils.remove( windowsStatus, noMaximized );
                    windowsStatus = StringUtils.remove( windowsStatus, yesMaximized );
                    userSessionData.setStatus( windowsStatus );
                    userSessionData.setUpdated( new Date() );
                    userSessionData.setWindowId( id.equals( userSessionData.getWindowId() ) ? null : userSessionData.getWindowId() );
                    database.updateUserSession( userSessionData );
                    virtualUtil.setWindowsFocus( windowsStatus );
                }
                database.removeUserSessionApplication( userSessionId, id );

                windowsBar.closeWindow( id );
                optionsBar.setArrangeWindowsButtonEnabled( windowsBar.isNotEmpty() && windowsBar.isNotOnlyOneWindow() );
                if ( Objects.nonNull( virtualClient ) && virtualClient.isDisconnectedByServerShutdown() )
                {
                    // disable running applications bar
                    UI.getCurrent()
                      .getPage()
                      .executeJs( Command.JS.DISABLE_BARS, true );

                    // close all windows
                    UI.getCurrent()
                      .getPage()
                      .executeJs( Command.JS.CLOSE_ALL_WINDOWS, canvasDiv.getElement() );
                    windowsBar.closeAllWindows();

                    // stop application
                    stopApplication( runningApplicationsBar.isEmpty() || runningApplicationsBar.isOnlyOneRunningApplication() );
                    if ( runningApplicationsBar.isEmpty() && virtualUtil.isLogOutInProcess() )
                    {
                        virtualUtil.setLogOutInProcess( false );
                    }
                    checkOptionBarButtons();

                    // enable running applications bar
                    UI.getCurrent()
                      .getPage()
                      .executeJs( Command.JS.DISABLE_BARS, false );
                }

                // update running application title
                if ( virtualUtil.isNotLogOutInProcess() && virtualUtil.isNotRunningApplicationInClosingProcess() && Objects.nonNull( userSessionData ) && windowsBar.isNotEmpty() )
                {
                    if ( windowsBar.isOnlyOneWindow() )
                    {
                        ApplicationData applicationData = userSessionData.getApplicationData();
                        if ( Objects.isNull( userSessionData.getWindowId() ) && StringUtils.isNotBlank( userSessionData.getStatus() ) )
                        {
                            String windowId = StringUtils.remove( userSessionData.getStatus(), STRING_ARRAY_SPLITTER );
                            UserSessionApplicationData userSessionApplicationData = database.getUserSessionApplication( userSessionData.getId(), Math.abs( Integer.parseInt( windowId ) ) );
                            applicationData = Objects.nonNull( userSessionApplicationData ) ? database.getApplication( userSessionApplicationData.getApplicationId() ) : applicationData;
                        }
                        runningApplicationsBar.updateTitle( applicationData.getName(), applicationData.getIconData() );
                    }
                    else
                    {
                        runningApplicationsBar.updateTitle( virtualUtil.getTranslation( Translation.APPLICATIONS_MULTIPLE ), new IconData( ZERO_WINDOW_ID, FeatherIcon.COPY.getIconName(), null ) );
                    }
                }
            }

            private void createNewWindow( Integer id )
            {
                if ( Objects.nonNull( virtualClient ) && Objects.nonNull( virtualClient.getWindow( id ) ) )
                {
                    String noMaximized = String.format( MAXIMIZED_NO_INDICATOR, id );
                    String yesMaximized = String.format( MAXIMIZED_YES_INDICATOR, id );
                    Window window = virtualClient.getWindow( id );
                    UserSessionData userSessionData = database.getUserSession( userSessionId );
                    if ( Objects.nonNull( window ) && Objects.nonNull( userSessionData ) )
                    {
                        // update running application bar
                        ApplicationData applicationData =
                            virtualUtil.canChildApplicationStarted() ? database.getApplication( virtualUtil.getChildApplicationId() ) : userSessionData.getApplicationData();
                        if ( runningApplicationsBar.addRunningApplication( userSessionId, applicationData.getName(), applicationData.getIconData() ) )
                        {
                            windowsBar.closeAllWindows();
                        }

                        // save window status
                        String windowsStatus = StringUtils.defaultString( userSessionData.getStatus(), StringUtils.EMPTY );
                        if ( virtualClient.isNormalWindow( window ) && window.getParentId() <= NO_PARENT_WINDOW && !StringUtils.contains( windowsStatus, noMaximized )
                            && !StringUtils.contains( windowsStatus, yesMaximized ) )
                        {
                            userSessionData.setWindowId( StringUtils.isEmpty( windowsStatus ) ? id : userSessionData.getWindowId() );
                            windowsStatus = StringUtils.join( windowsStatus, window.isMaximized() ? yesMaximized : noMaximized );
                        }
                        userSessionData.setLocked( true );
                        userSessionData.setStatus( windowsStatus );
                        userSessionData.setUpdated( new Date() );
                        database.updateUserSession( userSessionData );
                        window.setMaximized( StringUtils.contains( windowsStatus, yesMaximized ) );
                        virtualUtil.setWindowsFocus( windowsStatus );

                        // draw window icon if it missing
                        if ( Objects.isNull( window.getIcon() ) && ( virtualClient.isDialogWindow( window ) || virtualClient.isNormalWindow( window ) ) )
                        {
                            window.setIcon( applicationData.getIconData()
                                                           .getIcon() );
                            virtualClient.virtualClientAction( id, VirtualClientAction.DRAW_WINDOW_ICON );
                        }

                        // save session of the child application
                        if ( virtualClient.isNormalWindow( window ) && window.getParentId() <= NO_PARENT_WINDOW && virtualClient.getNormalWindowsCounter() > ONLY_ONE_WINDOW
                            && ( virtualUtil.canChildApplicationStarted() || Objects.isNull( database.getUserSessionApplication( userSessionId, id ) ) ) )
                        {
                            database.addUserSessionApplication( userSessionId, applicationData.getId(), id, userId );
                            virtualUtil.resetChildApplicationId();
                        }

                        // add new window from running application
                        if ( virtualClient.isNormalWindow( window ) && window.getParentId() <= NO_PARENT_WINDOW )
                        {
                            windowsBar.createNewWindow( window.getId(), window.getTitle(), window.getWindowType(), window.getIcon(), window.isMaximized() );
                            optionsBar.setArrangeWindowsButtonEnabled( windowsBar.isNotEmpty() && windowsBar.isNotOnlyOneWindow() );
                        }

                        // if windows must be recreated, close windows with no drawing
                        if ( virtualUtil.isRecreateNewWindowEnabled() )
                        {
                            // close all dialog windows first
                            virtualClient.getWindows()
                                         .stream()
                                         .filter( this::isDialogWindow )
                                         .filter( w -> !Objects.equals( w.getId(), window.getId() ) )
                                         .forEach( w -> getSender().onSend( new ClientCloseWindowPacket( w.getId() ) ) );

                            // close all other windows
                            virtualClient.getWindows()
                                         .stream()
                                         .filter( this::isNotDialogWindow )
                                         .filter( w -> !Objects.equals( w.getId(), window.getId() ) )
                                         .filter( w -> w.isNotDrawingStarted() )
                                         .filter( w -> StringUtils.equals( w.getApplicationName(), window.getApplicationName() ) )
                                         .forEach( w -> getSender().onSend( new ClientCloseWindowPacket( w.getId() ) ) );
                        }

                        // update running application title
                        if ( windowsBar.isNotEmpty() && windowsBar.isNotOnlyOneWindow() )
                        {
                            runningApplicationsBar.updateTitle( virtualUtil.getTranslation( Translation.APPLICATIONS_MULTIPLE ), new IconData( ZERO_WINDOW_ID, FeatherIcon.COPY.getIconName(), null ) );
                        }

                        // show all windows
                        if ( window.isNotVisible() )
                        {
                            windowsBar.windowsBarAction( window.getId(), window.isMaximized() ? WindowsBarAction.MAXIMIZE_WINDOW : WindowsBarAction.RESTORE_WINDOW );
                            if ( virtualUtil.isNotDrawingStarted() && Objects.isNull( drawingStartedRunner ) )
                            {
                                drawingStartedRunner = new DrawingStartedRunner( virtualUtil )
                                {
                                    @Override
                                    public void drawingStarted()
                                    {
                                        getUI().ifPresent( ui -> ui.access( () -> {
                                            virtualUiAction( UNDEFINED_WINDOW, VirtualUiAction.UPDATE_WINDOWS_FOCUS );
                                            UI.getCurrent()
                                              .getPage()
                                              .executeJs( Command.JS.ALL_WINDOWS_VISIBLE, canvasUI.getCanvasDiv(), true );
                                            drawingStartedRunner = null;
                                            ui.push();
                                        } ) );
                                    }
                                };
                                virtualUtil.startRunnerService( drawingStartedRunner );
                            }
                        }
                        else
                        {
                            virtualUiAction( UNDEFINED_WINDOW, VirtualUiAction.UPDATE_WINDOWS_FOCUS );
                        }
                    }
                }
            }

            private void newWindowFocus( Integer id, Object... data )
            {
                if ( data.length == 1 )
                {
                    boolean maximized = (boolean) data[0];
                    if ( Objects.nonNull( dialogChildAppConnectingUtil ) )
                    {
                        CloseDialogChildAppConnectingUtilRunner childAppConnectingUtilRunner = new CloseDialogChildAppConnectingUtilRunner( virtualClient )
                        {
                            @Override
                            public void closeDialog()
                            {
                                getUI().ifPresent( ui -> ui.access( () -> {
                                    if ( Objects.nonNull( dialogChildAppConnectingUtil ) )
                                    {
                                        dialogChildAppConnectingUtil.close();
                                        dialogChildAppConnectingUtil = null;
                                    }
                                    ui.push();
                                } ) );
                            }
                        };
                        virtualUtil.startRunnerService( childAppConnectingUtilRunner );
                    }
                    windowsBar.setNewWindowFocus( id );
                    windowsBar.setWindowMaximized( id, maximized );
                    windowFocus = id;
                }
            }

            private void newWindowTitle( Integer id, Object... data )
            {
                if ( data.length == 3 )
                {
                    boolean maximized = (boolean) data[2];
                    byte[] icon = (byte[]) data[1];
                    String title = (String) data[0];
                    windowsBar.updateWindow( id, title, icon );
                    windowsBar.setWindowMaximized( id, maximized );
                }
            }

            private void recreateNewWindow( Integer id )
            {
                if ( Objects.nonNull( virtualClient ) && id == UNDEFINED_WINDOW )
                {
                    UserSessionData userSessionData = database.getUserSession( runningApplicationsBar.getActiveRunningApplicationId() );
                    if ( Objects.nonNull( userSessionData ) )
                    {
                        ApplicationData applicationData = userSessionData.getApplicationData();
                        if ( virtualUtil.haveToStartChildApplication() )
                        {
                            applicationData = Objects.requireNonNullElse( database.getApplication( virtualUtil.getChildApplicationId() ), applicationData );
                        }

                        // close all dialog windows before start new client
                        virtualClient.getWindows()
                                     .stream()
                                     .filter( this::isDialogWindow )
                                     .forEach( window -> getSender().onSend( new ClientCloseWindowPacket( window.getId() ) ) );

                        // close all other windows where drawing has not yet begun
                        virtualClient.getWindows()
                                     .stream()
                                     .filter( window -> window.isNotDrawingStarted() )
                                     .forEach( window -> getSender().onSend( new ClientCloseWindowPacket( window.getId() ) ) );

                        // start new client
                        virtualUtil.setDrawingStarted( false );
                        virtualClient.setWindowAutoCenterEnabled( true );
                        ChildApplicationRunner childApplicationRunner = new ChildApplicationRunner( userSessionData, applicationData.getCommand(), null, virtualUtil )
                        {
                            @Override
                            public void applicationStarted()
                            {
                                getUI().ifPresent( ui -> ui.access( () -> {
                                    virtualUtil.resetChildApplicationId();
                                    if ( Objects.nonNull( dialogChildAppConnectingUtil ) )
                                    {
                                        UserSessionData userSessionData = database.getUserSession( runningApplicationsBar.getActiveRunningApplicationId() );
                                        if ( Objects.nonNull( userSessionData ) )
                                        {
                                            String[] windowIds = StringUtils.split( userSessionData.getStatus(), STRING_ARRAY_SPLITTER );
                                            int windowFocusId = Math.abs( Integer.parseInt( windowIds[windowIds.length - 1] ) );
                                            virtualClient.virtualClientAction( windowFocusId, VirtualClientAction.NEW_WINDOW_FOCUS );
                                        }
                                        else
                                        {
                                            dialogChildAppConnectingUtil.close();
                                            dialogChildAppConnectingUtil = null;
                                        }
                                    }
                                    else if ( Objects.nonNull( virtualClient ) && Objects.nonNull( virtualClient.getConnector()
                                                                                                                .getDialogConnectingUtil() )
                                        && virtualClient.getConnector()
                                                        .getDialogConnectingUtil()
                                                        .isOpened() )
                                    {
                                        virtualClient.getConnector()
                                                     .getDialogConnectingUtil()
                                                     .close();
                                    }
                                    ui.push();
                                } ) );
                            }

                            @Override
                            public void connectionProgress( double progress )
                            {
                                getUI().ifPresent( ui -> ui.access( () -> {
                                    if ( Objects.nonNull( dialogChildAppConnectingUtil ) && virtualUtil.isRecreateNewWindowEnabled()
                                        && !StringUtils.equals( dialogChildAppConnectingUtil.getMessage(), virtualUtil.getTranslation( Translation.APPLICATION_ERROR_CONNECT ) ) )
                                    {
                                        dialogChildAppConnectingUtil.setProgressBarVisible( true );
                                        dialogChildAppConnectingUtil.setProgressValue( progress );
                                    }
                                    ui.push();
                                } ) );
                            }

                            @Override
                            public void startError()
                            {
                                getUI().ifPresent( ui -> ui.access( () -> {
                                    virtualUtil.resetChildApplicationId();
                                    virtualUtil.setRecreateNewWindowEnabled( false );
                                    if ( Objects.nonNull( dialogChildAppConnectingUtil )
                                        && !StringUtils.equals( dialogChildAppConnectingUtil.getMessage(), virtualUtil.getTranslation( Translation.APPLICATION_ERROR_CONNECT ) ) )
                                    {
                                        dialogChildAppConnectingUtil.setButtonsVisible( true );
                                        dialogChildAppConnectingUtil.setMessage( virtualUtil.getTranslation( Translation.APPLICATION_ERROR_CONNECT ) );
                                        dialogChildAppConnectingUtil.setProgressBarVisible( false );
                                        dialogChildAppConnectingUtil.addOpenedChangeListener( event -> {
                                            if ( !event.isOpened() )
                                            {
                                                virtualUtil.setDrawingStarted( true );
                                            }
                                        } );
                                    }
                                    ui.push();
                                } ) );
                            }
                        };
                        virtualUtil.startRunnerService( childApplicationRunner );
                    }
                }
            }

            private void startChildApplication( Integer id )
            {
                if ( Objects.nonNull( virtualClient ) && id == UNDEFINED_WINDOW && virtualUtil.canChildApplicationStarted() )
                {
                    UI.getCurrent()
                      .getPage()
                      .executeJs( Command.JS.ALL_WINDOWS_VISIBLE, canvasUI.getCanvasDiv(), false );
                    if ( Objects.isNull( dialogConnectingUtil ) || !dialogConnectingUtil.isOpened() )
                    {
                        getUI().ifPresent( ui -> ui.access( () -> {
                            dialogChildAppConnectingUtil =
                                new DialogConnectingUtil( virtualUtil.getTranslation( Translation.APPLICATION_WAIT ), virtualUtil.getTranslation( Translation.BUTTON_CLOSE ) );
                            dialogChildAppConnectingUtil.setButtonsVisible( false );
                            dialogChildAppConnectingUtil.setProgressBarVisible( false );
                            dialogChildAppConnectingUtil.open();
                            ui.push();
                        } ) );
                    }

                    // start new client
                    UserSessionData userSessionData = database.getUserSession( runningApplicationsBar.getActiveRunningApplicationId() );
                    ApplicationData applicationData = database.getApplication( virtualUtil.getChildApplicationId() );
                    if ( Objects.nonNull( userSessionData ) && Objects.nonNull( applicationData ) )
                    {
                        virtualUtil.setDrawingStarted( false );
                        virtualUtil.setChildApplicationCanStarted( true );
                        virtualClient.setWindowAutoCenterEnabled( true );
                        ChildApplicationRunner childApplicationRunner = new ChildApplicationRunner( userSessionData, applicationData.getCommand(), virtualClient, virtualUtil )
                        {
                            @Override
                            public void applicationStarted()
                            {
                                getUI().ifPresent( ui -> ui.access( () -> {
                                    virtualUtil.resetChildApplicationId();
                                    if ( Objects.nonNull( dialogChildAppConnectingUtil ) )
                                    {
                                        UserSessionData userSessionData = database.getUserSession( runningApplicationsBar.getActiveRunningApplicationId() );
                                        if ( Objects.nonNull( userSessionData ) )
                                        {
                                            String[] windowIds = StringUtils.split( userSessionData.getStatus(), STRING_ARRAY_SPLITTER );
                                            int windowFocusId = Math.abs( Integer.parseInt( windowIds[windowIds.length - 1] ) );
                                            virtualClient.virtualClientAction( windowFocusId, VirtualClientAction.NEW_WINDOW_FOCUS );
                                        }
                                        else
                                        {
                                            dialogChildAppConnectingUtil.close();
                                            dialogChildAppConnectingUtil = null;
                                        }
                                    }
                                    else if ( Objects.nonNull( virtualClient ) && Objects.nonNull( virtualClient.getConnector()
                                                                                                                .getDialogConnectingUtil() )
                                        && virtualClient.getConnector()
                                                        .getDialogConnectingUtil()
                                                        .isOpened() )
                                    {
                                        virtualClient.getConnector()
                                                     .getDialogConnectingUtil()
                                                     .close();
                                    }
                                    ui.push();
                                } ) );
                            }

                            @Override
                            public void connectionProgress( double progress )
                            {
                                getUI().ifPresent( ui -> ui.access( () -> {
                                    if ( virtualUtil.isNotRecreateNewWindowEnabled() )
                                    {
                                        if ( Objects.isNull( dialogConnectingUtil ) || !dialogConnectingUtil.isOpened() )
                                        {
                                            dialogChildAppConnectingUtil =
                                                new DialogConnectingUtil( virtualUtil.getTranslation( Translation.APPLICATION_WAIT ), virtualUtil.getTranslation( Translation.BUTTON_CLOSE ) );
                                            dialogChildAppConnectingUtil.setButtonsVisible( false );
                                            dialogChildAppConnectingUtil.setProgressBarVisible( true );
                                            dialogChildAppConnectingUtil.setProgressValue( progress );
                                        }
                                        else
                                        {
                                            dialogConnectingUtil.setButtonsVisible( false );
                                            dialogConnectingUtil.setProgressBarVisible( true );
                                            dialogConnectingUtil.setProgressValue( progress );
                                        }
                                    }
                                    ui.push();
                                } ) );
                            }

                            @Override
                            public void startError()
                            {
                                getUI().ifPresent( ui -> ui.access( () -> {
                                    virtualUtil.resetChildApplicationId();
                                    virtualUtil.setRecreateNewWindowEnabled( false );
                                    if ( Objects.isNull( dialogConnectingUtil ) || !dialogConnectingUtil.isOpened() )
                                    {
                                        dialogChildAppConnectingUtil.setButtonsVisible( true );
                                        dialogChildAppConnectingUtil.setMessage( virtualUtil.getTranslation( Translation.APPLICATION_ERROR_CONNECT ) );
                                        dialogChildAppConnectingUtil.setProgressBarVisible( false );
                                        dialogChildAppConnectingUtil.addOpenedChangeListener( event -> {
                                            if ( !event.isOpened() )
                                            {
                                                virtualUtil.setDrawingStarted( true );
                                            }
                                        } );
                                    }
                                    else
                                    {
                                        dialogConnectingUtil.setButtonsVisible( true );
                                        dialogConnectingUtil.setMessage( virtualUtil.getTranslation( Translation.APPLICATION_ERROR_CONNECT ) );
                                        dialogConnectingUtil.setProgressBarVisible( false );
                                        virtualUtil.setDrawingStarted( true );
                                    }
                                    ui.push();
                                } ) );
                            }
                        };
                        virtualUtil.startRunnerService( childApplicationRunner );
                    }
                    else
                    {
                        getUI().ifPresent( ui -> ui.access( () -> {
                            dialogChildAppConnectingUtil.close();
                            ui.push();
                        } ) );
                    }
                }
            }

            private void updateWindowFocus()
            {
                if ( Objects.nonNull( virtualClient ) )
                {
                    UserSessionData userSessionData = database.getUserSession( runningApplicationsBar.getActiveRunningApplicationId() );
                    if ( Objects.nonNull( userSessionData ) )
                    {
                        Window window = virtualClient.getWindows()
                                                     .stream()
                                                     .filter( this::isDialogWindow )
                                                     .findFirst()
                                                     .orElse( null );
                        String windowStatus = Objects.nonNull( window ) ? String.format( MAXIMIZED_NO_INDICATOR, window.getId() ) : StringUtils.EMPTY;
                        UI.getCurrent()
                          .getPage()
                          .executeJs( Command.JS.UPDATE_WINDOWS_FOCUS, canvasUI.getCanvasDiv(), StringUtils.join( userSessionData.getStatus(), windowStatus ) );
                    }
                    virtualClient.setWindowAutoCenterEnabled( true );
                }
            }

            @Override
            public void virtualUiAction( Integer id, VirtualUiAction action, Object... data )
            {
                if ( Objects.nonNull( id ) )
                {
                    switch ( action )
                    {
                        case ARRANGE_WINDOWS:
                            arrangeWindows( id );
                            break;
                        case CLOSE_APPLICATION:
                            closeApplication( id );
                            break;
                        case CLOSE_WINDOW:
                            closeWindow( id );
                            break;
                        case CREATE_NEW_WINDOW:
                            createNewWindow( id );
                            break;
                        case NEW_WINDOW_FOCUS:
                            newWindowFocus( id, data );
                            break;
                        case DRAW_WINDOW_ICON:
                        case NEW_WINDOW_TITLE:
                            newWindowTitle( id, data );
                            break;
                        case RECREATE_NEW_WINDOW:
                            recreateNewWindow( id );
                            break;
                        case START_CHILD_APPLICATION:
                            startChildApplication( id );
                            break;
                        case UPDATE_WINDOWS_FOCUS:
                            updateWindowFocus();
                            break;
                        default:
                            break;
                    }

                    if ( Objects.nonNull( virtualClient ) && virtualUtil.isAllRunningInSameInstance() )
                    {
                        virtualClient.setOldRunningApplicationDetached( true );
                    }

                    if ( Objects.nonNull( optionsBar ) && Objects.nonNull( runningApplicationsBar ) )
                    {
                        optionsBar.setDownloadButtonEnabled( !runningApplicationsBar.isEmpty() );
                        optionsBar.setScreenshotEnabled( !runningApplicationsBar.isEmpty() );
                        optionsBar.setUploadButtonEnabled( !runningApplicationsBar.isEmpty() );
                    }
                }
            }
        };
    }

    private void closeBrowser()
    {
        LOGGER.warn( BROWSER_CLOSED );
        if ( Objects.nonNull( virtualUtil ) )
        {
            virtualUtil.setBrowserOpened( false );
            virtualUtil.logOutUser();
            virtualUtil.stopAllStartedServices();
            virtualUtil.setUserName( null );
            virtualUtil.setUserPassword( null );
            database.close();
        }
    }

    private void registerKeyboardEventsProcessor( UI ui )
    {
        // key down
        ui.getElement()
          .addEventListener( KeyboardEventProcessor.KEYBOARD_KEYDOWN, domEvent -> {
              if ( Objects.nonNull( virtualClient ) && Objects.nonNull( virtualClient.getWindow( windowFocus ) ) )
              {
                  Map<String, String> modifiersMap = virtualClient.getWindow( windowFocus )
                                                                  .getModifiersMap();
                  KeyboardEventData keyboardEventData = keyboardEventProcessor.process( domEvent.getEventData(), modifiersMap, true, clientProperties.isWindowsOS(), clientProperties.isMacOS() );
                  if ( Objects.nonNull( keyboardEventData ) )
                  {
                      virtualClient.getSender()
                                   .onSend( new ClientKeyActionPacket( windowFocus, keyboardEventData.getKeyCode(), keyboardEventData.getKeyValue(), keyboardEventData.getKeyName(),
                                                                       keyboardEventData.getKeyString(), keyboardEventData.getModifiers(), keyboardEventData.getKeyPressed1() ) );
                      if ( Objects.nonNull( keyboardEventData.getKeyPressed2() ) )
                      {
                          virtualClient.getSender()
                                       .onSend( new ClientKeyActionPacket( windowFocus, keyboardEventData.getKeyCode(), keyboardEventData.getKeyValue(), keyboardEventData.getKeyName(),
                                                                           keyboardEventData.getKeyString(), keyboardEventData.getModifiers(), keyboardEventData.getKeyPressed2() ) );
                      }
                  }
              }
          } )
          .addEventData( KeyboardEventProcessor.KEYBOARD_ALT_KEY )
          .addEventData( KeyboardEventProcessor.KEYBOARD_CTRL_KEY )
          .addEventData( KeyboardEventProcessor.KEYBOARD_IS_COMPOSING )
          .addEventData( KeyboardEventProcessor.KEYBOARD_KEY_CODE )
          .addEventData( KeyboardEventProcessor.KEYBOARD_KEY_NAME )
          .addEventData( KeyboardEventProcessor.KEYBOARD_KEY_STRING )
          .addEventData( KeyboardEventProcessor.KEYBOARD_LOCATION )
          .addEventData( KeyboardEventProcessor.KEYBOARD_META_KEY )
          .addEventData( KeyboardEventProcessor.KEYBOARD_SHIFT_KEY )
          .addEventData( KeyboardEventProcessor.KEYBOARD_WHICH );

        // key up
        ui.getElement()
          .addEventListener( KeyboardEventProcessor.KEYBOARD_KEYUP, domEvent -> {
              if ( Objects.nonNull( virtualClient ) && Objects.nonNull( virtualClient.getWindow( windowFocus ) ) )
              {
                  Map<String, String> modifiersMap = virtualClient.getWindow( windowFocus )
                                                                  .getModifiersMap();
                  KeyboardEventData keyboardEvent = keyboardEventProcessor.process( domEvent.getEventData(), modifiersMap, false, clientProperties.isWindowsOS(), clientProperties.isMacOS() );
                  if ( Objects.nonNull( keyboardEvent ) )
                  {
                      virtualClient.getSender()
                                   .onSend( new ClientKeyActionPacket( windowFocus, keyboardEvent.getKeyCode(), keyboardEvent.getKeyValue(), keyboardEvent.getKeyName(), keyboardEvent.getKeyString(),
                                                                       keyboardEvent.getModifiers(), keyboardEvent.getKeyPressed1() ) );
                      if ( Objects.nonNull( keyboardEvent.getKeyPressed2() ) )
                      {
                          virtualClient.getSender()
                                       .onSend( new ClientKeyActionPacket( windowFocus, keyboardEvent.getKeyCode(), keyboardEvent.getKeyValue(), keyboardEvent.getKeyName(),
                                                                           keyboardEvent.getKeyString(), keyboardEvent.getModifiers(), keyboardEvent.getKeyPressed2() ) );
                      }
                  }
              }
          } )
          .addEventData( KeyboardEventProcessor.KEYBOARD_ALT_KEY )
          .addEventData( KeyboardEventProcessor.KEYBOARD_CTRL_KEY )
          .addEventData( KeyboardEventProcessor.KEYBOARD_IS_COMPOSING )
          .addEventData( KeyboardEventProcessor.KEYBOARD_KEY_CODE )
          .addEventData( KeyboardEventProcessor.KEYBOARD_KEY_NAME )
          .addEventData( KeyboardEventProcessor.KEYBOARD_KEY_STRING )
          .addEventData( KeyboardEventProcessor.KEYBOARD_LOCATION )
          .addEventData( KeyboardEventProcessor.KEYBOARD_META_KEY )
          .addEventData( KeyboardEventProcessor.KEYBOARD_SHIFT_KEY )
          .addEventData( KeyboardEventProcessor.KEYBOARD_WHICH );

    }

    private void registerMouseEventsProcessor()
    {
        // mouse move
        canvasUI.getElement()
                .addEventListener( MouseEventProcessor.MOUSE_MOVE, domEvent -> {
                    mouseEventProcessor.setCapsLock( keyboardEventProcessor.getCapsLock() );
                    mouseEventProcessor.setNumLock( keyboardEventProcessor.getNumLock() );
                    if ( Objects.nonNull( virtualClient ) )
                    {
                        MouseMoveData mouseMoveData = mouseEventProcessor.processMouseMove( virtualClient, domEvent.getEventData(), clientProperties.isMacOS() );
                        if ( Objects.nonNull( mouseMoveData ) )
                        {
                            virtualClient.getSender()
                                         .onSend( new ClientPointerPositionPacket( mouseMoveData.getId(), mouseMoveData.getX(), mouseMoveData.getY() + (int) applicationAndNavigatiobBarHeight,
                                                                                   mouseMoveData.getModifiers() ) );
                            virtualClient.getSender()
                                         .onSend( new ClientPointerPositionPacket( ZERO_WINDOW_ID, mouseMoveData.getX(), mouseMoveData.getY() + (int) applicationAndNavigatiobBarHeight,
                                                                                   mouseMoveData.getModifiers() ) );
                        }
                    }
                } )
                .addEventData( MouseEventProcessor.ELEMENT_ID )
                .addEventData( MouseEventProcessor.KEYBOARD_ALT_KEY )
                .addEventData( MouseEventProcessor.KEYBOARD_CTRL_KEY )
                .addEventData( MouseEventProcessor.KEYBOARD_META_KEY )
                .addEventData( MouseEventProcessor.KEYBOARD_SHIFT_KEY )
                .addEventData( MouseEventProcessor.MOUSE_PAGE_X )
                .addEventData( MouseEventProcessor.MOUSE_PAGE_Y )
                .throttle( virtualUtil.getMouseMoveThrottlePeriod() );

        // mouse down
        canvasUI.getElement()
                .addEventListener( MouseEventProcessor.MOUSE_DOWN, domEvent -> {
                    mouseEventProcessor.setCapsLock( keyboardEventProcessor.getCapsLock() );
                    mouseEventProcessor.setNumLock( keyboardEventProcessor.getNumLock() );
                    if ( Objects.nonNull( virtualClient ) )
                    {
                        MouseClickData mouseClickData = mouseEventProcessor.processMouseClick( virtualClient, domEvent.getEventData(), true, clientProperties.isMacOS() );
                        if ( Objects.nonNull( mouseClickData ) )
                        {
                            virtualUtil.setRecreateNewWindowEnabled( false );
                            virtualClient.getSender()
                                         .onSend( new ClientButtonActionPacket( mouseClickData.getId(), mouseClickData.getX(), mouseClickData.getY() + (int) applicationAndNavigatiobBarHeight,
                                                                                mouseClickData.getButton(), mouseClickData.getModifiers(), mouseClickData.getButtonPressed() ) );
                            virtualClient.getSender()
                                         .onSend( new ClientButtonActionPacket( ZERO_WINDOW_ID, mouseClickData.getX(), mouseClickData.getY() + (int) applicationAndNavigatiobBarHeight,
                                                                                mouseClickData.getButton(), mouseClickData.getModifiers(), false ) );
                        }
                        virtualClient.resetReconnectCounter();
                    }
                } )
                .addEventData( MouseEventProcessor.ELEMENT_ID )
                .addEventData( MouseEventProcessor.KEYBOARD_ALT_KEY )
                .addEventData( MouseEventProcessor.KEYBOARD_CTRL_KEY )
                .addEventData( MouseEventProcessor.KEYBOARD_META_KEY )
                .addEventData( MouseEventProcessor.KEYBOARD_SHIFT_KEY )
                .addEventData( MouseEventProcessor.KEYBOARD_WHICH )
                .addEventData( MouseEventProcessor.MOUSE_BUTTON )
                .addEventData( MouseEventProcessor.MOUSE_PAGE_X )
                .addEventData( MouseEventProcessor.MOUSE_PAGE_Y );

        // mouse up
        canvasUI.getElement()
                .addEventListener( MouseEventProcessor.MOUSE_UP, domEvent -> {
                    mouseEventProcessor.setCapsLock( keyboardEventProcessor.getCapsLock() );
                    mouseEventProcessor.setNumLock( keyboardEventProcessor.getNumLock() );
                    if ( Objects.nonNull( virtualClient ) )
                    {
                        MouseClickData mouseClickData = mouseEventProcessor.processMouseClick( virtualClient, domEvent.getEventData(), false, clientProperties.isMacOS() );
                        if ( Objects.nonNull( mouseClickData ) )
                        {
                            virtualUtil.setRecreateNewWindowEnabled( false );
                            virtualClient.getSender()
                                         .onSend( new ClientButtonActionPacket( mouseClickData.getId(), mouseClickData.getX(), mouseClickData.getY() + (int) applicationAndNavigatiobBarHeight,
                                                                                mouseClickData.getButton(), mouseClickData.getModifiers(), mouseClickData.getButtonPressed() ) );
                            virtualClient.getSender()
                                         .onSend( new ClientButtonActionPacket( ZERO_WINDOW_ID, mouseClickData.getX(), mouseClickData.getY() + (int) applicationAndNavigatiobBarHeight,
                                                                                mouseClickData.getButton(), mouseClickData.getModifiers(), false ) );
                        }
                        virtualClient.resetReconnectCounter();
                    }
                } )
                .addEventData( MouseEventProcessor.ELEMENT_ID )
                .addEventData( MouseEventProcessor.KEYBOARD_ALT_KEY )
                .addEventData( MouseEventProcessor.KEYBOARD_CTRL_KEY )
                .addEventData( MouseEventProcessor.KEYBOARD_META_KEY )
                .addEventData( MouseEventProcessor.KEYBOARD_SHIFT_KEY )
                .addEventData( MouseEventProcessor.KEYBOARD_WHICH )
                .addEventData( MouseEventProcessor.MOUSE_BUTTON )
                .addEventData( MouseEventProcessor.MOUSE_PAGE_X )
                .addEventData( MouseEventProcessor.MOUSE_PAGE_Y );

        // context menu
        canvasUI.getElement()
                .addEventListener( MouseEventProcessor.MOUSE_CONTEXTMENU, domEvent -> {
                    mouseEventProcessor.setCapsLock( keyboardEventProcessor.getCapsLock() );
                    mouseEventProcessor.setNumLock( keyboardEventProcessor.getNumLock() );
                    if ( Objects.nonNull( virtualClient ) )
                    {
                        MouseClickData mouseClickData = mouseEventProcessor.processMouseClick( virtualClient, domEvent.getEventData(), true, clientProperties.isMacOS() );
                        if ( Objects.nonNull( mouseClickData ) )
                        {
                            virtualUtil.setRecreateNewWindowEnabled( false );
                            virtualClient.getSender()
                                         .onSend( new ClientButtonActionPacket( mouseClickData.getId(), mouseClickData.getX(), mouseClickData.getY() + (int) applicationAndNavigatiobBarHeight,
                                                                                mouseClickData.getButton(), mouseClickData.getModifiers(), mouseClickData.getButtonPressed() ) );
                            virtualClient.getSender()
                                         .onSend( new ClientButtonActionPacket( mouseClickData.getId(), mouseClickData.getX(), mouseClickData.getY() + (int) applicationAndNavigatiobBarHeight,
                                                                                mouseClickData.getButton(), mouseClickData.getModifiers(), false ) );
                        }
                        virtualClient.resetReconnectCounter();
                    }
                } )
                .addEventData( MouseEventProcessor.ELEMENT_ID )
                .addEventData( MouseEventProcessor.KEYBOARD_ALT_KEY )
                .addEventData( MouseEventProcessor.KEYBOARD_CTRL_KEY )
                .addEventData( MouseEventProcessor.KEYBOARD_META_KEY )
                .addEventData( MouseEventProcessor.KEYBOARD_SHIFT_KEY )
                .addEventData( MouseEventProcessor.KEYBOARD_WHICH )
                .addEventData( MouseEventProcessor.MOUSE_PAGE_X )
                .addEventData( MouseEventProcessor.MOUSE_PAGE_Y );

        // mouse wheel
        if ( StringUtils.isNotBlank( mouseEventProcessor.getMouseWheelEventId() ) )
        {
            canvasUI.getElement()
                    .addEventListener( mouseEventProcessor.getMouseWheelEventId(), domEvent -> {
                        mouseEventProcessor.setCapsLock( keyboardEventProcessor.getCapsLock() );
                        mouseEventProcessor.setNumLock( keyboardEventProcessor.getNumLock() );
                        if ( Objects.nonNull( virtualClient ) )
                        {
                            List<Object> mouseDatas = mouseEventProcessor.processMouseWheel( virtualClient, domEvent.getEventData(), clientProperties.isMacOS() );
                            if ( CollectionUtils.isNotEmpty( mouseDatas ) )
                            {
                                virtualUtil.setRecreateNewWindowEnabled( false );
                                mouseDatas.forEach( mouseData -> {
                                    if ( mouseData instanceof MouseWheelData )
                                    {
                                        MouseWheelData mouseWheelData = (MouseWheelData) mouseData;
                                        if ( mouseWheelData.getButton() != UNDEFINED_VALUE )
                                        {
                                            virtualClient.getSender()
                                                         .onSend( new ClientWheelMotionPacket( mouseWheelData.getId(), mouseWheelData.getX(), mouseWheelData.getY(), mouseWheelData.getButton(),
                                                                                               mouseWheelData.getDistance(), mouseWheelData.getModifiers() ) );
                                        }
                                    }
                                    else if ( mouseData instanceof MouseClickData )
                                    {
                                        MouseClickData mouseClickData = (MouseClickData) mouseData;
                                        virtualClient.getSender()
                                                     .onSend( new ClientButtonActionPacket( mouseClickData.getId(), mouseClickData.getX(),
                                                                                            mouseClickData.getY() + (int) applicationAndNavigatiobBarHeight, mouseClickData.getButton(),
                                                                                            mouseClickData.getModifiers(), mouseClickData.getButtonPressed() ) );
                                        virtualClient.getSender()
                                                     .onSend( new ClientButtonActionPacket( mouseClickData.getId(), mouseClickData.getX(),
                                                                                            mouseClickData.getY() + (int) applicationAndNavigatiobBarHeight, mouseClickData.getButton(),
                                                                                            mouseClickData.getModifiers(), false ) );
                                        virtualClient.resetReconnectCounter();
                                    }
                                } );
                            }
                        }
                    } )
                    .addEventData( MouseEventProcessor.ELEMENT_ID )
                    .addEventData( MouseEventProcessor.KEYBOARD_ALT_KEY )
                    .addEventData( MouseEventProcessor.KEYBOARD_CTRL_KEY )
                    .addEventData( MouseEventProcessor.KEYBOARD_META_KEY )
                    .addEventData( MouseEventProcessor.KEYBOARD_SHIFT_KEY )
                    .addEventData( MouseEventProcessor.KEYBOARD_WHICH )
                    .addEventData( MouseEventProcessor.MOUSE_AXIS )
                    .addEventData( MouseEventProcessor.MOUSE_DELTA_MODE )
                    .addEventData( MouseEventProcessor.MOUSE_DELTA_X )
                    .addEventData( MouseEventProcessor.MOUSE_DELTA_Y )
                    .addEventData( MouseEventProcessor.MOUSE_DETAIL )
                    .addEventData( MouseEventProcessor.MOUSE_PAGE_X )
                    .addEventData( MouseEventProcessor.MOUSE_PAGE_Y )
                    .addEventData( MouseEventProcessor.MOUSE_WHEEL_DELTA )
                    .addEventData( MouseEventProcessor.MOUSE_WHEEL_DELTA_X )
                    .addEventData( MouseEventProcessor.MOUSE_WHEEL_DELTA_Y );
        }
    }

    public VerticalLayout getLayout()
    {
        // dVirtual util
        if ( Objects.isNull( virtualUtil ) )
        {
            new DialogErrorUtil( DialogErrorUtil.ERROR_DVIRTUAL ).open();
            return this;
        }

        // database
        if ( Objects.isNull( database ) || database.isNotInitialized() )
        {
            new DialogErrorUtil( DialogErrorUtil.ERROR_DVIRTUAL ).open();
            return this;
        }

        // setup's user data
        if ( virtualUtil.isSetupUserActive() )
        {
            userData = new UserData();
            userData.setId( SETUP_USER_ID );
            userData.setUserName( virtualUtil.getTranslation( Translation.SETUP_ADMINISTRATION ) );
            userData.setAdministrator( true );
            userData.setEnabled( true );
        }

        // browser dimensions and client properties
        virtualUtil.setBrowserOpened( true );
        clientProperties = new ClientProperties();

        // canvas UI
        canvasUI = new CanvasUI( false );
        virtualUtil.setCanvasId( canvasUI.getCanvasId() );
        keyboardAndMouseEventsProcessorsRegistered = false;

        // navigation bar
        navigationBar = new NavigationBar( userData, database, virtualUtil )
        {
            private static final long serialVersionUID = 5436354524761704199L;

            private static final String CLASS_APPLICATION_PREVIEW = "application-preview";

            private static final String CLASS_APPLICATION_PREVIEW_OVERLAY = "overlay";

            private static final String CLASS_APPLICATION_PREVIEW_TEXT = "text";

            private long activeRunningApplicationId = 0;

            private Component component = null;

            private Div applicationPreviewDiv = null;

            private Div applicationPreviewOverlayDiv = null;

            private void setComponentsVisible( UI ui, ApplicationData applicationData )
            {
                ui.getPage()
                  .executeJs( Command.JS.CLIPBOARD_ACTIVE, false );
                component.setVisible( true );
                canvasUI.getCanvasDiv()
                        .setVisible( false );
                canvasUI.removeClassName( CLASS_TEST_BORDER );
                if ( Objects.nonNull( applicationPreviewDiv ) )
                {
                    Div canvasDiv = canvasUI.getCanvasDiv();
                    canvasDiv.removeClassName( CLASS_APPLICATION_PREVIEW );
                    canvasDiv.remove( applicationPreviewDiv, applicationPreviewOverlayDiv );
                    applicationPreviewDiv = null;
                    applicationPreviewOverlayDiv = null;
                }
                ( (ApplicationsUI) component ).editApplicationData( ui, applicationData, false );
                ( (ApplicationsUI) component ).getApplicationEditDialog()
                                              .addOpenedChangeListener( event -> {
                                                  if ( !event.isOpened() )
                                                  {
                                                      canvasUI.remove( component );
                                                      navigationBar.applicationsButtonClick();
                                                      navigationBar.applicationSelect( database.getApplication( applicationData ) );
                                                  }
                                              } );
            }

            @Override
            public void addComponent( Component component )
            {
                this.component = component;
                setUserButtonEnabled( false );
                virtualUtil.stopCheckBrowserHealthService();
                applicationsBar.setVisible( false );
                runningApplicationsBar.setVisible( false );
                separatorIcon.setVisible( false );
                windowsBar.setVisible( false );
                optionsBar.setVisible( false );
                canvasUI.add( component );
                canvasUI.addClassNames( CLASS_NO_BORDER, CLASS_NO_BORDER_BOTTOM );
                canvasUI.expand( component );
                canvasUI.getCanvasDiv()
                        .setVisible( false );
                UI.getCurrent()
                  .getPage()
                  .executeJs( Command.JS.CLIPBOARD_ACTIVE, false );

                // detach server
                if ( Objects.nonNull( virtualClient ) )
                {
                    activeRunningApplicationId = runningApplicationsBar.getActiveRunningApplicationId();
                    windowsBar.closeAllWindows();
                    runningApplicationsBar.removeAllRunningApplications();
                    virtualClient.detachServer();
                }
            }

            @Override
            public void removeComponent( Component component )
            {
                this.component = null;
                setUserButtonEnabled( true );
                canvasUI.remove( component );
                canvasUI.removeClassNames( CLASS_NO_BORDER, CLASS_NO_BORDER_BOTTOM );
                canvasUI.getCanvasDiv()
                        .setVisible( true );
                applicationsBar.refresh();
                applicationsBar.setVisible( applicationsBar.isNotEmpty() );
                navigationBar.refresh();
                runningApplicationsBar.setVisible( true );
                separatorIcon.setVisible( true );
                windowsBar.setVisible( true );
                optionsBar.setVisible( true );
                virtualUtil.setApplicationTesting( false );
                virtualUtil.setApplicationTestingEditDialogOpened( false );
                virtualUtil.setApplicationTestingError( false );
                UI.getCurrent()
                  .getPage()
                  .executeJs( Command.JS.CLIPBOARD_ACTIVE, true );

                // attach server
                UserSessionData userSessionData = database.getUserSession( activeRunningApplicationId );
                if ( Objects.nonNull( userSessionData ) )
                {
                    List<UserSessionData> userSessionDatas = database.getAllUserSessions( userSessionData.getCanvasId() );
                    virtualUtil.setWindowsStatus( userSessionData.getStatus() );
                    if ( CollectionUtils.isNotEmpty( userSessionDatas ) )
                    {
                        virtualUtil.waitToConnectorServiceIsDown();
                        userSessionDatas.stream()
                                        .filter( Objects::nonNull )
                                        .sorted( Comparator.comparingLong( UserSessionData::getId ) )
                                        .collect( Collectors.toList() )
                                        .forEach( sessionData -> {
                                            // unlock detached locked servers if needed
                                            if ( sessionData.isLocked()
                                                            .booleanValue() )
                                            {
                                                virtualUtil.detachLockedServer( sessionData.getUserData()
                                                                                           .getUserName(),
                                                                                sessionData.getServerData(), sessionData.getPort() );
                                            }

                                            // add running application
                                            ApplicationData applicationData = sessionData.getApplicationData();
                                            List<Integer> windowIds = new ArrayList<>( Arrays.asList( sessionData.getWindowId() ) );
                                            windowIds.addAll( database.getUserSessionApplications( sessionData )
                                                                      .stream()
                                                                      .mapToInt( UserSessionApplicationData::getWindowId )
                                                                      .boxed()
                                                                      .collect( Collectors.toList() ) );
                                            CollectionUtils.filter( windowIds, PredicateUtils.notNullPredicate() );
                                            runningApplicationsBar.setTooltipContext( virtualUtil.getWindowsTilte( userData.getUserName(), sessionData.getServerData(), sessionData.getPort(),
                                                                                                                   ArrayUtils.toPrimitive( windowIds.toArray( Integer[]::new ) ) ) );
                                            if ( StringUtils.countMatches( sessionData.getStatus(), STRING_ARRAY_SPLITTER ) == ONLY_ONE_WINDOW )
                                            {
                                                if ( Objects.isNull( sessionData.getWindowId() ) )
                                                {
                                                    String windowId = StringUtils.remove( sessionData.getStatus(), STRING_ARRAY_SPLITTER );
                                                    UserSessionApplicationData userSessionApplicationData =
                                                        database.getUserSessionApplication( sessionData.getId(), Math.abs( Integer.parseInt( windowId ) ) );
                                                    applicationData =
                                                        Objects.nonNull( userSessionApplicationData ) ? database.getApplication( userSessionApplicationData.getApplicationId() ) : applicationData;
                                                }
                                                runningApplicationsBar.addRunningApplication( sessionData.getId(), applicationData.getName(), applicationData.getIconData() );
                                            }
                                            else
                                            {
                                                runningApplicationsBar.addRunningApplication( sessionData.getId(), virtualUtil.getTranslation( Translation.APPLICATIONS_MULTIPLE ),
                                                                                              new IconData( ZERO_WINDOW_ID, FeatherIcon.COPY.getIconName(), null ) );
                                            }
                                            runningApplicationsBar.setRunningApplicationInactive();
                                        } );

                        // activate application
                        virtualUtil.setRunningApplicationInClosingProcess( false );
                        runningApplicationsBar.setRunningApplicationActive( activeRunningApplicationId );
                    }
                }
                activeRunningApplicationId = RunningApplicationsBar.NO_RUNNING_APPLICATIONS;

                // start check browser health
                Calendar calendar = Calendar.getInstance();
                checkBrowserHealthRunner.setBrowserTime( calendar.getTimeInMillis() );
                checkBrowserHealthRunner.setSessionTime( calendar.getTimeInMillis() );
                virtualUtil.startCheckBrowserHealthService( checkBrowserHealthRunner );
            }

            @Override
            public void selectApplication( UserData userData, ApplicationData applicationData )
            {
                applicationsBar.selectApplication( userData, applicationData.getId() );
            }

            @Override
            public void testingApplication( ApplicationData applicationData )
            {
                // only user with no active sessions can run test
                if ( CollectionUtils.isNotEmpty( database.getAllUserSessions( getUserData().getId() ) ) )
                {
                    new DialogInfoUtil( virtualUtil.getTranslation( Translation.APPLICATION_TEST_DISABLED ), virtualUtil.getTranslation( Translation.BUTTON_CLOSE ) ).open();
                }
                else
                {
                    Div canvasDiv = canvasUI.getCanvasDiv();
                    virtualUtil.setApplicationTesting( true );
                    canvasUI.addClassName( CLASS_TEST_BORDER );
                    canvasDiv.setVisible( true );
                    component.setVisible( false );

                    // application preview paragraphs
                    String[] applicationPreview = StringUtils.splitByWholeSeparator( virtualUtil.getTranslation( Translation.APPLICATION_TEST_PREVIEW ), STRING_NEW_LINE );
                    Paragraph[] paragraphs = new Paragraph[applicationPreview.length];
                    for ( int index = 0; index < applicationPreview.length; index++ )
                    {
                        paragraphs[index] = new Paragraph( StringUtils.stripToEmpty( applicationPreview[index] ) );
                    }

                    // application preview text
                    applicationPreviewDiv = new Div();
                    Span applicationPreviewSpan = new Span( paragraphs );
                    applicationPreviewDiv.add( applicationPreviewSpan );
                    applicationPreviewDiv.addClassName( CLASS_APPLICATION_PREVIEW_TEXT );
                    canvasDiv.addClassName( CLASS_APPLICATION_PREVIEW );
                    canvasDiv.add( applicationPreviewDiv );

                    // application preview overlay layer
                    applicationPreviewOverlayDiv = new Div();
                    applicationPreviewOverlayDiv.getElement()
                                                .setAttribute( STYLE_CONTEXT_MENU, STYLE_CONTEXT_MENU_VALUE );
                    applicationPreviewOverlayDiv.addClassName( CLASS_APPLICATION_PREVIEW_OVERLAY );
                    canvasDiv.add( applicationPreviewOverlayDiv );

                    // available servers for application testing
                    List<ServersGroupData> selectedServersGroupDatas = Objects.requireNonNullElse( applicationData.getServersGroupDatas(), new ArrayList<>() );
                    List<ServerData> allServerDatas = new ArrayList<>();
                    selectedServersGroupDatas.forEach( serversGroupData -> allServerDatas.addAll( serversGroupData.getServerDatas() ) );
                    List<ServerData> uniqueServerDatas = allServerDatas.stream()
                                                                       .distinct()
                                                                       .filter( Objects::nonNull )
                                                                       .filter( ServerData::isEnabled )
                                                                       .collect( Collectors.toList() );
                    // application testing notification
                    NotificationUtil notificationUtil = new NotificationUtil();
                    notificationUtil.setMessage( virtualUtil.getTranslation( Translation.APPLICATION_TESTING ) );
                    notificationUtil.setOpened( true );
                    notificationUtil.addThemeVariants( NotificationVariant.LUMO_ERROR );
                    notificationUtil.addDetachListener( event -> virtualUtil.setApplicationTesting( false ) );
                    Button notificationButton = notificationUtil.getButton();
                    notificationButton.setEnabled( notificationUtil.isNotStopped() && CollectionUtils.size( uniqueServerDatas ) > TEST_SERVERS );
                    notificationButton.setText( CollectionUtils.size( uniqueServerDatas ) <= TEST_SERVERS ? virtualUtil.getTranslation( Translation.BUTTON_CLOSE )
                                    : virtualUtil.getTranslation( Translation.BUTTON_APPLICATION_TEST_STOP ) );
                    notificationButton.setVisible( true );
                    notificationButton.addClickListener( event -> {
                        if ( notificationUtil.getProgressValue() < NotificationUtil.PROGRESS_30PERCENT )
                        {
                            notificationButton.setEnabled( false );
                            notificationUtil.setMessage( virtualUtil.getTranslation( Translation.APPLICATION_TEST_STOP ) );
                            notificationUtil.setProgressBarVisible( false );
                            notificationUtil.setStatusVisible( false );
                            notificationUtil.setStopped( true );
                            notificationUtil.setDuration( NotificationUtil.NOTIFICATION_CLOSE_LONG_DELAY );
                            setComponentsVisible( UI.getCurrent(), applicationData );
                            virtualUtil.setApplicationTesting( false );
                        }
                        else if ( windowsBar.isEmpty() && virtualUtil.isNotRunningApplicationInClosingProcess() )
                        {
                            notificationUtil.setDuration( NotificationUtil.NOTIFICATION_CLOSE_DELAY );
                            setComponentsVisible( UI.getCurrent(), applicationData );
                        }
                        else
                        {
                            // wait until running test application is not closed
                            virtualUtil.setRunningApplicationInClosingProcess( true );
                            CloseTestApplicationRunner closeTestApplicationRunner = new CloseTestApplicationRunner( virtualUtil )
                            {
                                @Override
                                public void testApplicatonClosed()
                                {
                                    getUI().ifPresent( ui -> ui.access( () -> {
                                        setComponentsVisible( ui, applicationData );
                                        ( (ApplicationsUI) component ).getApplicationEditDialog()
                                                                      .addOpenedChangeListener( event -> {
                                                                          if ( !event.isOpened() )
                                                                          {
                                                                              notificationUtil.setDuration( NotificationUtil.NOTIFICATION_CLOSE_DELAY );
                                                                          }
                                                                      } );
                                        ui.push();
                                    } ) );
                                }
                            };
                            virtualUtil.startRunnerService( closeTestApplicationRunner );

                            // close application
                            if ( Objects.nonNull( virtualClient ) )
                            {
                                // close child windows
                                virtualClient.getWindowIds()
                                             .stream()
                                             .sorted( Comparator.reverseOrder() )
                                             .filter( id -> virtualClient.isDialogWindow( virtualClient.getWindow( id ) ) )
                                             .forEach( id -> virtualClient.getSender()
                                                                          .onSend( new ClientCloseWindowPacket( id ) ) );
                                // close parent windows
                                virtualClient.getWindowIds()
                                             .stream()
                                             .sorted( Comparator.reverseOrder() )
                                             .filter( id -> virtualClient.isNormalWindow( virtualClient.getWindow( id ) ) )
                                             .forEach( id -> virtualClient.getSender()
                                                                          .onSend( new ClientCloseWindowPacket( id ) ) );
                            }
                        }
                    } );

                    // application testing runner
                    TestApplicationRunner testApplicationRunner =
                        new TestApplicationRunner( UI.getCurrent(), applicationData.getCommand(), applicationData.getCommandOptions(), uniqueServerDatas, notificationUtil, virtualUtil )
                        {
                            @Override
                            public void applicationStarted( ServerData serverData, int xpraPort )
                            {
                                // open dialog of connection status
                                dialogConnectingUtil =
                                    new DialogConnectingUtil( virtualUtil.getTranslation( Translation.APPLICATION_CONNECT ), virtualUtil.getTranslation( Translation.BUTTON_CLOSE ) );
                                dialogConnectingUtil.setButtonsVisible( false );
                                dialogConnectingUtil.setProgressBarVisible( false );
                                OpenDialogWindowRunner openDialogWindowRunner = new OpenDialogWindowRunner()
                                {
                                    @Override
                                    public void openDialogWindow()
                                    {
                                        getUI().ifPresent( ui -> ui.access( () -> {
                                            dialogConnectingUtil.open();
                                            ui.push();
                                        } ) );
                                    }
                                };
                                virtualUtil.startRunnerService( openDialogWindowRunner );
                                virtualUtil.setDrawingStarted( false );

                                // connect with the Xpra server
                                long userSessionId = database.addUserSession( userData, serverData, xpraPort, applicationData, null, canvasUI.getCanvasId(), clientProperties.getWidth(),
                                                                              clientProperties.getHeight(), UserSessionData.UNDEFINED_WINDOW, UserSessionData.DEFAULT_LOCKED );
                                UserSessionData userSessionData = database.getUserSession( userSessionId );
                                virtualClient = createNewVirtualClient( userData.getId(), userSessionId, canvasUI.getCanvasDiv(), clientProperties, virtualUtil );
                                virtualClient.connect( serverData.getAddress(), userSessionData.getPort(), database, userSessionData, dialogConnectingUtil );

                                // application testing: close all dialog windows before starting the second instance
                                if ( virtualUtil.isApplicationTesting() )
                                {
                                    virtualClient.getWindows()
                                                 .stream()
                                                 .filter( window -> virtualClient.isDialogWindow( window ) )
                                                 .forEach( window -> virtualClient.getSender()
                                                                                  .onSend( new ClientCloseWindowPacket( window.getId() ) ) );
                                }
                            }

                            @Override
                            public void applicationTestingStarted()
                            {
                                getUI().ifPresent( ui -> ui.access( () -> {
                                    navigationBar.setEnabled( false );
                                    ui.push();
                                } ) );
                            }

                            @Override
                            public void applicationTestingStopped()
                            {
                                getUI().ifPresent( ui -> ui.access( () -> {
                                    navigationBar.setEnabled( true );
                                    ui.push();
                                } ) );
                            }

                            @Override
                            public void updatingApplicationData( String iconName, boolean oneInstanceLocked, boolean sameInstance )
                            {
                                Window window = virtualClient.getWindows()
                                                             .stream()
                                                             .filter( w -> virtualClient.isNormalWindow( w ) )
                                                             .findFirst()
                                                             .orElse( null );
                                if ( Objects.nonNull( window ) )
                                {
                                    // icon data
                                    IconData iconData = database.getIcon( applicationData.getIconData() );
                                    if ( Objects.nonNull( iconData ) )
                                    {
                                        if ( ArrayUtils.isNotEmpty( window.getIcon() ) )
                                        {
                                            iconData = ArrayUtils.isEquals( iconData.getIcon(), window.getIcon() ) ? iconData : new IconData( IconData.NEW_ICON, iconName, window.getIcon() );
                                        }
                                    }
                                    else
                                    {
                                        if ( ArrayUtils.isNotEmpty( window.getIcon() ) )
                                        {
                                            iconData = new IconData( IconData.NEW_ICON, iconName, window.getIcon() );
                                        }
                                        else
                                        {
                                            iconData = database.getIcon( IconData.DEFAULT_ICON );
                                            iconData = Objects.requireNonNullElse( iconData, new IconData( IconData.NEW_ICON, IconData.DEFAULT_ICON, null ) );
                                        }
                                    }
                                    applicationData.setIconData( iconData );

                                    // one instance
                                    int oneInstance = oneInstanceLocked ? ApplicationData.ONE_INSTANCE_ENABLED_LOCKED : applicationData.getOneInstance();
                                    oneInstance = !oneInstanceLocked && oneInstance <= ApplicationData.ONE_INSTANCE_ENABLED_LOCKED ? ApplicationData.ONE_INSTANCE_DISABLED : oneInstance;
                                    applicationData.setOneInstance( oneInstance );
                                    applicationData.setSameInstance( sameInstance );
                                }
                            }
                        };
                    virtualUtil.startRunnerService( testApplicationRunner );
                }

            }

            @Override
            public void logOutUser()
            {
                if ( Objects.nonNull( runningApplicationsBar ) && runningApplicationsBar.isNotEmpty() )
                {
                    LogOutUserDialog logOutUserDialog = new LogOutUserDialog( virtualUtil )
                    {
                        private static final long serialVersionUID = 8605024640912157345L;

                        @Override
                        public void closeAllAplications( boolean closeAllAplications )
                        {
                            if ( closeAllAplications )
                            {
                                // close all applications
                                virtualUtil.setLogOutInProcess( true );
                                runningApplicationsBar.closeRunningApplication();
                            }
                            else
                            {
                                // leave all applications running
                                if ( Objects.nonNull( virtualClient ) && Objects.nonNull( database ) )
                                {
                                    UserSessionData userSessionData = database.getUserSession( runningApplicationsBar.getActiveRunningApplicationId() );
                                    if ( Objects.nonNull( userSessionData ) )
                                    {
                                        userSessionData.setLocked( false );
                                        userSessionData.setUpdated( new Date() );
                                        database.updateUserSession( userSessionData );
                                    }
                                    virtualUtil.setLogOutInProcess( true );
                                    virtualClient.detachServer();
                                }
                                logOut();
                            }
                        }
                    };
                    logOutUserDialog.open();
                }
                else
                {
                    logOut();
                }
            }
        };
        add( navigationBar );

        // applications bar
        applicationsBar = new ApplicationsBar( database, canvasUI.getCanvasId(), virtualUtil )
        {

            private static final long serialVersionUID = -7659457660900126379L;

            @Override
            public void detachOldApplication()
            {
                // detach old running application
                if ( Objects.nonNull( virtualClient ) && Objects.nonNull( runningApplicationsBar ) && !runningApplicationsBar.isEmpty() )
                {
                    UserSessionData sessionData = database.getUserSession( runningApplicationsBar.getActiveRunningApplicationId() );
                    if ( Objects.nonNull( sessionData ) )
                    {
                        List<Integer> windowIds = new ArrayList<>( Arrays.asList( sessionData.getWindowId() ) );
                        database.getUserSessionApplications( sessionData )
                                .stream()
                                .forEach( userSessionApplication -> windowIds.add( userSessionApplication.getWindowId() ) );
                        CollectionUtils.filter( windowIds, PredicateUtils.notNullPredicate() );
                        runningApplicationsBar.setTooltipContext( virtualUtil.getWindowsTilte( userData.getUserName(), sessionData.getServerData(), sessionData.getPort(),
                                                                                               ArrayUtils.toPrimitive( windowIds.toArray( Integer[]::new ) ) ) );
                        sessionData.setLocked( false );
                        sessionData.setUpdated( new Date() );
                        database.updateUserSession( sessionData );
                    }
                    virtualClient.setOldRunningApplicationDetached( false );
                    virtualClient.detachServer();
                }
            }

            @Override
            public void closeChildWindows()
            {
                if ( Objects.nonNull( virtualClient ) && Objects.nonNull( virtualClient.getSender() ) )
                {
                    virtualClient.getWindows()
                                 .stream()
                                 .filter( window -> virtualClient.isDialogWindow( window ) )
                                 .forEach( window -> virtualClient.getSender()
                                                                  .onSend( new ClientCloseWindowPacket( window.getId() ) ) );
                }
            }

            @Override
            public void runApplication( long userSessionId, DialogConnectingUtil dialogConnectingUtil )
            {
                if ( Objects.isNull( dialogConnectingUtil ) )
                {
                    if ( virtualUtil.haveToStartChildApplication() && Objects.equals( runningApplicationsBar.getActiveRunningApplicationId(), userSessionId ) )
                    {
                        // start child application
                        virtualClient.virtualClientAction( UNDEFINED_WINDOW, VirtualClientAction.START_CHILD_APPLICATION );
                    }
                    else if ( Objects.equals( runningApplicationsBar.getActiveRunningApplicationId(), userSessionId ) )
                    {
                        // set new window focus
                        UserSessionData userSessionData = database.getUserSession( userSessionId );
                        String[] windowIds = StringUtils.split( userSessionData.getStatus(), STRING_ARRAY_SPLITTER );
                        int windowFocusId = Math.abs( Integer.parseInt( windowIds[windowIds.length - 1] ) );
                        virtualClient.virtualClientAction( windowFocusId, VirtualClientAction.NEW_WINDOW_FOCUS );
                    }
                    else
                    {
                        // set running application active
                        runningApplicationsBar.setRunningApplicationActive( userSessionId );
                    }
                }
                else
                {
                    // run new application
                    UserSessionData userSessionData = database.getUserSession( userSessionId );
                    if ( Objects.nonNull( userSessionData ) )
                    {
                        // wait until connection to the server is not detached for the current running application
                        DetachServerRunner detachServerRunner = new DetachServerRunner( virtualUtil )
                        {
                            @Override
                            public void serverDetached()
                            {
                                // new virtual client
                                ServerData serverData = userSessionData.getServerData();
                                virtualUtil.setDrawingStarted( false );
                                virtualClient = createNewVirtualClient( userData.getId(), userSessionId, canvasUI.getCanvasDiv(), clientProperties, virtualUtil );
                                virtualClient.setOldRunningApplicationDetached( false );
                                virtualClient.connect( serverData.getAddress(), userSessionData.getPort(), database, userSessionData, dialogConnectingUtil );
                            }
                        };
                        virtualUtil.startRunnerService( detachServerRunner );
                    }
                }
            }
        };
        applicationsBar.setUserData( userData );
        applicationsBar.setVisible( applicationsBar.isNotEmpty() );
        if ( virtualUtil.isApplicationsBarVisible() )
        {
            add( applicationsBar );
        }

        // main layouts
        VerticalLayout mainLayout = new VerticalLayout( canvasUI );
        mainLayout.setMargin( false );
        mainLayout.setPadding( false );
        mainLayout.setSpacing( false );
        mainLayout.setSizeFull();
        mainLayout.expand( canvasUI );

        add( mainLayout );
        expand( mainLayout );

        // running applications bar
        runningApplicationsBar = new RunningApplicationsBar( virtualUtil )
        {
            private static final long serialVersionUID = -5537596429766367682L;

            @Override
            public void closeRunningApplication()
            {
                if ( virtualUtil.isBrowserOpened() )
                {
                    if ( CollectionUtils.isEmpty( virtualClient.getWindows() ) )
                    {
                        virtualClient.stopServer();
                        removeRunningApplication( getActiveRunningApplicationId() );
                    }
                    else if ( virtualClient.isDisconnectedBySecurityError() || virtualClient.isDisconnectedByServerError() )
                    {
                        windowsBar.closeAllWindows();
                        removeRunningApplication( getActiveRunningApplicationId() );
                    }
                    else if ( virtualClient.isDisconnectedByServerDetach() )
                    {
                        windowsBar.closeAllWindows();
                        removeAllRunningApplications();
                    }
                    else
                    {
                        UserSessionData userSessionData = database.getUserSession( getActiveRunningApplicationId() );
                        if ( Objects.nonNull( userSessionData ) )
                        {
                            CloseWindowsRunner closeWindowsRunner = new CloseWindowsRunner( virtualClient );
                            virtualUtil.startRunnerService( closeWindowsRunner );
                        }
                    }

                    // enable running applications bar
                    UI.getCurrent()
                      .getPage()
                      .executeJs( Command.JS.DISABLE_BARS, false );
                }
            }

            @Override
            public void closeOldAndOpenNewRunningApplication()
            {
                if ( virtualUtil.isBrowserOpened() )
                {
                    UserSessionData userSessionData = database.getUserSession( getActiveRunningApplicationId() );
                    if ( Objects.nonNull( userSessionData ) )
                    {
                        // open dialog of connection status
                        dialogConnectingUtil = new DialogConnectingUtil( virtualUtil.getTranslation( Translation.APPLICATION_CONNECT ), virtualUtil.getTranslation( Translation.BUTTON_CLOSE ) );
                        dialogConnectingUtil.setButtonsVisible( false );
                        dialogConnectingUtil.setProgressBarVisible( false );
                        dialogConnectingUtil.addOpenedChangeListener( event -> {
                            if ( !event.isOpened() )
                            {
                                userSessionData.setDesktopHeight( clientProperties.getHeight() );
                                userSessionData.setDesktopWidth( clientProperties.getWidth() );
                                userSessionData.setUpdated( new Date() );
                                database.updateUserSession( userSessionData );
                                virtualUtil.setDesktopHeight( clientProperties.getHeight() );
                                virtualUtil.setDesktopWidth( clientProperties.getWidth() );
                                virtualUtil.setWindowsStatus( userSessionData.getStatus() );
                            }
                        } );
                        OpenDialogWindowRunner openDialogWindowRunner = new OpenDialogWindowRunner()
                        {
                            @Override
                            public void openDialogWindow()
                            {
                                getUI().ifPresent( ui -> ui.access( () -> {
                                    dialogConnectingUtil.open();
                                    ui.push();
                                } ) );
                            }
                        };
                        virtualUtil.startRunnerService( openDialogWindowRunner );

                        // detach runningApplication from old runningApplication
                        if ( Objects.nonNull( virtualClient ) && !isEmpty() && getOldRunningApplicationId() != RunningApplicationsBar.NO_RUNNING_APPLICATIONS
                            && getOldRunningApplicationId() != getActiveRunningApplicationId() )
                        {
                            UserSessionData sessionData = database.getUserSession( getOldRunningApplicationId() );
                            if ( Objects.nonNull( sessionData ) )
                            {
                                List<Integer> windowIds = new ArrayList<>( Arrays.asList( sessionData.getWindowId() ) );
                                database.getUserSessionApplications( sessionData )
                                        .stream()
                                        .forEach( userSessionApplication -> windowIds.add( userSessionApplication.getWindowId() ) );
                                CollectionUtils.filter( windowIds, PredicateUtils.notNullPredicate() );
                                setTooltipContext( virtualUtil.getWindowsTilte( userData.getUserName(), sessionData.getServerData(), sessionData.getPort(),
                                                                                ArrayUtils.toPrimitive( windowIds.toArray( Integer[]::new ) ) ) );
                                sessionData.setLocked( false );
                                sessionData.setUpdated( new Date() );
                                database.updateUserSession( sessionData );
                                virtualClient.setClosingApplicationDialogVisible( false );
                                virtualClient.setOldRunningApplicationDetached( false );
                                virtualClient.detachServer();
                            }
                            else
                            {
                                // remove application because it is removed from database
                                removeRunningApplication( getOldRunningApplicationId() );
                            }
                        }
                        userSessionData.setLocked( true );
                        database.updateUserSession( userSessionData );
                        windowsBar.closeAllWindows();

                        // wait until connection to the server is not detached for the current old running application
                        DetachServerRunner detachServerRunner = new DetachServerRunner( virtualUtil )
                        {
                            @Override
                            public void serverDetached()
                            {
                                // select new running application
                                ServerData serverData = userSessionData.getServerData();
                                virtualUtil.setDrawingStarted( false );
                                virtualUtil.setDesktopHeight( userSessionData.getDesktopHeight() );
                                virtualUtil.setDesktopWidth( userSessionData.getDesktopWidth() );
                                virtualClient = createNewVirtualClient( userData.getId(), userSessionData.getId(), canvasUI.getCanvasDiv(), clientProperties, virtualUtil );
                                virtualClient.setApplicationRunning( false );
                                virtualClient.setWindowAutoCenterEnabled( false );
                                virtualClient.connect( serverData.getAddress(), userSessionData.getPort(), database, userSessionData, dialogConnectingUtil );
                                setOldRunningApplicationId( getActiveRunningApplicationId() );
                            }
                        };
                        virtualUtil.startRunnerService( detachServerRunner );
                    }

                    // enable running applications bar
                    UI.getCurrent()
                      .getPage()
                      .executeJs( Command.JS.DISABLE_BARS, false );
                }
            }
        };

        // windows bar
        windowsBar = new WindowsBar( virtualUtil )
        {

            private static final long serialVersionUID = -3423472887580042721L;

            @Override
            public void windowsBarAction( int id, WindowsBarAction action )
            {
                if ( Objects.nonNull( virtualClient ) && Objects.nonNull( virtualClient.getSender() ) )
                {
                    switch ( action )
                    {
                        case CLOSE_WINDOW:
                            virtualClient.setWindowClosedByUser( true );
                            // close child windows first
                            virtualClient.getWindows()
                                         .stream()
                                         .filter( window -> virtualClient.isDialogWindow( window ) )
                                         .filter( window -> window.getParentId() == id )
                                         .forEach( window -> virtualClient.getSender()
                                                                          .onSend( new ClientCloseWindowPacket( window.getId() ) ) );
                            virtualClient.getSender()
                                         .onSend( new ClientCloseWindowPacket( id ) );
                            break;
                        case NEW_WINDOW_FOCUS:
                            setWindowFocus( id );
                            break;
                        case MAXIMIZE_WINDOW:
                        case RESTORE_WINDOW:
                            UI.getCurrent()
                              .getPage()
                              .executeJs( Command.JS.WINDOW_ACTION, canvasUI.getCanvasDiv(), id, action.toString() );
                            break;
                        default:
                            break;
                    }
                }
            }
        };

        // options bar
        optionsBar = new OptionsBar( virtualUtil )
        {
            private static final long serialVersionUID = -5182517977731526174L;

            private DownloadFileDialog downloadFileDialog = null;

            private UploadFileDialog uploadFileDialog = null;

            @Override
            public void arrangeWindows()
            {
                if ( Objects.nonNull( virtualClient ) )
                {
                    virtualClient.virtualUiAction( UNDEFINED_WINDOW, VirtualUiAction.ARRANGE_WINDOWS );
                }
            }

            @Override
            public void downloadFile()
            {
                UserSessionData userSessionData = database.getUserSession( runningApplicationsBar.getActiveRunningApplicationId() );
                if ( isDownloadButtonEnabled() && Objects.nonNull( userSessionData ) && Objects.nonNull( virtualUtil ) && ( Objects.isNull( downloadFileDialog ) || !downloadFileDialog.isOpened() ) )
                {
                    downloadFileDialog = new DownloadFileDialog( userSessionData, virtualUtil )
                    {
                        private static final long serialVersionUID = 8907088168972720077L;

                        private DialogInfoUtil dialogInfoUtil = new DialogInfoUtil( virtualUtil.getTranslation( Translation.APPLICATION_WAIT ), StringUtils.EMPTY );

                        @Override
                        public void downloadingFile( String downloadedFileName )
                        {
                            getUI().ifPresent( ui -> {
                                if ( Objects.nonNull( virtualClient ) && Objects.nonNull( userSessionData ) )
                                {
                                    FileDownloadRunner fileDownloadRunner = new FileDownloadRunner( virtualUtil, userSessionData )
                                    {
                                        @Override
                                        public void startDownloadingFile()
                                        {
                                            ui.access( () -> {
                                                dialogInfoUtil.setButtonsVisible( false );
                                                dialogInfoUtil.open();
                                                ui.push();
                                            } );
                                        }

                                        @Override
                                        public void downloadingFile( StreamResource fileStreamResource, String fileName )
                                        {
                                            ui.access( () -> {
                                                Anchor anchor = new Anchor( fileStreamResource, StringUtils.EMPTY );
                                                anchor.getElement()
                                                      .getStyle()
                                                      .set( STYLE_DISPLAY, STYLE_DISPLAY_NONE );
                                                anchor.getElement()
                                                      .setAttribute( ANCHOR_DOWNLOAD, fileName );
                                                getThis().add( anchor );
                                                ui.getPage()
                                                  .executeJs( Command.JS.ANCHOR_CLICK, anchor );
                                                ui.push();
                                            } );
                                        }

                                        @Override
                                        public void stopDownloadingFile()
                                        {
                                            ui.access( () -> {
                                                try
                                                {
                                                    TimeUnit.MILLISECONDS.sleep( SLEEP_DIALOG_INTERVAL );
                                                    dialogInfoUtil.close();
                                                }
                                                catch ( InterruptedException exception )
                                                {
                                                    Thread.currentThread()
                                                          .interrupt();
                                                }
                                                ui.push();
                                            } );
                                        }
                                    };
                                    fileDownloadRunner.setDownloadedFileName( downloadedFileName );
                                    virtualUtil.startFileDownloadService( fileDownloadRunner );
                                }
                            } );
                        }
                    };
                    downloadFileDialog.open();
                }
            }

            @Override
            public void takeScreenshot()
            {
                UI.getCurrent()
                  .getPage()
                  .executeJs( Command.JS.SCREENSHOT, canvasUI.getCanvasDiv() );
            }

            @Override
            public void uploadFile()
            {
                UserSessionData userSessionData = database.getUserSession( runningApplicationsBar.getActiveRunningApplicationId() );
                if ( isUploadButtonEnabled() && Objects.nonNull( userSessionData ) && Objects.nonNull( virtualUtil ) && ( Objects.isNull( uploadFileDialog ) || !uploadFileDialog.isOpened() ) )
                {
                    uploadFileDialog = new UploadFileDialog( userSessionData, virtualUtil )
                    {
                        private static final long serialVersionUID = -5728654724551722864L;

                        private DialogInfoUtil dialogInfoUtil =
                            new DialogInfoUtil( virtualUtil.getTranslation( Translation.APPLICATION_WAIT ), virtualUtil.getTranslation( Translation.BUTTON_CLOSE ) );

                        @Override
                        public void prepareUploadingFile()
                        {
                            getUI().ifPresent( ui -> ui.access( () -> {
                                dialogInfoUtil.setButtonsVisible( false );
                                dialogInfoUtil.setMessage( virtualUtil.getTranslation( Translation.APPLICATION_WAIT ) );
                                dialogInfoUtil.open();
                                ui.push();
                            } ) );
                        }

                        @Override
                        public void uploadingFile( String srcUploadedFileName, String dstUploadedFileName )
                        {
                            getUI().ifPresent( ui -> {
                                if ( Objects.nonNull( virtualClient ) && Objects.nonNull( userSessionData ) )
                                {
                                    FileUploadRunner fileUploadRunner = new FileUploadRunner( virtualUtil, userSessionData )
                                    {
                                        @Override
                                        public void startUploadingFile()
                                        {
                                            ui.access( () -> {
                                                if ( !dialogInfoUtil.isOpened() )
                                                {
                                                    dialogInfoUtil.setButtonsVisible( false );
                                                    dialogInfoUtil.setMessage( virtualUtil.getTranslation( Translation.APPLICATION_WAIT ) );
                                                    dialogInfoUtil.open();
                                                }
                                                ui.push();
                                            } );
                                        }

                                        @Override
                                        public void stopUploadingFile( String fileName )
                                        {
                                            ui.access( () -> {
                                                try
                                                {
                                                    TimeUnit.MILLISECONDS.sleep( SLEEP_DIALOG_INTERVAL );
                                                    uploadFileDialog.select( fileName );
                                                    dialogInfoUtil.setButtonsVisible( true );
                                                    dialogInfoUtil.setMessage( virtualUtil.getTranslation( Translation.DATA_UPLOAD ) );
                                                }
                                                catch ( InterruptedException exception )
                                                {
                                                    Thread.currentThread()
                                                          .interrupt();
                                                }
                                                ui.push();
                                            } );
                                        }
                                    };
                                    fileUploadRunner.setUploadedFileName( srcUploadedFileName, dstUploadedFileName );
                                    virtualUtil.startFileUploadService( fileUploadRunner );
                                }
                            } );
                        }
                    };
                    uploadFileDialog.open();
                }
            }
        };
        optionsBar.setScreenshotButtonVisible( userData.isScreenshot() );
        optionsBar.setUploadButtonVisible( userData.isUpload() );
        optionsBar.setDownloadButtonVisible( userData.isDownload() );
        optionsBar.setFullScreenButtonVisible( userData.isFullScreen() );

        // separator between running applications bar and windows bar
        separatorIcon = new Icon( VaadinIcon.LINE_V );

        // bars layout
        if ( virtualUtil.isAllRunningInSameInstance() )
        {
            barsLayout = new HorizontalLayout( windowsBar, optionsBar );
            barsLayout.setAlignSelf( Alignment.CENTER, windowsBar, optionsBar );
        }
        else
        {
            barsLayout = new HorizontalLayout( runningApplicationsBar, separatorIcon, windowsBar, optionsBar );
            barsLayout.setAlignSelf( Alignment.CENTER, runningApplicationsBar, separatorIcon, windowsBar, optionsBar );
        }
        barsLayout.setJustifyContentMode( JustifyContentMode.END );
        barsLayout.addClassName( CLASS_BARS_LAYOUT );
        add( barsLayout );

        // execute startup javascripts
        UI.getCurrent()
          .getPage()
          .executeJs( Command.JS.CLIENT_PROPERTIES, canvasUI.getCanvasDiv() );
        UI.getCurrent()
          .getPage()
          .executeJs( Command.JS.INIT, canvasUI.getCanvasDiv() );
        UI.getCurrent()
          .getPage()
          .executeJs( Command.JS.CLIPBOARD_ACTIVE, true );
        UI.getCurrent()
          .getPage()
          .executeJs( Command.JS.TRANSLATIONS, virtualUtil.getTranslation( Translation.APPLICATIONS ), virtualUtil.getTranslation( Translation.APPLICATIONS_RUNNING ) );

        // start browser health checking
        virtualUtil.stopCheckBrowserHealthService();
        checkBrowserHealthRunner = new CheckBrowserHealthRunner( getSessionTimeout() )
        {
            @Override
            public void checkBrowser()
            {
                getUI().ifPresent( ui -> ui.access( () -> {
                    ui.getPage()
                      .executeJs( Command.JS.PING_FROM_BROWSER, canvasUI.getCanvasDiv() );
                    ui.push();
                } ) );
            }

            @Override
            public void closeBrowser( boolean showMessage )
            {
                if ( showMessage )
                {
                    getUI().ifPresent( ui -> ui.access( () -> {
                        UserSessionData sessionData = database.getUserSession( runningApplicationsBar.getActiveRunningApplicationId() );
                        if ( Objects.nonNull( sessionData ) )
                        {
                            sessionData.setLocked( false );
                            sessionData.setUpdated( new Date() );
                            database.updateUserSession( sessionData );
                        }
                        if ( Objects.nonNull( virtualClient ) )
                        {
                            virtualClient.detachServer();
                        }
                        virtualUtil.stopCheckBrowserHealthService();
                        DialogInfoUtil dialogInfoUtil = new DialogInfoUtil( virtualUtil.getTranslation( Translation.USERS_SESSION_EXPIRED ), virtualUtil.getTranslation( Translation.BUTTON_CLOSE ) );
                        dialogInfoUtil.setButtonsVisible( true );
                        dialogInfoUtil.addDetachListener( event -> getThis().closeBrowser() );
                        dialogInfoUtil.open();
                        ui.push();
                    } ) );
                }
                else
                {
                    getThis().closeBrowser();
                }
            }
        };
        virtualUtil.startCheckBrowserHealthService( checkBrowserHealthRunner );

        // active sessions
        List<UserSessionData> userSessionDatas = database.getAllUserSessions( userData )
                                                         .stream()
                                                         .filter( userSessionsData -> !StringUtils.equals( userSessionsData.getCanvasId(), UserSessionData.NEW_CANVAS_ID )
                                                             && !StringUtils.equals( userSessionsData.getCanvasId(), canvasUI.getCanvasId() ) )
                                                         .collect( Collectors.toList() );
        // check if session is up and running
        userSessionDatas.stream()
                        .collect( Collectors.toList() )
                        .forEach( userSessionData -> {
                            ServerData serverData = userSessionData.getServerData();
                            Channel channel = null;
                            Session session = null;
                            try
                            {
                                // start SSH session
                                session = SshUtil.createSession( virtualUtil.getUserName(),
                                                                 EncryptionUtil.decrypt( virtualUtil.getUserPassword(), virtualUtil.getSecretKeySpec(), virtualUtil.getSecureIV() ),
                                                                 serverData.getAddress(), serverData.getSshPort() );
                                session.connect();

                                // get status of running XPRA server
                                boolean serverRunning = true;
                                byte[] temporaryValue = new byte[BYTES_LENGTH];
                                channel = session.openChannel( Command.SSH.EXECUTE );
                                ( (ChannelExec) channel ).setCommand( Command.XPRA.LIST );
                                InputStream inputStream = channel.getInputStream();
                                channel.connect();
                                for ( int t = 0; t < MAX_WAIT_INTERVAL; t += SLEEP_INTERVAL )
                                {
                                    TimeUnit.MILLISECONDS.sleep( SLEEP_INTERVAL );
                                    if ( channel.isClosed() )
                                    {
                                        break;
                                    }
                                }
                                int length = NO_STREAM_VALUE;
                                while ( ( length = inputStream.read( temporaryValue, 0, BYTES_LENGTH ) ) > NO_STREAM_VALUE )
                                {
                                    if ( length > 0 )
                                    {
                                        String status = new String( temporaryValue, 0, length );
                                        serverRunning = StringUtils.contains( status, String.format( Command.XPRA.LIST_STATUS, userSessionData.getPort() ) );
                                        break;
                                    }
                                }
                                inputStream.close();
                                channel.disconnect();
                                session.disconnect();

                                if ( !serverRunning )
                                {
                                    userSessionData.setStatus( StringUtils.EMPTY );
                                }
                            }
                            catch ( IllegalThreadStateException | InterruptedException | IOException | JSchException exception )
                            {
                                Thread.currentThread()
                                      .interrupt();
                            }
                            finally
                            {
                                if ( Objects.nonNull( channel ) && channel.isConnected() )
                                {
                                    channel.disconnect();
                                }

                                if ( Objects.nonNull( session ) && session.isConnected() )
                                {
                                    session.disconnect();
                                }
                            }
                        } );

        // remove bad sessions: window status is blank
        userSessionDatas.stream()
                        .filter( userSessionData -> StringUtils.isBlank( userSessionData.getStatus() ) )
                        .collect( Collectors.toList() )
                        .forEach( userSessionData -> {
                            ServerData serverData = userSessionData.getServerData();
                            StopServerRunner stopServerRunner = new StopServerRunner( database, userSessionData.getId(), serverData, userSessionData.getPort(), virtualUtil );
                            getVirtualUtil().startRunnerService( stopServerRunner );
                        } );
        userSessionDatas.removeIf( userSessionData -> StringUtils.isBlank( userSessionData.getStatus() ) );

        // remove bad sessions: window id is null and not running any session applications
        userSessionDatas.stream()
                        .filter( userSessionData -> Objects.isNull( userSessionData.getWindowId() ) )
                        .collect( Collectors.toList() )
                        .forEach( userSessionData -> {
                            if ( CollectionUtils.isEmpty( database.getUserSessionApplications( userSessionData ) ) )
                            {
                                ServerData serverData = userSessionData.getServerData();
                                StopServerRunner stopServerRunner = new StopServerRunner( database, userSessionData.getId(), serverData, userSessionData.getPort(), virtualUtil );
                                getVirtualUtil().startRunnerService( stopServerRunner );
                            }
                        } );

        // add child applications
        for ( UserSessionData userSessionData : new ArrayList<>( userSessionDatas ) )
        {
            database.getUserSessionApplications( userSessionData )
                    .stream()
                    .forEach( userSessionApplicationData -> {
                        ApplicationData applicationData = database.getApplication( userSessionApplicationData.getApplicationId() );
                        UserSessionData sessionData = new UserSessionData();
                        sessionData.setId( userSessionData.getId() );
                        sessionData.setStatus( userSessionData.getStatus() );
                        sessionData.setCanvasId( userSessionData.getCanvasId() );
                        sessionData.setUpdated( new Date() );
                        sessionData.setWindowId( userSessionApplicationData.getWindowId() );
                        sessionData.setApplicationData( applicationData );
                        userSessionDatas.add( sessionData );
                    } );
        }
        userSessionDatas.removeIf( userSessionData -> Objects.isNull( userSessionData.getWindowId() ) );
        userSessionDatas.sort( ( UserSessionData data1, UserSessionData data2 ) -> data1.getApplicationData()
                                                                                        .getName()
                                                                                        .compareTo( data2.getApplicationData()
                                                                                                         .getName() ) );
        if ( CollectionUtils.isNotEmpty( userSessionDatas ) )
        {
            runningApplicationsBar.setOldRunningApplicationId( RunningApplicationsBar.NO_RUNNING_APPLICATIONS );
            DialogUserSessionActivationUtil dialogUserSessionActivationUtil = new DialogUserSessionActivationUtil( userSessionDatas, virtualUtil )
            {
                private static final long serialVersionUID = -5986187778051038593L;

                @Override
                public void activateSession( String sessionId )
                {
                    List<UserSessionData> userSessionDatas = database.getAllUserSessions( sessionId );
                    UserSessionData activeUserSessionData = userSessionDatas.stream()
                                                                            .sorted( Comparator.comparing( UserSessionData::getUpdated )
                                                                                               .reversed() )
                                                                            .findFirst()
                                                                            .orElse( null );
                    if ( CollectionUtils.isNotEmpty( userSessionDatas ) && Objects.nonNull( activeUserSessionData ) )
                    {
                        userSessionDatas.stream()
                                        .filter( Objects::nonNull )
                                        .sorted( Comparator.comparingLong( UserSessionData::getId ) )
                                        .collect( Collectors.toList() )
                                        .forEach( userSessionData -> {
                                            // unlock detached locked servers if needed
                                            if ( userSessionData.isLocked()
                                                                .booleanValue() )
                                            {
                                                virtualUtil.detachLockedServer( userSessionData.getUserData()
                                                                                               .getUserName(),
                                                                                userSessionData.getServerData(), userSessionData.getPort() );
                                            }

                                            // tooltip of running application
                                            if ( Objects.equals( userSessionData.getId(), activeUserSessionData.getId() ) )
                                            {
                                                runningApplicationsBar.setTooltipContext( StringUtils.EMPTY );
                                            }
                                            else
                                            {
                                                List<Integer> windowIds = new ArrayList<>( Arrays.asList( userSessionData.getWindowId() ) );
                                                windowIds.addAll( database.getUserSessionApplications( userSessionData )
                                                                          .stream()
                                                                          .mapToInt( UserSessionApplicationData::getWindowId )
                                                                          .boxed()
                                                                          .collect( Collectors.toList() ) );
                                                CollectionUtils.filter( windowIds, PredicateUtils.notNullPredicate() );
                                                runningApplicationsBar.setTooltipContext( virtualUtil.getWindowsTilte( userData.getUserName(), userSessionData.getServerData(),
                                                                                                                       userSessionData.getPort(),
                                                                                                                       ArrayUtils.toPrimitive( windowIds.toArray( Integer[]::new ) ) ) );
                                            }

                                            // add application
                                            if ( StringUtils.countMatches( userSessionData.getStatus(), STRING_ARRAY_SPLITTER ) == ONLY_ONE_WINDOW )
                                            {
                                                ApplicationData applicationData = userSessionData.getApplicationData();
                                                if ( Objects.isNull( userSessionData.getWindowId() ) )
                                                {
                                                    String windowId = StringUtils.remove( userSessionData.getStatus(), STRING_ARRAY_SPLITTER );
                                                    UserSessionApplicationData userSessionApplicationData =
                                                        database.getUserSessionApplication( userSessionData.getId(), Math.abs( Integer.parseInt( windowId ) ) );
                                                    applicationData =
                                                        Objects.nonNull( userSessionApplicationData ) ? database.getApplication( userSessionApplicationData.getApplicationId() ) : applicationData;
                                                }
                                                runningApplicationsBar.addRunningApplication( userSessionData.getId(), applicationData.getName(), applicationData.getIconData() );
                                            }
                                            else
                                            {
                                                runningApplicationsBar.addRunningApplication( userSessionData.getId(), virtualUtil.getTranslation( Translation.APPLICATIONS_MULTIPLE ),
                                                                                              new IconData( ZERO_WINDOW_ID, FeatherIcon.COPY.getIconName(), null ) );
                                            }
                                            runningApplicationsBar.setRunningApplicationInactive();
                                            userSessionData.setCanvasId( canvasUI.getCanvasId() );
                                            userSessionData.setLocked( false );
                                            userSessionData.setUpdated( new Date() );
                                            database.updateUserSession( userSessionData );
                                        } );

                        // close dialog
                        close();

                        // activate application
                        virtualUtil.setDesktopHeight( activeUserSessionData.getDesktopHeight() );
                        virtualUtil.setDesktopWidth( activeUserSessionData.getDesktopWidth() );
                        virtualUtil.setDrawingStarted( false );
                        virtualUtil.setWindowsStatus( activeUserSessionData.getStatus() );
                        activeUserSessionData.setLocked( true );
                        activeUserSessionData.setUpdated( new Date() );
                        database.updateUserSession( activeUserSessionData );
                        runningApplicationsBar.setRunningApplicationActive( activeUserSessionData.getId() );
                    }
                }
            };
            dialogUserSessionActivationUtil.open();
        }

        // powered by
        if ( Objects.isNull( poweredByBar ) )
        {
            poweredByBar = new PoweredByBar();
        }
        add( poweredByBar );

        // dVirtual setup: not yet complete
        if ( Objects.nonNull( database ) && ( virtualUtil.isSetupUserActive() || userData.isAdministrator() ) )
        {
            if ( database.isServersDatabaseEmpty() )
            {
                // servers setup: no servers registered
                navigationBar.setupServers();
                new DialogInfoUtil( virtualUtil.getTranslation( Translation.SETUP_SERVERS ), virtualUtil.getTranslation( Translation.BUTTON_CLOSE ) ).open();
            }
            else if ( database.isServerGroupsDatabaseEmpty() )
            {
                // server groups setup: no server groups registered
                navigationBar.setupServerGroups();
                new DialogInfoUtil( virtualUtil.getTranslation( Translation.SETUP_SERVER_GROUPS ), virtualUtil.getTranslation( Translation.BUTTON_CLOSE ) ).open();
            }
            else if ( database.isApplicationsDatabaseEmpty() )
            {
                // applications setup: no applications registered
                navigationBar.setupApplications();
                new DialogInfoUtil( virtualUtil.getTranslation( Translation.SETUP_APPLICATIONS ), virtualUtil.getTranslation( Translation.BUTTON_CLOSE ) ).open();
            }
            else if ( database.isUsersDatabaseEmpty() )
            {
                // users setup: no users registered
                navigationBar.setupUsers();
                new DialogInfoUtil( virtualUtil.getTranslation( Translation.SETUP_USERS ), virtualUtil.getTranslation( Translation.BUTTON_CLOSE ) ).open();
            }
            else if ( !virtualUtil.isSetupFileDeleted() )
            {
                new DialogInfoUtil( virtualUtil.getTranslation( Translation.SETUP_DELETE ), virtualUtil.getTranslation( Translation.BUTTON_CLOSE ) ).open();
            }
        }

        return this;
    }

    public VirtualUtil getVirtualUtil()
    {
        return virtualUtil;
    }

    public void setVirtualUtil( VirtualUtil virtualUtil )
    {
        this.virtualUtil = virtualUtil;
    }

    public Database getDatabase()
    {
        return database;
    }

    public void setDatabase( Database database )
    {
        this.database = database;
    }

    public UserData getUserData()
    {
        return userData;
    }

    public void setUserData( String userName, String userPassword )
    {
        userData = database.getUser( userName );
        if ( Objects.nonNull( userData ) )
        {
            Set<ServerData> allServerDatas = new HashSet<>();
            List<ServersGroupData> serversGroupDatas = new ArrayList<>();
            database.removeUserSessionUnactiveApplications( userData.getId() );
            database.getUserApplications( userData )
                    .stream()
                    .filter( Objects::nonNull )
                    .forEach( userApplicationData -> serversGroupDatas.addAll( userApplicationData.getApplicationData()
                                                                                                  .getServersGroupDatas() ) );
            database.getUserApplicationGroups( userData )
                    .stream()
                    .filter( Objects::nonNull )
                    .forEach( userApplicationsGroupData -> userApplicationsGroupData.getApplicationsGroupData()
                                                                                    .getApplicationDatas()
                                                                                    .forEach( applicationData -> serversGroupDatas.addAll( applicationData.getServersGroupDatas() ) ) );
            serversGroupDatas.stream()
                             .filter( Objects::nonNull )
                             .forEach( serversGroupData -> allServerDatas.addAll( serversGroupData.getServerDatas() ) );
            CollectionUtils.filter( allServerDatas, PredicateUtils.notNullPredicate() );
            CollectionUtils.filter( allServerDatas, ServerData::isEnabled );

            boolean userCanLogin = false;
            for ( int index = 0; index < CollectionUtils.size( allServerDatas ) && index < MAX_SERVERS; index++ )
            {
                userCanLogin = virtualUtil.canUserLogin( userName, userPassword, IterableUtils.get( allServerDatas, index ) );
                if ( userCanLogin )
                {
                    break;
                }
            }

            if ( userCanLogin )
            {
                virtualUtil.setUserName( userName );
                virtualUtil.setUserPassword( EncryptionUtil.encrypt( userPassword, virtualUtil.getSecretKeySpec(), virtualUtil.getSecureIV() ) );
            }
            else
            {
                virtualUtil.setUserName( null );
                virtualUtil.setUserPassword( null );
                userData = null;
            }
        }
        else
        {
            LOGGER.error( ERROR_LOGIN );
        }
    }

    public long getSessionTimeout()
    {
        return sessionTimeout;
    }

    public void setSessionTimeout( long sessionTimeout )
    {
        this.sessionTimeout = sessionTimeout;
    }

    @Override
    protected void onDetach( DetachEvent detachEvent )
    {
        if ( Objects.nonNull( canvasUI ) )
        {
            canvasUI.removeAll();
        }

        if ( Objects.nonNull( applicationsBar ) )
        {
            applicationsBar.removeAll();
        }

        if ( Objects.nonNull( barsLayout ) )
        {
            barsLayout.removeAll();
        }

        if ( Objects.nonNull( navigationBar ) )
        {
            navigationBar.removeAll();
        }

        if ( Objects.nonNull( optionsBar ) )
        {
            optionsBar.removeAll();
        }

        if ( Objects.nonNull( poweredByBar ) )
        {
            remove( poweredByBar );
            poweredByBar.removeAll();
            poweredByBar = null;
        }

        if ( Objects.nonNull( runningApplicationsBar ) )
        {
            runningApplicationsBar.removeAll();
        }

        if ( Objects.nonNull( windowsBar ) )
        {
            windowsBar.removeAll();
        }

        virtualUtil.stopAllStartedServices();
        database.close();
        removeAll();
        detachEvent.getUI()
                   .getPage()
                   .executeJs( Command.JS.RESET );
    }

    @ClientCallable
    public void setAllWindowsVisible()
    {
        if ( Objects.nonNull( virtualClient ) && Objects.nonNull( windowsBar ) )
        {
            windowsBar.setAllWindowsVisible();
            virtualClient.getWindows()
                         .forEach( window -> window.setVisible( true ) );
            if ( Objects.nonNull( virtualClient.getConnector() ) )
            {
                DialogConnectingUtil connectingUtil = virtualClient.getConnector()
                                                                   .getDialogConnectingUtil();
                if ( Objects.nonNull( connectingUtil ) )
                {
                    connectingUtil.close();
                }
            }
        }
    }

    @ClientCallable
    public void setBrowserClosed()
    {
        closeBrowser();
    }

    @ClientCallable
    public void setBrowserHidden( Boolean hidden )
    {
        if ( Objects.nonNull( virtualClient ) && Objects.nonNull( hidden ) && Objects.nonNull( virtualClient.getSender() ) )
        {
            if ( Boolean.TRUE.equals( hidden ) )
            {
                virtualClient.getSender()
                             .onSend( new ClientSuspendPacket() );
            }
            else
            {
                virtualClient.getSender()
                             .onSend( new ClientResumePacket() );
            }
        }
    }

    @ClientCallable
    public void setClientProperties( Integer width, Integer height, Integer dpi, String colorGamut, Boolean webpEnabled, Boolean windowsOS, Boolean macOS )
    {
        if ( Objects.nonNull( width ) && Objects.nonNull( height ) && Objects.nonNull( dpi ) && Objects.nonNull( webpEnabled ) && Objects.nonNull( windowsOS ) && Objects.nonNull( macOS ) )
        {
            clientProperties.setClientProperties( width, height, dpi, colorGamut, webpEnabled, windowsOS, macOS );
            virtualUtil.setDesktopWidth( clientProperties.getWidth() );
            virtualUtil.setDesktopHeight( clientProperties.getHeight() );
        }
    }

    @ClientCallable
    public void setClipboardToken( String clipboardData )
    {
        if ( Objects.nonNull( virtualClient ) && Objects.nonNull( clipboardData ) && Objects.nonNull( virtualClient.getSender() ) )
        {
            virtualClient.getSender()
                         .onSend( new ClientClipboardTokenPacket( clipboardData ) );
        }
    }

    @ClientCallable
    public void setDamageSequence( Integer id, Integer width, Integer height, Integer sequence, Integer frameTime )
    {
        if ( Objects.nonNull( virtualClient ) && Objects.nonNull( virtualUtil ) && Objects.nonNull( id ) && Objects.nonNull( width ) && Objects.nonNull( height ) && Objects.nonNull( sequence )
            && Objects.nonNull( frameTime ) && Objects.nonNull( virtualClient.getSender() ) )
        {
            // send ACK packet to the server
            virtualClient.getSender()
                         .onSend( new ClientDamageSequencePacket( id, width, height, sequence, frameTime, StringUtils.EMPTY ) );
            virtualClient.setUpdateEnabled( true );
            if ( Objects.nonNull( virtualClient.getWindow( id ) ) )
            {
                virtualClient.getWindow( id )
                             .setDrawingStarted( true );
            }
            if ( virtualUtil.isNotDrawingStarted() )
            {
                virtualUtil.setDrawingStarted( virtualClient.isDrawingStartedInAllDialogOrNormalWindows() );
                if ( virtualUtil.isDrawingStarted() )
                {
                    virtualUtil.setRecreateNewWindowAgainEnabled( false );
                    virtualUtil.setRecreateNewWindowEnabled( false );
                }
            }
        }
    }

    @ClientCallable
    public void setDesktopSize( Integer width, Integer height )
    {
        if ( Objects.nonNull( virtualClient ) && Objects.nonNull( width ) && Objects.nonNull( height ) && Objects.nonNull( virtualClient.getSender() ) )
        {
            clientProperties.setClientProperties( width, height, clientProperties.getDpi(), clientProperties.getColorGamut(), clientProperties.isWebpEnabled(), clientProperties.isWindowsOS(),
                                                  clientProperties.isMacOS() );
            virtualClient.getSender()
                         .onSend( new ClientDesktopSizePacket( width, height, clientProperties.getDpi() ) );
            virtualUtil.setDesktopWidth( clientProperties.getWidth() );
            virtualUtil.setDesktopHeight( clientProperties.getHeight() );

            UserSessionData userSessionData = database.getUserSession( runningApplicationsBar.getActiveRunningApplicationId() );
            if ( Objects.nonNull( userSessionData ) )
            {
                userSessionData.setDesktopWidth( clientProperties.getWidth() );
                userSessionData.setDesktopHeight( clientProperties.getHeight() );
                userSessionData.setUpdated( new Date() );
                database.updateUserSession( userSessionData );
            }
        }
    }

    @ClientCallable
    public void setFullScreen( Boolean fullScreen )
    {
        if ( Objects.nonNull( fullScreen ) )
        {
            if ( Objects.nonNull( navigationBar ) )
            {
                applicationAndNavigatiobBarHeight = fullScreen.booleanValue() ? navigationBar.getNavigationBarHeight() : 0;
                navigationBar.setFullScreen( fullScreen );
            }
            if ( Objects.nonNull( optionsBar ) )
            {
                optionsBar.setFullScreen( fullScreen );
            }
            if ( Objects.nonNull( poweredByBar ) )
            {
                poweredByBar.setFullScreen( fullScreen );
            }
        }
    }

    @ClientCallable
    public void setFullScreenStatus( Boolean fullScreenStatus )
    {
        if ( Objects.nonNull( optionsBar ) && Objects.nonNull( fullScreenStatus ) )
        {
            optionsBar.setFullScreenStatus( fullScreenStatus );
        }
    }

    @ClientCallable
    public void setKeyboardAndMouseProperties( String capsLock, String numLock, String mouseWheelEventId )
    {
        keyboardEventProcessor.setCapsLock( StringUtils.isNotBlank( capsLock ) ? Boolean.parseBoolean( capsLock ) : null );
        keyboardEventProcessor.setNumLock( StringUtils.isNotBlank( numLock ) ? Boolean.parseBoolean( numLock ) : null );
        mouseEventProcessor.setMouseWheelEventId( mouseWheelEventId );

        if ( !keyboardAndMouseEventsProcessorsRegistered )
        {
            keyboardAndMouseEventsProcessorsRegistered = true;
            registerKeyboardEventsProcessor( UI.getCurrent() );
            registerMouseEventsProcessor();
        }
    }

    @ClientCallable
    public void setKeyClick( Integer id, Integer keyCode, Integer keyValue, String keyName, String keyString, String modifiers, Boolean keyPressed1, Boolean keyPressed2 )
    {
        if ( Objects.nonNull( virtualClient ) && Objects.nonNull( id ) && Objects.nonNull( keyCode ) && Objects.nonNull( keyValue ) && Objects.nonNull( keyName ) && Objects.nonNull( keyString )
            && Objects.nonNull( modifiers ) && Objects.nonNull( keyPressed1 ) && Objects.nonNull( virtualClient.getSender() ) )
        {
            virtualClient.getSender()
                         .onSend( new ClientKeyActionPacket( id, keyCode, keyValue, keyName, keyString, Arrays.asList( modifiers, MODIFIERS_SPLITTER ), keyPressed1 ) );
            if ( Objects.nonNull( keyPressed2 ) )
            {
                virtualClient.getSender()
                             .onSend( new ClientKeyActionPacket( id, keyCode, keyValue, keyName, keyString, Arrays.asList( modifiers, MODIFIERS_SPLITTER ), keyPressed2 ) );
            }
        }

        // session time
        if ( Objects.nonNull( checkBrowserHealthRunner ) )
        {
            Calendar calendar = Calendar.getInstance();
            checkBrowserHealthRunner.setSessionTime( calendar.getTimeInMillis() );
        }
    }

    @ClientCallable
    public void setPingFromBrowser( String canvasId )
    {
        if ( Objects.nonNull( checkBrowserHealthRunner ) && Objects.nonNull( canvasUI ) && StringUtils.equals( canvasUI.getCanvasId(), canvasId ) )
        {
            Calendar calendar = Calendar.getInstance();
            checkBrowserHealthRunner.setBrowserTime( calendar.getTimeInMillis() );
        }
    }

    @ClientCallable
    public void setWindowClosed( Integer id )
    {
        if ( Objects.nonNull( virtualClient ) && Objects.nonNull( id ) && Objects.nonNull( virtualClient.getSender() ) )
        {
            Window window = virtualClient.getWindow( id );
            if ( Objects.nonNull( window ) )
            {
                // close all dialog windows first
                virtualClient.getWindows()
                             .stream()
                             .filter( w -> virtualClient.isDialogWindow( w ) )
                             .filter( w -> !Objects.equals( w.getId(), window.getId() ) )
                             .forEach( w -> virtualClient.getSender()
                                                         .onSend( new ClientCloseWindowPacket( w.getId() ) ) );

                // close normal windows with override redirect status (those windows were created at drag/drop of some part of window )
                virtualClient.getWindows()
                             .stream()
                             .filter( w -> virtualClient.isNormalWindowWithOverrideRedirect( w ) )
                             .filter( w -> !Objects.equals( w.getId(), window.getId() ) )
                             .collect( Collectors.toList() )
                             .forEach( w -> virtualClient.getSender()
                                                         .onSend( new ClientCloseWindowPacket( w.getId() ) ) );
                // close window
                virtualClient.setWindowClosedByUser( true );
                virtualClient.getSender()
                             .onSend( new ClientCloseWindowPacket( id ) );
            }
        }
    }

    @ClientCallable
    public void setWindowDimensions( Integer id, Integer x, Integer y, Integer width, Integer height )
    {
        if ( Objects.nonNull( virtualClient ) && Objects.nonNull( id ) && Objects.nonNull( x ) && Objects.nonNull( y ) && Objects.nonNull( width ) && Objects.nonNull( height )
            && Objects.nonNull( virtualClient.getSender() ) )
        {
            Window window = virtualClient.getWindow( id );
            if ( Objects.nonNull( window ) )
            {
                if ( !window.isMaximized() || Objects.isNull( window.getRestoredWindowDimension() ) )
                {
                    window.setRestoredWindowDimension( x, y, width == 0 ? ClientProperties.DEFAULT_WIDTH : width, height == 0 ? ClientProperties.DEFAULT_HEIGHT : height );
                }
                window.setX( x );
                window.setY( y );
                window.setWidth( width );
                window.setHeight( height );
                virtualClient.getSender()
                             .onSend( new ClientConfigureWindowPacket( id, x, y, width, height ) );
            }
        }
    }

    @ClientCallable
    public void setWindowDimensionsProperties( Integer id, Double dx, Double dy, Double titleHeight, Double offsetLeft, Double offsetTop, Double borderWidth )
    {
        if ( Objects.nonNull( virtualClient ) && Objects.nonNull( id ) && Objects.nonNull( dx ) && Objects.nonNull( dy ) && Objects.nonNull( titleHeight ) && Objects.nonNull( offsetLeft )
            && Objects.nonNull( offsetTop ) && Objects.nonNull( borderWidth ) )
        {
            Window window = virtualClient.getWindow( id );
            if ( Objects.nonNull( window ) )
            {
                window.setDx( dx );
                window.setDy( dy );
                window.setTitleHeight( titleHeight );
                window.setOffsetLeft( offsetLeft );
                window.setOffsetTop( offsetTop );
                window.setBorderWidth( borderWidth );
            }
        }
    }

    @ClientCallable
    public void setWindowFocus( Integer id )
    {
        if ( Objects.nonNull( virtualClient ) && Objects.nonNull( id ) && Objects.nonNull( virtualClient.getSender() ) )
        {
            Window window = virtualClient.getWindow( id );
            if ( Objects.nonNull( window ) && window.isVisible() && virtualClient.isNormalWindow( window ) )
            {
                UserSessionData userSessionData = database.getUserSession( runningApplicationsBar.getActiveRunningApplicationId() );
                if ( Objects.nonNull( userSessionData ) )
                {
                    String noMaximized = String.format( MAXIMIZED_NO_INDICATOR, id );
                    String yesMaximized = String.format( MAXIMIZED_YES_INDICATOR, id );
                    String windowsStatus = StringUtils.defaultString( userSessionData.getStatus(), StringUtils.EMPTY );
                    windowsStatus = StringUtils.remove( windowsStatus, noMaximized );
                    windowsStatus = StringUtils.remove( windowsStatus, yesMaximized );
                    windowsStatus = StringUtils.join( windowsStatus, window.isMaximized() ? yesMaximized : noMaximized );
                    if ( !StringUtils.equals( userSessionData.getStatus(), windowsStatus ) )
                    {
                        userSessionData.setStatus( windowsStatus );
                        userSessionData.setUpdated( new Date() );
                        database.updateUserSession( userSessionData );
                    }
                    virtualUtil.setWindowsFocus( windowsStatus );
                    windowFocus = id;
                }
            }
            virtualClient.getSender()
                         .onSend( new ClientFocusPacket( id ) );
        }
    }

    @ClientCallable
    public void setWindowMaximized( Integer id, Integer width, Integer height, Boolean windowMaximized )
    {
        if ( Objects.nonNull( virtualClient ) && Objects.nonNull( id ) && Objects.nonNull( width ) && Objects.nonNull( height ) && Objects.nonNull( windowMaximized ) )
        {
            Window window = virtualClient.getWindow( id );
            if ( Objects.nonNull( window ) && virtualClient.isNormalWindow( window ) )
            {
                window.setMaximized( windowMaximized );
                UserSessionData userSessionData = database.getUserSession( runningApplicationsBar.getActiveRunningApplicationId() );
                if ( Objects.nonNull( userSessionData ) )
                {
                    String noMaximized = String.format( MAXIMIZED_NO_INDICATOR, id );
                    String yesMaximized = String.format( MAXIMIZED_YES_INDICATOR, id );
                    String windowsStatus = StringUtils.defaultString( userSessionData.getStatus(), StringUtils.EMPTY );
                    windowsStatus = StringUtils.remove( windowsStatus, noMaximized );
                    windowsStatus = StringUtils.remove( windowsStatus, yesMaximized );
                    windowsStatus = StringUtils.join( windowsStatus, window.isMaximized() ? yesMaximized : noMaximized );
                    if ( !StringUtils.equals( userSessionData.getStatus(), windowsStatus ) )
                    {
                        userSessionData.setStatus( windowsStatus );
                        userSessionData.setUpdated( new Date() );
                        database.updateUserSession( userSessionData );
                    }
                    virtualUtil.setWindowsFocus( windowsStatus );
                }

                WindowDimension windowDimension = window.getRestoredWindowDimension();
                if ( window.isMaximized() || Objects.isNull( windowDimension ) )
                {
                    setWindowDimensions( id, 0, 0, width, height );
                }
                else
                {
                    setWindowDimensions( id, windowDimension.getX(), windowDimension.getY(), windowDimension.getWidth(), windowDimension.getHeight() );
                }
            }
        }
    }

    @ClientCallable
    public void setWindowPosition( Integer id, Integer x, Integer y )
    {
        if ( Objects.nonNull( virtualClient ) && Objects.nonNull( id ) && Objects.nonNull( x ) && Objects.nonNull( y ) )
        {
            Window window = virtualClient.getWindow( id );
            if ( Objects.nonNull( window ) )
            {
                setWindowDimensions( id, x, y, window.getWidth(), window.getHeight() );
            }
        }
    }

}
