/**
 * Copyright (C) 2020-2022 Dušan Vejnović <vaadin@dussan.org>
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.dussan.vaadin.dvirtual.ui.servers;

import java.util.ArrayList;
import java.util.EnumMap;
import java.util.List;
import java.util.Map;
import java.util.Objects;

import org.apache.commons.collections4.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.dussan.vaadin.dvirtual.database.Database;
import org.dussan.vaadin.dvirtual.database.columns.ServerColumns;
import org.dussan.vaadin.dvirtual.database.data.ServerData;
import org.dussan.vaadin.dvirtual.i18n.Translation;
import org.dussan.vaadin.dvirtual.runners.TooltipRunner;
import org.dussan.vaadin.dvirtual.utils.VirtualUtil;
import org.dussan.vaadin.dvirtual.utils.dialog.DialogInfoUtil;

import com.vaadin.flow.component.UI;
import com.vaadin.flow.component.button.Button;
import com.vaadin.flow.component.combobox.ComboBox;
import com.vaadin.flow.component.grid.Grid;
import com.vaadin.flow.component.grid.HeaderRow;
import com.vaadin.flow.component.html.Div;
import com.vaadin.flow.component.icon.Icon;
import com.vaadin.flow.component.icon.VaadinIcon;
import com.vaadin.flow.component.orderedlayout.HorizontalLayout;
import com.vaadin.flow.component.orderedlayout.VerticalLayout;
import com.vaadin.flow.component.textfield.TextField;
import com.vaadin.flow.data.provider.DataProvider;
import com.vaadin.flow.data.provider.SortDirection;
import com.vaadin.flow.data.provider.SortOrder;
import com.vaadin.flow.data.renderer.ComponentRenderer;
import com.vaadin.flow.data.value.ValueChangeMode;

public class ServersUI
    extends VerticalLayout
{

    private static final long serialVersionUID = 8660665089111540952L;

    private static final int FILTER_VALUE_CHANGE_TIMEOUT = 500;

    private static final int VISIBLE_ROWS = 50;

    private static final long NO_ELEMENTS = 0;

    private static final String CLASS_SETTINGS = "settings";

    private static final String STYLE_COLOR = "color";

    private static final String STYLE_COLOR_RED_VALUE = "red";

    private static final String STYLE_DISPLAY = "display";

    private static final String STYLE_DISPLAY_VALUE = "grid";

    private static final String STYLE_GRID_COMBOBOX = "grid-combobox";

    private static final String STYLE_HEIGHT = "height";

    private static final String STYLE_HEIGHT_16PX_VALUE = "16px";

    private static final String STYLE_TEXT_ALIGN = "text-align";

    private static final String STYLE_TEXT_ALIGN_VALUE = "center";

    private static final String STYLE_MARGIN_LEFT = "margin-left";

    private static final String STYLE_MARGIN_RIGHT = "margin-right";

    private static final String STYLE_MARGIN_VALUE = "1px";

    private static final String STYLE_WIDTH = "width";

    private static final String STYLE_WIDTH_100PX_VALUE = "100px";

    private static final String STYLE_WIDTH_130PX_VALUE = "130px";

    private static final String STYLE_WIDTH_150PX_VALUE = "150px";

    private static final String STYLE_WIDTH_16PX_VALUE = "16px";

    private transient List<ServerData> serverDatas = null;

    private transient Map<ServerColumns, String> filterMap = null;

    private Database database = null;

    private DataProvider<ServerData, Void> dataProvider = null;

    private Grid<ServerData> serverGrid = null;

    private Grid.Column<ServerData> editDeleteColumn = null;

    private VirtualUtil virtualUtil = null;

    private ServersUI()
    {
        setMargin( false );
        setSpacing( false );
        setSizeFull();
        addClassName( CLASS_SETTINGS );
        filterMap = new EnumMap<>( ServerColumns.class );
    }

    public ServersUI( Database database, VirtualUtil virtualUtil )
    {
        this();
        this.database = database;
        this.virtualUtil = virtualUtil;

        dataProvider = DataProvider.fromCallbacks( query -> {
            int offset = query.getOffset();
            int limit = query.getLimit();

            List<String> orders = new ArrayList<>();
            for ( SortOrder<String> order : query.getSortOrders() )
            {
                orders.add( SortDirection.DESCENDING.equals( order.getDirection() ) ? Database.ORDER_DESCENDING + order.getSorted() : order.getSorted() );
            }

            return new ArrayList<>( this.database.getServers( offset, limit, filterMap, orders ) ).stream();
        }, query -> database.getServerRows( filterMap )
                            .intValue() );

        serverDatas = new ArrayList<>( database.getAllServers() );
        serverGrid = new Grid<>();
        serverGrid.setDataProvider( dataProvider );
        serverGrid.setPageSize( VISIBLE_ROWS );
        serverGrid.getStyle()
                  .set( STYLE_DISPLAY, STYLE_DISPLAY_VALUE )
                  .set( STYLE_MARGIN_LEFT, STYLE_MARGIN_VALUE )
                  .set( STYLE_MARGIN_RIGHT, STYLE_MARGIN_VALUE );
        serverGrid.addItemDoubleClickListener( event -> {
            serverGrid.select( event.getItem() );
            editServerData( event.getItem() ).open();
        } );
        add( serverGrid );

        // name
        Grid.Column<ServerData> nameColumn = serverGrid.addColumn( ServerData::getName )
                                                       .setFlexGrow( 1 )
                                                       .setFrozen( true )
                                                       .setResizable( true )
                                                       .setSortable( true )
                                                       .setSortProperty( ServerColumns.NAME.getName() )
                                                       .setWidth( STYLE_WIDTH_150PX_VALUE )
                                                       .setHeader( virtualUtil.getTranslation( ServerColumns.NAME.getValueForTranslation() ) );
        // note
        Grid.Column<ServerData> noteColumn = serverGrid.addColumn( ServerData::getNote )
                                                       .setFlexGrow( 3 )
                                                       .setResizable( true )
                                                       .setSortable( true )
                                                       .setSortProperty( ServerColumns.NOTE.getName() )
                                                       .setHeader( virtualUtil.getTranslation( ServerColumns.NOTE.getValueForTranslation() ) );
        // address
        Grid.Column<ServerData> addressColumn = serverGrid.addColumn( ServerData::getAddress )
                                                          .setFlexGrow( 1 )
                                                          .setResizable( true )
                                                          .setSortable( true )
                                                          .setWidth( STYLE_WIDTH_150PX_VALUE )
                                                          .setSortProperty( ServerColumns.ADDRESS.getName() )
                                                          .setHeader( virtualUtil.getTranslation( ServerColumns.ADDRESS.getValueForTranslation() ) );
        // SSH port
        Grid.Column<ServerData> sshPortColumn = serverGrid.addColumn( ServerData::getSshPort )
                                                          .setFlexGrow( 0 )
                                                          .setResizable( true )
                                                          .setSortable( true )
                                                          .setSortProperty( ServerColumns.SSH_PORT.getName() )
                                                          .setWidth( STYLE_WIDTH_130PX_VALUE )
                                                          .setHeader( virtualUtil.getTranslation( ServerColumns.SSH_PORT.getValueForTranslation() ) );
        // XPRA min port
        Grid.Column<ServerData> xpraMinPortColumn = serverGrid.addColumn( ServerData::getXpraMinPort )
                                                              .setFlexGrow( 0 )
                                                              .setResizable( true )
                                                              .setSortable( true )
                                                              .setSortProperty( ServerColumns.XPRA_MIN_PORT.getName() )
                                                              .setWidth( STYLE_WIDTH_130PX_VALUE )
                                                              .setHeader( virtualUtil.getTranslation( ServerColumns.XPRA_MIN_PORT.getValueForTranslation() ) );
        // XPRA max port
        Grid.Column<ServerData> xpraMaxPortColumn = serverGrid.addColumn( ServerData::getXpraMaxPort )
                                                              .setFlexGrow( 0 )
                                                              .setResizable( true )
                                                              .setSortable( true )
                                                              .setSortProperty( ServerColumns.XPRA_MAX_PORT.getName() )
                                                              .setWidth( STYLE_WIDTH_130PX_VALUE )
                                                              .setHeader( virtualUtil.getTranslation( ServerColumns.XPRA_MAX_PORT.getValueForTranslation() ) );
        // XPRA version
        Grid.Column<ServerData> xpraVersionColumn = serverGrid.addColumn( ServerData::getXpraVersion )
                                                              .setFlexGrow( 0 )
                                                              .setResizable( true )
                                                              .setSortable( true )
                                                              .setSortProperty( ServerColumns.XPRA_VERSION.getName() )
                                                              .setWidth( STYLE_WIDTH_130PX_VALUE )
                                                              .setHeader( virtualUtil.getTranslation( ServerColumns.XPRA_VERSION.getValueForTranslation() ) );
        // XPRA max users
        Grid.Column<ServerData> maxUsersColumn = serverGrid.addColumn( ServerData::getMaxUsers )
                                                           .setFlexGrow( 0 )
                                                           .setResizable( true )
                                                           .setSortable( true )
                                                           .setSortProperty( ServerColumns.MAX_USERS.getName() )
                                                           .setWidth( STYLE_WIDTH_130PX_VALUE )
                                                           .setHeader( virtualUtil.getTranslation( ServerColumns.MAX_USERS.getValueForTranslation() ) );
        // enabled
        Grid.Column<ServerData> enabledColumn = serverGrid.addColumn( new ComponentRenderer<>( Div::new, ( div, serverData ) -> {
            Icon icon = serverData.isEnabled() ? new Icon( VaadinIcon.CHECK ) : new Icon( VaadinIcon.CLOSE );
            icon.getStyle()
                .set( STYLE_HEIGHT, STYLE_HEIGHT_16PX_VALUE )
                .set( STYLE_WIDTH, STYLE_WIDTH_16PX_VALUE );
            div.add( icon );
            div.getStyle()
               .set( STYLE_TEXT_ALIGN, STYLE_TEXT_ALIGN_VALUE );
        } ) )
                                                          .setFlexGrow( 0 )
                                                          .setResizable( true )
                                                          .setSortable( true )
                                                          .setSortProperty( ServerColumns.ENABLED.getName() )
                                                          .setWidth( STYLE_WIDTH_130PX_VALUE )
                                                          .setHeader( virtualUtil.getTranslation( ServerColumns.ENABLED.getValueForTranslation() ) );
        // edit/delete data
        editDeleteColumn = serverGrid.addColumn( new ComponentRenderer<>( serverData -> {
            // delete
            Button deleteButton = new Button( new Icon( VaadinIcon.TRASH ), event -> {
                boolean noElements = database.getServersGroupServerRows( serverData ) == NO_ELEMENTS;
                String message = noElements ? virtualUtil.getTranslation( Translation.SERVER_REMOVE, serverData.getName() )
                                : virtualUtil.getTranslation( Translation.SERVER_REMOVE_SERVERS_GROUP, serverData.getName() );
                ServerRemoveDialog dialog = new ServerRemoveDialog( message, virtualUtil )
                {
                    private static final long serialVersionUID = -3863278710881267810L;

                    @Override
                    public void savingData()
                    {
                        removeServer( serverData );
                    }
                };
                dialog.open();
                serverGrid.select( serverData );
            } );
            deleteButton.getStyle()
                        .set( STYLE_COLOR, STYLE_COLOR_RED_VALUE );

            // edit
            Button editButton = new Button( new Icon( VaadinIcon.EDIT ), event -> {
                serverGrid.select( serverData );
                editServerData( serverData ).open();
            } );

            if ( virtualUtil.isTooltipsVisible() )
            {
                TooltipRunner tooltipRunner1 = new TooltipRunner( deleteButton, virtualUtil.getTranslation( Translation.BUTTON_DELETE ), UI.getCurrent() );
                virtualUtil.startRunnerService( tooltipRunner1 );
                TooltipRunner tooltipRunner2 = new TooltipRunner( editButton, virtualUtil.getTranslation( Translation.BUTTON_EDIT ), UI.getCurrent() );
                virtualUtil.startRunnerService( tooltipRunner2 );
            }

            HorizontalLayout layout = new HorizontalLayout( editButton, deleteButton );
            layout.setMargin( false );
            layout.setPadding( false );
            layout.setSpacing( false );
            layout.setSizeUndefined();
            return layout;
        } ) )
                                     .setFlexGrow( 0 )
                                     .setWidth( STYLE_WIDTH_100PX_VALUE )
                                     .setResizable( true );
        editDeleteColumn.setVisible( CollectionUtils.isNotEmpty( serverDatas ) );

        // filter: name
        HeaderRow filterRow = serverGrid.appendHeaderRow();
        TextField nameFilter = new TextField();
        nameFilter.setPlaceholder( virtualUtil.getTranslation( Translation.FIELD_FILTER ) );
        nameFilter.setSizeFull();
        nameFilter.setValueChangeMode( ValueChangeMode.LAZY );
        nameFilter.setValueChangeTimeout( FILTER_VALUE_CHANGE_TIMEOUT );
        nameFilter.addValueChangeListener( event -> {
            if ( nameFilter.isEmpty() )
            {
                filterMap.remove( ServerColumns.NAME );
            }
            else
            {
                filterMap.put( ServerColumns.NAME, nameFilter.getValue() );
            }
            dataProvider.refreshAll();
        } );
        filterRow.getCell( nameColumn )
                 .setComponent( nameFilter );

        // filter: note
        TextField noteFilter = new TextField();
        noteFilter.setPlaceholder( virtualUtil.getTranslation( Translation.FIELD_FILTER ) );
        noteFilter.setSizeFull();
        noteFilter.setValueChangeMode( ValueChangeMode.LAZY );
        noteFilter.setValueChangeTimeout( FILTER_VALUE_CHANGE_TIMEOUT );
        noteFilter.addValueChangeListener( event -> {
            if ( noteFilter.isEmpty() )
            {
                filterMap.remove( ServerColumns.NOTE );
            }
            else
            {
                filterMap.put( ServerColumns.NOTE, noteFilter.getValue() );
            }
            dataProvider.refreshAll();
        } );
        filterRow.getCell( noteColumn )
                 .setComponent( noteFilter );

        // filter: address
        TextField addressFilter = new TextField();
        addressFilter.setPlaceholder( virtualUtil.getTranslation( Translation.FIELD_FILTER ) );
        addressFilter.setSizeFull();
        addressFilter.setValueChangeMode( ValueChangeMode.LAZY );
        addressFilter.setValueChangeTimeout( FILTER_VALUE_CHANGE_TIMEOUT );
        addressFilter.addValueChangeListener( event -> {
            if ( addressFilter.isEmpty() )
            {
                filterMap.remove( ServerColumns.ADDRESS );
            }
            else
            {
                filterMap.put( ServerColumns.ADDRESS, addressFilter.getValue() );
            }
            dataProvider.refreshAll();
        } );
        filterRow.getCell( addressColumn )
                 .setComponent( addressFilter );

        // filter: SSH port
        TextField sshPortFilter = new TextField();
        sshPortFilter.setPlaceholder( virtualUtil.getTranslation( Translation.FIELD_FILTER ) );
        sshPortFilter.setSizeFull();
        sshPortFilter.setValueChangeMode( ValueChangeMode.LAZY );
        sshPortFilter.setValueChangeTimeout( FILTER_VALUE_CHANGE_TIMEOUT );
        sshPortFilter.addValueChangeListener( event -> {
            if ( sshPortFilter.isEmpty() )
            {
                filterMap.remove( ServerColumns.SSH_PORT );
            }
            else
            {
                filterMap.put( ServerColumns.SSH_PORT, sshPortFilter.getValue() );
            }
            dataProvider.refreshAll();
        } );
        filterRow.getCell( sshPortColumn )
                 .setComponent( sshPortFilter );

        // filter: XPRA min port
        TextField xpraMinPortFilter = new TextField();
        xpraMinPortFilter.setPlaceholder( virtualUtil.getTranslation( Translation.FIELD_FILTER ) );
        xpraMinPortFilter.setSizeFull();
        xpraMinPortFilter.setValueChangeMode( ValueChangeMode.LAZY );
        xpraMinPortFilter.setValueChangeTimeout( FILTER_VALUE_CHANGE_TIMEOUT );
        xpraMinPortFilter.addValueChangeListener( event -> {
            if ( xpraMinPortFilter.isEmpty() )
            {
                filterMap.remove( ServerColumns.XPRA_MIN_PORT );
            }
            else
            {
                filterMap.put( ServerColumns.XPRA_MIN_PORT, xpraMinPortFilter.getValue() );
            }
            dataProvider.refreshAll();
        } );
        filterRow.getCell( xpraMinPortColumn )
                 .setComponent( xpraMinPortFilter );

        // filter: XPRA max port
        TextField xpraVersionFilter = new TextField();
        xpraVersionFilter.setPlaceholder( virtualUtil.getTranslation( Translation.FIELD_FILTER ) );
        xpraVersionFilter.setSizeFull();
        xpraVersionFilter.setValueChangeMode( ValueChangeMode.LAZY );
        xpraVersionFilter.setValueChangeTimeout( FILTER_VALUE_CHANGE_TIMEOUT );
        xpraVersionFilter.addValueChangeListener( event -> {
            if ( xpraVersionFilter.isEmpty() )
            {
                filterMap.remove( ServerColumns.XPRA_VERSION );
            }
            else
            {
                filterMap.put( ServerColumns.XPRA_VERSION, xpraVersionFilter.getValue() );
            }
            dataProvider.refreshAll();
        } );
        filterRow.getCell( xpraVersionColumn )
                 .setComponent( xpraVersionFilter );

        // filter: XPRA max port
        TextField xpraMaxPortFilter = new TextField();
        xpraMaxPortFilter.setPlaceholder( virtualUtil.getTranslation( Translation.FIELD_FILTER ) );
        xpraMaxPortFilter.setSizeFull();
        xpraMaxPortFilter.setValueChangeMode( ValueChangeMode.LAZY );
        xpraMaxPortFilter.setValueChangeTimeout( FILTER_VALUE_CHANGE_TIMEOUT );
        xpraMaxPortFilter.addValueChangeListener( event -> {
            if ( xpraMaxPortFilter.isEmpty() )
            {
                filterMap.remove( ServerColumns.XPRA_MAX_PORT );
            }
            else
            {
                filterMap.put( ServerColumns.XPRA_MAX_PORT, xpraMaxPortFilter.getValue() );
            }
            dataProvider.refreshAll();
        } );
        filterRow.getCell( xpraMaxPortColumn )
                 .setComponent( xpraMaxPortFilter );

        // filter: maximum users
        TextField maxUsersFilter = new TextField();
        maxUsersFilter.setPlaceholder( virtualUtil.getTranslation( Translation.FIELD_FILTER ) );
        maxUsersFilter.setSizeFull();
        maxUsersFilter.setValueChangeMode( ValueChangeMode.LAZY );
        maxUsersFilter.setValueChangeTimeout( FILTER_VALUE_CHANGE_TIMEOUT );
        maxUsersFilter.addValueChangeListener( event -> {
            if ( maxUsersFilter.isEmpty() )
            {
                filterMap.remove( ServerColumns.MAX_USERS );
            }
            else
            {
                filterMap.put( ServerColumns.MAX_USERS, maxUsersFilter.getValue() );
            }
            dataProvider.refreshAll();
        } );
        filterRow.getCell( maxUsersColumn )
                 .setComponent( maxUsersFilter );

        // filter: enabled
        String filterAll = virtualUtil.getTranslation( Translation.DATA_FILTER_ALL );
        String filterNo = virtualUtil.getTranslation( Translation.DATA_FILTER_NO );
        String filterYes = virtualUtil.getTranslation( Translation.DATA_FILTER_YES );
        ComboBox<String> enabledFilter = new ComboBox<>();
        enabledFilter.setAllowCustomValue( false );
        enabledFilter.setClassName( STYLE_GRID_COMBOBOX );
        enabledFilter.setItems( filterAll, filterYes, filterNo );
        enabledFilter.setValue( filterAll );
        enabledFilter.setSizeFull();
        enabledFilter.addValueChangeListener( event -> {
            if ( StringUtils.equals( enabledFilter.getValue(), filterYes ) )
            {
                filterMap.put( ServerColumns.ENABLED, Boolean.TRUE.toString() );
            }
            else if ( StringUtils.equals( enabledFilter.getValue(), filterNo ) )
            {
                filterMap.put( ServerColumns.ENABLED, Boolean.FALSE.toString() );
            }
            else
            {
                filterMap.remove( ServerColumns.ENABLED );
            }
            dataProvider.refreshAll();
        } );
        filterRow.getCell( enabledColumn )
                 .setComponent( enabledFilter );
    }

    private ServerEditDialog editServerData( ServerData serverData )
    {
        return ( new ServerEditDialog( serverData.getId(), serverData, virtualUtil )
        {
            private static final long serialVersionUID = -7843033096057035322L;

            @Override
            public void savingData( ServerData data )
            {
                if ( Objects.nonNull( data ) )
                {
                    boolean closeDialog = true;
                    if ( Objects.nonNull( database.getServer( data.getId() ) ) )
                    {
                        closeDialog = updateServer( data );
                    }
                    else
                    {
                        closeDialog = addServer( data );
                    }

                    if ( closeDialog )
                    {
                        close();
                    }
                }
                else
                {
                    new DialogInfoUtil( virtualUtil.getTranslation( Translation.DATA_ERROR_SAVE ), virtualUtil.getTranslation( Translation.BUTTON_CLOSE ) ).open();
                }
            }
        } );
    }

    public boolean addServer( ServerData serverData )
    {
        if ( Objects.nonNull( serverData ) )
        {
            serverData.setId( database.addServer( serverData ) );
            if ( !Objects.equals( serverData.getId(), ServerData.UNDEFINED_SERVER ) )
            {
                serverDatas.add( serverData );
                dataProvider.refreshAll();
                serverGrid.select( serverData );
                editDeleteColumn.setVisible( CollectionUtils.isNotEmpty( serverDatas ) );
                return true;
            }
            else
            {
                new DialogInfoUtil( virtualUtil.getTranslation( Translation.SERVER_ERROR_ADD ), virtualUtil.getTranslation( Translation.BUTTON_CLOSE ) ).open();
            }
        }
        else
        {
            new DialogInfoUtil( virtualUtil.getTranslation( Translation.DATA_ERROR_SAVE ), virtualUtil.getTranslation( Translation.BUTTON_CLOSE ) ).open();
        }
        return false;
    }

    public boolean addServer( int id, String name, String note, String address, int sshPort, int xpraMin, int xpraMax, String xpraVersion, int maxUSers, boolean enabled )
    {
        return addServer( new ServerData( id, name, note, address, sshPort, xpraMin, xpraMax, xpraVersion, maxUSers, enabled ) );
    }

    public boolean removeServer( ServerData serverData )
    {
        if ( Objects.nonNull( serverData ) )
        {
            if ( database.removeServer( serverData ) )
            {
                serverGrid.deselectAll();
                serverDatas.remove( serverData );
                dataProvider.refreshAll();
                editDeleteColumn.setVisible( CollectionUtils.isNotEmpty( serverDatas ) );
                return true;
            }
            else
            {
                new DialogInfoUtil( virtualUtil.getTranslation( Translation.SERVER_ERROR_REMOVE ), virtualUtil.getTranslation( Translation.BUTTON_CLOSE ) ).open();
            }
        }
        else
        {
            new DialogInfoUtil( virtualUtil.getTranslation( Translation.DATA_ERROR_SAVE ), virtualUtil.getTranslation( Translation.BUTTON_CLOSE ) ).open();
        }
        return false;
    }

    public boolean removeServer( int serverId )
    {
        return removeServer( database.getServer( serverId ) );
    }

    public boolean updateServer( ServerData serverData )
    {
        if ( Objects.nonNull( serverData ) )
        {
            if ( database.updateServer( serverData ) )
            {
                serverDatas.removeIf( data -> Objects.equals( data.getId(), serverData.getId() ) );
                serverDatas.add( serverData );
                dataProvider.refreshAll();
                serverGrid.select( serverData );
                editDeleteColumn.setVisible( CollectionUtils.isNotEmpty( serverDatas ) );
                return true;
            }
            else
            {
                new DialogInfoUtil( virtualUtil.getTranslation( Translation.DATA_ERROR_SAVE, serverData.getName() ), virtualUtil.getTranslation( Translation.BUTTON_CLOSE ) ).open();
            }
        }
        else
        {
            new DialogInfoUtil( virtualUtil.getTranslation( Translation.DATA_ERROR_SAVE ), virtualUtil.getTranslation( Translation.BUTTON_CLOSE ) ).open();
        }
        return false;
    }

    public boolean updateServer( int id, String name, String note, String address, int sshPort, int xpraMin, int xpraMax, String xpraVersion, int maxUSers, boolean enabled )
    {
        return updateServer( new ServerData( id, name, note, address, sshPort, xpraMin, xpraMax, xpraVersion, maxUSers, enabled ) );
    }

    public void addNewServer()
    {
        ServerEditDialog dialog = new ServerEditDialog( ServerData.NEW_SERVER, new ServerData(), virtualUtil )
        {
            private static final long serialVersionUID = 9195546423011060539L;

            @Override
            public void savingData( ServerData serverData )
            {
                if ( Objects.nonNull( serverData ) )
                {
                    serverData.setId( database.addServer( serverData ) );
                    if ( !Objects.equals( serverData.getId(), ServerData.UNDEFINED_SERVER ) )
                    {
                        serverDatas.add( serverData );
                        dataProvider.refreshAll();
                        serverGrid.select( serverData );
                        editDeleteColumn.setVisible( CollectionUtils.isNotEmpty( serverDatas ) );
                        close();
                    }
                    else
                    {
                        new DialogInfoUtil( virtualUtil.getTranslation( Translation.SERVER_ERROR_EXISTS, serverData.getName() ), virtualUtil.getTranslation( Translation.BUTTON_CLOSE ) ).open();
                    }
                }
                else
                {
                    new DialogInfoUtil( virtualUtil.getTranslation( Translation.DATA_ERROR_SAVE ), virtualUtil.getTranslation( Translation.BUTTON_CLOSE ) ).open();
                }
            }
        };
        dialog.open();
    }

}
