/**
 * Copyright (C) 2020-2022 Dušan Vejnović <vaadin@dussan.org>
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.dussan.vaadin.dvirtual.utils.dialog;

import java.io.ByteArrayInputStream;
import java.util.ArrayList;
import java.util.List;
import java.util.Objects;
import java.util.stream.Collectors;

import org.apache.commons.lang3.StringUtils;
import org.dussan.vaadin.dvirtual.database.data.IconData;
import org.dussan.vaadin.dvirtual.database.data.UserSessionData;
import org.dussan.vaadin.dvirtual.i18n.Translation;
import org.dussan.vaadin.dvirtual.utils.VirtualUtil;
import org.dussan.vaadin.dvirtual.utils.dialog.common.DialogUtil;

import com.vaadin.flow.component.button.Button;
import com.vaadin.flow.component.grid.Grid;
import com.vaadin.flow.component.html.Div;
import com.vaadin.flow.component.html.Image;
import com.vaadin.flow.component.html.Label;
import com.vaadin.flow.component.icon.IronIcon;
import com.vaadin.flow.component.orderedlayout.FlexComponent.JustifyContentMode;
import com.vaadin.flow.component.orderedlayout.HorizontalLayout;
import com.vaadin.flow.component.orderedlayout.VerticalLayout;
import com.vaadin.flow.data.provider.ListDataProvider;
import com.vaadin.flow.data.renderer.ComponentRenderer;
import com.vaadin.flow.server.StreamResource;

public abstract class DialogUserSessionActivationUtil
    extends DialogUtil
{

    private static final long serialVersionUID = 6416047241427562050L;

    private static final String ATTRIBUTE_THEME = "theme";

    private static final String ATTRIBUTE_THEME_VALUE = "dark";

    private static final String CLASS_ACTIVE_SESSIONS = "active-sessions";

    private static final String CLASS_ICON = "icon";

    private static final String CLASS_LABEL = "label";

    private static final String DIALOG_HEIGHT = "auto";

    private static final String DIALOG_WIDTH = "auto";

    private static final String ICON_NAME_SEPARATOR = ":";

    private static final String INNER_HTML = "innerHTML";

    private static final String STYLE_FONT_SIZE = "font-size";

    private static final String STYLE_FONT_SIZE_VALUE = "var(--lumo-font-size-l)";

    private static final String STYLE_MARGIN_BOTTOM = "margin-bottom";

    private static final String STYLE_MARGIN_LEFT = "margin-left";

    private static final String STYLE_MARGIN_RIGHT = "margin-right";

    private static final String STYLE_MARGIN_VALUE = "0px";

    private static final String STYLE_PADDING_BOTTOM = "padding-bottom";

    private static final String STYLE_PADDING_RIGHT = "padding-right";

    private static final String STYLE_PADDING_VALUE = "0px";

    private static final String STYLE_WIDTH_100PERCENT_VALUE = "100%";

    private static final String STYLE_WIDTH_150PX_VALUE = "150px";

    private Button cancelButton = null;

    public DialogUserSessionActivationUtil( List<UserSessionData> userSessionDatas, VirtualUtil virtualUtil )
    {
        setCloseOnEsc( true );
        setWidth( DIALOG_WIDTH );
        setHeight( DIALOG_HEIGHT );

        Div messageDiv = new Div();
        messageDiv.getElement()
                  .getStyle()
                  .set( STYLE_FONT_SIZE, STYLE_FONT_SIZE_VALUE );
        messageDiv.getElement()
                  .setProperty( INNER_HTML, virtualUtil.getTranslation( Translation.USERS_SESSION_ACTIVE ) );

        List<String> userSessionCanvasIds = new ArrayList<>();
        userSessionDatas.stream()
                        .filter( Objects::nonNull )
                        .forEach( userSessionData -> userSessionCanvasIds.add( userSessionData.getCanvasId() ) );
        List<String> uniqueUserSessionCanvasIds = userSessionCanvasIds.stream()
                                                                      .distinct()
                                                                      .collect( Collectors.toList() );

        ListDataProvider<String> dataProvider = new ListDataProvider<>( uniqueUserSessionCanvasIds );
        Grid<String> userSessionsGrid = new Grid<>();
        userSessionsGrid.setDataProvider( dataProvider );
        userSessionsGrid.addClassName( CLASS_ACTIVE_SESSIONS );
        userSessionsGrid.addItemDoubleClickListener( event -> {
            cancelButton.setEnabled( false );
            activateSession( event.getItem() );
        } );

        // active sessions
        userSessionsGrid.addColumn( new ComponentRenderer<>( Div::new, ( userSessionsDiv, userSessionCanvasId ) -> {
            userSessionDatas.stream()
                            .filter( Objects::nonNull )
                            .filter( userSessionData -> StringUtils.equals( userSessionData.getCanvasId(), userSessionCanvasId ) )
                            .forEach( userSessionData -> {
                                Div userSessionDiv = new Div();
                                IconData iconData = userSessionData.getApplicationData()
                                                                   .getIconData();
                                if ( Objects.nonNull( iconData ) )
                                {
                                    if ( Objects.isNull( iconData.getIcon() ) )
                                    {
                                        String iconName = iconData.getName();
                                        IronIcon ironIcon = new IronIcon( iconName.contains( ICON_NAME_SEPARATOR ) ? iconName.split( ICON_NAME_SEPARATOR )[0] : iconName,
                                                                          iconName.contains( ICON_NAME_SEPARATOR ) ? iconName.split( ICON_NAME_SEPARATOR )[1] : iconName );
                                        ironIcon.addClassName( CLASS_ICON );
                                        userSessionDiv.add( ironIcon );

                                    }
                                    else
                                    {
                                        StreamResource streamResource = new StreamResource( iconData.getName(), () -> new ByteArrayInputStream( iconData.getIcon() ) );
                                        Image image = new Image( streamResource, iconData.getName() );
                                        image.addClassName( CLASS_ICON );
                                        userSessionDiv.add( image );
                                    }
                                }

                                Label label = new Label( userSessionData.getApplicationData()
                                                                        .getName() );
                                label.addClassName( CLASS_LABEL );

                                userSessionDiv.add( label );
                                userSessionsDiv.add( userSessionDiv );
                            } );
        } ) )
                        .setFlexGrow( 1 )
                        .setResizable( false );

        // connect
        userSessionsGrid.addColumn( new ComponentRenderer<>( userSession -> {
            Button connectButton = new Button( virtualUtil.getTranslation( Translation.BUTTON_CONNECT ), event -> {
                cancelButton.setEnabled( false );
                activateSession( userSession );
            } );
            connectButton.setDisableOnClick( true );
            connectButton.setWidth( STYLE_WIDTH_100PERCENT_VALUE );

            HorizontalLayout layout = new HorizontalLayout( connectButton );
            layout.setMargin( false );
            layout.setPadding( false );
            layout.setSpacing( false );
            layout.setSizeUndefined();
            return layout;
        } ) )
                        .setFlexGrow( 0 )
                        .setWidth( STYLE_WIDTH_150PX_VALUE )
                        .setResizable( false );

        // cancel
        cancelButton = new Button( StringUtils.EMPTY, event -> close() );
        cancelButton.setDisableOnClick( true );
        cancelButton.setText( virtualUtil.getTranslation( Translation.BUTTON_CANCEL ) );
        cancelButton.getElement()
                    .setAttribute( ATTRIBUTE_THEME, ATTRIBUTE_THEME_VALUE );

        HorizontalLayout buttonsLayout = new HorizontalLayout( cancelButton );
        buttonsLayout.setJustifyContentMode( JustifyContentMode.CENTER );
        buttonsLayout.setMargin( true );
        buttonsLayout.setPadding( true );
        buttonsLayout.setSpacing( true );
        buttonsLayout.setWidth( STYLE_WIDTH_100PERCENT_VALUE );
        buttonsLayout.getStyle()
                     .set( STYLE_MARGIN_BOTTOM, STYLE_MARGIN_VALUE )
                     .set( STYLE_MARGIN_LEFT, STYLE_MARGIN_VALUE )
                     .set( STYLE_MARGIN_RIGHT, STYLE_MARGIN_VALUE )
                     .set( STYLE_PADDING_BOTTOM, STYLE_PADDING_VALUE )
                     .set( STYLE_PADDING_RIGHT, STYLE_PADDING_VALUE );

        VerticalLayout mainLayout = new VerticalLayout( messageDiv, userSessionsGrid, buttonsLayout );
        mainLayout.setMargin( false );
        mainLayout.setPadding( false );
        mainLayout.setSpacing( true );
        mainLayout.setSizeUndefined();
        mainLayout.setWidth( STYLE_WIDTH_100PERCENT_VALUE );
        add( mainLayout );
    }

    public abstract void activateSession( String sessionId );

}
