/**
 * Copyright (C) 2020-2022 Dušan Vejnović <vaadin@dussan.org>
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.dussan.vaadin.dvirtual.xpra.io.rencode;

import java.io.DataOutput;
import java.io.FilterOutputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.math.BigDecimal;
import java.math.BigInteger;
import java.util.Collection;
import java.util.Map;
import java.util.Objects;
import java.util.Map.Entry;

/**
 * Rencode type codes forked from rencode-java.
 * 
 * @see {@linkplain https://github.com/jksiezni/rencode4j/}
 */
public class RencodeOutputStream
    extends FilterOutputStream
    implements DataOutput
{

    private byte[] buffer = new byte[8];

    public RencodeOutputStream( OutputStream out )
    {
        super( out );
    }

    public void writeBoolean( boolean value )
        throws IOException
    {
        write( value ? Rencode.CHAR_TRUE : Rencode.CHAR_FALSE );
    }

    public void writeByte( int value )
        throws IOException
    {
        if ( 0 <= value && value < Rencode.INT_POS_FIXED_COUNT )
        {
            write( Rencode.INT_POS_FIXED_START + value );
        }
        else if ( -Rencode.INT_NEG_FIXED_COUNT <= value && value < 0 )
        {
            write( Rencode.INT_NEG_FIXED_START - 1 - value );
        }
        else
        {
            write( Rencode.CHAR_INT1 );
            write( value );
        }
    }

    public void writeShort( int value )
        throws IOException
    {
        if ( Byte.MIN_VALUE <= value && value <= Byte.MAX_VALUE )
        {
            writeByte( value );
        }
        else
        {
            write( Rencode.CHAR_INT2 );
            write( ( value >>> 8 ) & 0xFF );
            write( ( value >>> 0 ) & 0xFF );
        }
    }

    public void writeChar( int value )
        throws IOException
    {
        writeShort( value );
    }

    public void writeInt( int value )
        throws IOException
    {
        if ( Short.MIN_VALUE <= value && value <= Short.MAX_VALUE )
        {
            writeShort( value );
        }
        else
        {
            write( Rencode.CHAR_INT4 );
            write( ( value >>> 24 ) & 0xFF );
            write( ( value >>> 16 ) & 0xFF );
            write( ( value >>> 8 ) & 0xFF );
            write( ( value >>> 0 ) & 0xFF );
        }
    }

    public void writeLong( long value )
        throws IOException
    {
        if ( Integer.MIN_VALUE <= value && value <= Integer.MAX_VALUE )
        {
            writeInt( (int) value );
        }
        else
        {
            write( Rencode.CHAR_INT8 );
            buffer[0] = (byte) ( value >>> 56 );
            buffer[1] = (byte) ( value >>> 48 );
            buffer[2] = (byte) ( value >>> 40 );
            buffer[3] = (byte) ( value >>> 32 );
            buffer[4] = (byte) ( value >>> 24 );
            buffer[5] = (byte) ( value >>> 16 );
            buffer[6] = (byte) ( value >>> 8 );
            buffer[7] = (byte) ( value >>> 0 );
            write( buffer, 0, 8 );
        }
    }

    public void writeFloat( float value )
        throws IOException
    {
        write( Rencode.CHAR_FLOAT32 );
        final int val = Float.floatToIntBits( value );
        write( ( val >>> 24 ) & 0xFF );
        write( ( val >>> 16 ) & 0xFF );
        write( ( val >>> 8 ) & 0xFF );
        write( ( val >>> 0 ) & 0xFF );
    }

    public void writeDouble( double value )
        throws IOException
    {
        final long val = Double.doubleToLongBits( value );
        buffer[0] = (byte) ( val >>> 56 );
        buffer[1] = (byte) ( val >>> 48 );
        buffer[2] = (byte) ( val >>> 40 );
        buffer[3] = (byte) ( val >>> 32 );
        buffer[4] = (byte) ( val >>> 24 );
        buffer[5] = (byte) ( val >>> 16 );
        buffer[6] = (byte) ( val >>> 8 );
        buffer[7] = (byte) ( val >>> 0 );
        write( Rencode.CHAR_FLOAT64 );
        write( buffer, 0, 8 );
    }

    public void writeBytes( String value )
        throws IOException
    {
        writeUTF( value );
    }

    public void writeChars( String value )
        throws IOException
    {
        writeUTF( value );
    }

    public void writeUTF( String value )
        throws IOException
    {
        final byte[] bytes = value.getBytes( Rencode.UTF_8 );
        if ( bytes.length < Rencode.STRING_FIXED_COUNT )
        {
            write( Rencode.STRING_FIXED_START + bytes.length );
            write( bytes );
        }
        else
        {
            write( String.valueOf( bytes.length )
                         .getBytes( Rencode.UTF_8 ) );
            write( Rencode.LENGTH_DELIMITER );
            write( bytes );
        }
    }

    public void writeObject( Object value )
        throws IOException
    {
        if ( Objects.isNull( value ) )
        {
            write( Rencode.CHAR_NONE );
        }
        else if ( value instanceof Boolean )
        {
            writeBoolean( (Boolean) value );

        }
        else if ( value instanceof Character )
        {
            writeChar( (Character) value );

        }
        else if ( value instanceof String )
        {
            writeUTF( (String) value );

        }
        else if ( value instanceof Number )
        {
            writeNumber( (Number) value );

        }
        else if ( value instanceof Collection<?> )
        {
            writeCollection( (Collection<?>) value );

        }
        else if ( value instanceof Map<?, ?> )
        {
            writeMap( (Map<?, ?>) value );

        }
    }

    private void writeNumber( Number value )
        throws IOException
    {
        if ( value instanceof Float )
        {
            writeFloat( value.floatValue() );
        }
        else if ( value instanceof Double )
        {
            writeDouble( value.doubleValue() );
        }
        else if ( value instanceof BigDecimal || value instanceof BigInteger )
        {
            final String sval = String.valueOf( value );
            if ( sval.length() >= Rencode.MAX_INT_LENGTH )
            {
                throw new IllegalArgumentException( "A number " + sval + " should not exceed length of " + Rencode.MAX_INT_LENGTH );
            }
            write( Rencode.CHAR_INT );
            writeChars( sval );
        }
        else
        {
            writeLong( value.longValue() );
        }
    }

    public void writeCollection( Collection<?> list )
        throws IOException
    {
        if ( list.size() < Rencode.LIST_FIXED_COUNT )
        {
            write( Rencode.LIST_FIXED_START + list.size() );
            for ( Object elem : list )
            {
                writeObject( elem );
            }
        }
        else
        {
            write( Rencode.CHAR_LIST );
            for ( Object elem : list )
            {
                writeObject( elem );
            }
            write( Rencode.CHAR_TERMINATION );
        }
    }

    public <T> void writeMap( Map<T, ?> map )
        throws IOException
    {
        if ( map.size() < Rencode.DICTIONARY_FIXED_COUNT )
        {
            write( Rencode.DICTIONARY_FIXED_START + map.size() );
            for ( Entry<T, ?> elem : map.entrySet() )
            {
                writeObject( elem.getKey() );
                writeObject( elem.getValue() );
            }
        }
        else
        {
            write( Rencode.CHAR_DICT );
            for ( Entry<T, ?> elem : map.entrySet() )
            {
                writeObject( elem.getKey() );
                writeObject( elem.getValue() );
            }
            write( Rencode.CHAR_TERMINATION );
        }
    }

}
