/**
 * Copyright (C) 2020-2022 Dušan Vejnović <vaadin@dussan.org>
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.dussan.vaadin.dvirtual.xpra.window;

import java.io.Serializable;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.apache.commons.lang3.StringUtils;
import org.dussan.vaadin.dvirtual.xpra.packets.base.WindowType;
import org.dussan.vaadin.dvirtual.xpra.packets.server.ServerNewWindowPacket;
import org.dussan.vaadin.dvirtual.xpra.packets.server.ServerWindowMetadataPacket;

public class Window
    implements Serializable
{

    private static final long serialVersionUID = -1774427832741766039L;

    private boolean autoCenterEnabled = false;

    private boolean clipboardEnabled = false;

    private boolean drawingStarted = false;

    private boolean maximized = false;

    private boolean mousePreciseWheel = false;

    private boolean overrideRedirect = false;

    private boolean visible = false;

    private boolean windowRedrawRequsted = false;

    private byte[] icon = null;

    private double borderWidth = 0;

    private double dx = 0;

    private double dy = 0;

    private double mouseWheelDeltaX = 0;

    private double mouseWheelDeltaY = 0;

    private double offsetLeft = 0;

    private double offsetTop = 0;

    private double titleHeight = 0;

    private int height = 0;

    private int id = 0;

    private int minHeight = 0;

    private int minWidth = 0;

    private int parentId = 0;

    private int x = 0;

    private int y = 0;

    private int width = 0;

    private String applicationName = null;

    private String title = null;

    private transient List<String> modifiers = null;

    private Map<String, String> modifiersMap = null;

    private WindowType windowType = null;

    private WindowDimension restoredWindowDimension = null;

    private ServerWindowMetadataPacket metadata = null;

    public Window( ServerNewWindowPacket serverNewWindowPacket )
    {
        id = serverNewWindowPacket.getWindowId();
        x = serverNewWindowPacket.getX();
        y = serverNewWindowPacket.getY();
        width = serverNewWindowPacket.getWidth();
        height = serverNewWindowPacket.getHeight();
        metadata = serverNewWindowPacket.getMetadata();
        minWidth = metadata.getMinWidth();
        minHeight = metadata.getMinHeight();
        parentId = metadata.getParentId();
        applicationName = metadata.getApplicationName();
        title = metadata.getTitle();
        windowType = metadata.getWindowType();
        overrideRedirect = metadata.isOverrideRedirect();
        maximized = metadata.isMaximized();
        icon = metadata.getIcon()
                       .getImage();
        setRestoredWindowDimension( x, y, minWidth, height );
    }

    public boolean isAutoCenterEnabled()
    {
        return autoCenterEnabled;
    }

    public void setAutoCenterEnabled( boolean autoCenterEnabled )
    {
        this.autoCenterEnabled = autoCenterEnabled;
    }

    public boolean isClipboardEnabled()
    {
        return clipboardEnabled;
    }

    public void setClipboardEnabled( boolean clipboardEnabled )
    {
        this.clipboardEnabled = clipboardEnabled;
    }

    public boolean isDrawingStarted()
    {
        return drawingStarted;
    }

    public boolean isNotDrawingStarted()
    {
        return !drawingStarted;
    }

    public void setDrawingStarted( boolean drawingStarted )
    {
        this.drawingStarted = drawingStarted;
    }

    public boolean isMaximized()
    {
        return maximized;
    }

    public boolean isNotMaximized()
    {
        return !maximized;
    }

    public void setMaximized( boolean maximized )
    {
        setWindowRedrawRequsted( isMaximized() != maximized );
        this.maximized = maximized;
    }

    public boolean isMousePreciseWheel()
    {
        return mousePreciseWheel;
    }

    public void setMousePreciseWheel( boolean mousePreciseWheel )
    {
        this.mousePreciseWheel = mousePreciseWheel;
    }

    public boolean isOverrideRedirect()
    {
        return overrideRedirect;
    }

    public boolean isNotOverrideRedirect()
    {
        return !overrideRedirect;
    }

    public void setOverrideRedirect( boolean overrideRedirect )
    {
        this.overrideRedirect = overrideRedirect;
    }

    public boolean isVisible()
    {
        return visible;
    }

    public boolean isNotVisible()
    {
        return !visible;
    }

    public void setVisible( boolean visible )
    {
        this.visible = visible;
    }

    public boolean isWindowRedrawRequsted()
    {
        return windowRedrawRequsted;
    }

    public void setWindowRedrawRequsted( boolean windowRedrawRequsted )
    {
        this.windowRedrawRequsted = windowRedrawRequsted;
    }

    public Integer getId()
    {
        return id;
    }

    public Integer getParentId()
    {
        return parentId;
    }

    public boolean hasParent()
    {
        return ( parentId != ServerWindowMetadataPacket.NO_PARENT );
    }

    public byte[] getIcon()
    {
        return icon;
    }

    public void setIcon( byte[] icon )
    {
        this.icon = icon;
    }

    public ServerWindowMetadataPacket getMetadata()
    {
        return metadata;
    }

    public int getX()
    {
        return x;
    }

    public void setX( int x )
    {
        this.x = x;
    }

    public int getY()
    {
        return y;
    }

    public void setY( int y )
    {
        this.y = y;
    }

    public int getWidth()
    {
        return width;
    }

    public void setWidth( int width )
    {
        this.width = width;
    }

    public int getHeight()
    {
        return height;
    }

    public void setHeight( int height )
    {
        this.height = height;
    }

    public double getBorderWidth()
    {
        return borderWidth;
    }

    public void setBorderWidth( double borderWidth )
    {
        this.borderWidth = borderWidth;
    }

    public double getDx()
    {
        return dx;
    }

    public void setDx( double dx )
    {
        this.dx = dx;
    }

    public double getDy()
    {
        return dy;
    }

    public void setDy( double dy )
    {
        this.dy = dy;
    }

    public double getOffsetLeft()
    {
        return offsetLeft;
    }

    public void setOffsetLeft( double offsetLeft )
    {
        this.offsetLeft = offsetLeft;
    }

    public double getOffsetTop()
    {
        return offsetTop;
    }

    public void setOffsetTop( double offsetTop )
    {
        this.offsetTop = offsetTop;
    }

    public double getMouseWheelDeltaX()
    {
        return mouseWheelDeltaX;
    }

    public void setMouseWheelDeltaX( double mouseWheelDeltaX )
    {
        this.mouseWheelDeltaX = mouseWheelDeltaX;
    }

    public double getMouseWheelDeltaY()
    {
        return mouseWheelDeltaY;
    }

    public void setMouseWheelDeltaY( double mouseWheelDeltaY )
    {
        this.mouseWheelDeltaY = mouseWheelDeltaY;
    }

    public double getTitleHeight()
    {
        return titleHeight;
    }

    public void setTitleHeight( double titleHeight )
    {
        this.titleHeight = titleHeight;
    }

    public int getMinWidth()
    {
        return minWidth;
    }

    public void setMinWidth( int minWidth )
    {
        this.minWidth = minWidth;
    }

    public int getMinHeight()
    {
        return minHeight;
    }

    public void setMinHeight( int minHeight )
    {
        this.minHeight = minHeight;
    }

    public String getApplicationName()
    {
        return applicationName;
    }

    public void setApplicationName( String applicationName )
    {
        this.applicationName = StringUtils.defaultIfBlank( applicationName, this.applicationName );
    }

    public String getTitle()
    {
        return title;
    }

    public void setTitle( String title )
    {
        this.title = StringUtils.defaultIfBlank( title, this.title );
    }

    public List<String> getModifiers()
    {
        return modifiers;
    }

    public Map<String, String> getModifiersMap()
    {
        return modifiersMap;
    }

    public void setModifiers( List<String> modifiers )
    {
        this.modifiers = modifiers;
        modifiersMap = new HashMap<>();
        for ( int index = 0; index < modifiers.size(); index += 2 )
        {
            modifiersMap.put( modifiers.get( index ), modifiers.get( index + 1 ) );
        }
    }

    public WindowType getWindowType()
    {
        return windowType;
    }

    public void setWindowType( WindowType windowType )
    {
        this.windowType = windowType;
    }

    public WindowDimension getRestoredWindowDimension()
    {
        return restoredWindowDimension;
    }

    public void setRestoredWindowDimension( int x, int y, int width, int height )
    {
        restoredWindowDimension = new WindowDimension( x, y, width, height );
    }

}
