/**
 * Copyright (C) 2020-2022 Dušan Vejnović <vaadin@dussan.org>
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.dussan.vaadin.dvirtual;

import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertTrue;

import java.io.File;
import java.util.Arrays;
import java.util.List;

import org.dussan.vaadin.dvirtual.database.Database;
import org.dussan.vaadin.dvirtual.database.data.ApplicationData;
import org.dussan.vaadin.dvirtual.database.data.ApplicationsGroupData;
import org.dussan.vaadin.dvirtual.database.data.ServerData;
import org.dussan.vaadin.dvirtual.database.data.ServersGroupData;
import org.dussan.vaadin.dvirtual.database.data.UserData;
import org.junit.BeforeClass;
import org.junit.Test;

public class DatabaseMariadbTest
{

    private static final boolean TEST_DATA_ENABLED = false;

    private static final int TEST_DATA_ONE_INSTANCE = 0;

    private static final String DATABASE_FILENAME = "database.mariadb.conf";

    private static final String ERROR_APPLICATION_ADD = "Cannot add the new application.";

    private static final String ERROR_APPLICATION_REMOVE = "Cannot remove the new application.";

    private static final String ERROR_APPLICATIONS_GROUP_ADD = "Cannot add the new applications group.";

    private static final String ERROR_APPLICATIONS_GROUP_REMOVE = "Cannot remove the new applications group.";

    private static final String ERROR_DATABASE_INITIALIZATION = "Database is not initialized.";

    private static final String ERROR_SERVER_ADD = "Cannot add the new server.";

    private static final String ERROR_SERVER_REMOVE = "Cannot remove the new server.";

    private static final String ERROR_SERVERS_GROUP_ADD = "Cannot add the new servers group.";

    private static final String ERROR_SERVERS_GROUP_REMOVE = "Cannot remove the new servers group.";

    private static final String ERROR_USER_ADD = "Cannot add the new user.";

    private static final String ERROR_USER_REMOVE = "Cannot remove the new user.";

    private static final String FOLDER_SEPARATOR = File.separator;

    private static final String RESOURCE_FOLDER = String.join( FOLDER_SEPARATOR, "src", "test", "resources" );

    private static final String TEST_DATA = "TEST_DATA";

    private static final String TEST_DATA_IP = "10.10.10.10";

    private static String configFolder;

    private static Database database = null;

    @BeforeClass
    public static void init()
    {
        configFolder = new File( RESOURCE_FOLDER ).getAbsolutePath();
        database = new Database( configFolder, DATABASE_FILENAME );

        if ( database.isInitialized() )
        {
            // server data
            ServerData serverData = database.getServer( TEST_DATA );

            // servers group data
            ServersGroupData serversGroupData = database.getServersGroup( TEST_DATA );

            // application data
            List<ApplicationData> applicationDatas = database.getAllApplications();
            applicationDatas.removeIf( data -> !data.getName()
                                                    .equals( TEST_DATA ) );
            ApplicationData applicationData = applicationDatas.isEmpty() ? null : applicationDatas.get( 0 );

            // applications group data
            ApplicationsGroupData applicationsGroupData = database.getApplicationsGroup( TEST_DATA );

            // user data
            UserData userData = database.getUser( TEST_DATA );

            // remove user
            if ( userData != null )
            {
                database.removeUser( userData );
            }

            // remove applications group
            if ( applicationsGroupData != null )
            {
                database.removeApplicationsGroup( applicationsGroupData );
            }

            // remove application
            if ( applicationData != null )
            {
                database.removeApplication( applicationData );
            }

            // remove servers group
            if ( serversGroupData != null )
            {
                database.removeServersGroup( serversGroupData );
            }

            // remove server
            if ( serverData != null )
            {
                database.removeServer( serverData );
            }
        }
    }

    @Test
    public void database()
    {
        assertTrue( ERROR_DATABASE_INITIALIZATION, database.isInitialized() );
    }

    @Test
    public void serverTest()
    {
        ServerData serverData = new ServerData();
        serverData.setName( TEST_DATA );
        serverData.setNote( TEST_DATA );
        serverData.setAddress( TEST_DATA_IP );
        serverData.setSshPort( ServerData.DEFAULT_SSH_PORT );
        serverData.setXpraMinPort( ServerData.DEFAULT_XPRA_MIN_PORT );
        serverData.setXpraMaxPort( ServerData.DEFAULT_XPRA_MAX_PORT );
        serverData.setMaxUsers( ServerData.DEFAULT_MAX_USERS );
        serverData.setEnabled( TEST_DATA_ENABLED );

        // add server
        long serverId = database.addServer( serverData );
        assertTrue( ERROR_SERVER_ADD, serverId != ServerData.UNDEFINED_SERVER );

        // add again same server
        serverId = database.addServer( serverData );
        assertFalse( ERROR_SERVER_ADD, serverId != ServerData.UNDEFINED_SERVER );

        // remove server
        assertTrue( ERROR_SERVER_REMOVE, database.removeServer( serverData ) );
    }

    @Test
    public void serversGroupTest()
    {
        ServersGroupData serversGroupData = new ServersGroupData();
        serversGroupData.setName( TEST_DATA );
        serversGroupData.setNote( TEST_DATA );
        serversGroupData.setEnabled( TEST_DATA_ENABLED );

        // add servers group without servers
        long serversGroupId = database.addServersGroup( serversGroupData );
        assertTrue( ERROR_SERVERS_GROUP_ADD, serversGroupId == ServersGroupData.UNDEFINED_SERVERS_GROUP );

        // server data
        ServerData serverData = new ServerData();
        serverData.setName( TEST_DATA );
        serverData.setNote( TEST_DATA );
        serverData.setAddress( TEST_DATA_IP );
        serverData.setSshPort( ServerData.DEFAULT_SSH_PORT );
        serverData.setXpraMinPort( ServerData.DEFAULT_XPRA_MIN_PORT );
        serverData.setXpraMaxPort( ServerData.DEFAULT_XPRA_MAX_PORT );
        serverData.setMaxUsers( ServerData.DEFAULT_MAX_USERS );
        serverData.setEnabled( TEST_DATA_ENABLED );

        // add server
        long serverId = database.addServer( serverData );
        assertTrue( ERROR_SERVER_ADD, serverId != ServerData.UNDEFINED_SERVER );

        // add servers group with servers
        serversGroupData.setServerDatas( Arrays.asList( serverData ) );
        serversGroupId = database.addServersGroup( serversGroupData );
        assertTrue( ERROR_SERVERS_GROUP_ADD, serversGroupId != ServersGroupData.UNDEFINED_SERVERS_GROUP );
        assertTrue( serversGroupData.equals( new ServersGroupData( serversGroupData.getId(), serversGroupData.getName(), serversGroupData.getNote(), serversGroupData.isEnabled(),
                                                                   serversGroupData.getServerDatas() ) ) );
        // add again same servers group
        serversGroupId = database.addServersGroup( serversGroupData );
        assertFalse( ERROR_SERVERS_GROUP_ADD, serversGroupId != ServersGroupData.UNDEFINED_SERVERS_GROUP );

        // remove servers group
        assertTrue( ERROR_SERVERS_GROUP_REMOVE, database.removeServersGroup( serversGroupData ) );

        // remove server
        assertTrue( ERROR_SERVER_REMOVE, database.removeServer( serverData ) );
    }

    @Test
    public void applicationTest()
    {
        ApplicationData applicationData = new ApplicationData();
        applicationData.setName( TEST_DATA );
        applicationData.setCommand( TEST_DATA );
        applicationData.setNote( TEST_DATA );
        applicationData.setServerAutoSelect( TEST_DATA_ENABLED );
        applicationData.setOneInstance( TEST_DATA_ONE_INSTANCE );
        applicationData.setSameInstance( TEST_DATA_ENABLED );
        applicationData.setEnabled( TEST_DATA_ENABLED );

        // add application without servers group
        long applicationId = database.addApplication( applicationData );
        assertTrue( ERROR_APPLICATION_ADD, applicationId == ApplicationData.UNDEFINED_APPLICATION );

        // server data
        ServerData serverData = new ServerData();
        serverData.setName( TEST_DATA );
        serverData.setNote( TEST_DATA );
        serverData.setAddress( TEST_DATA_IP );
        serverData.setSshPort( ServerData.DEFAULT_SSH_PORT );
        serverData.setXpraMinPort( ServerData.DEFAULT_XPRA_MIN_PORT );
        serverData.setXpraMaxPort( ServerData.DEFAULT_XPRA_MAX_PORT );
        serverData.setMaxUsers( ServerData.DEFAULT_MAX_USERS );
        serverData.setEnabled( TEST_DATA_ENABLED );

        // add server
        long serverId = database.addServer( serverData );
        assertTrue( ERROR_SERVER_ADD, serverId != ServerData.UNDEFINED_SERVER );

        // servers group data
        ServersGroupData serversGroupData = new ServersGroupData();
        serversGroupData.setName( TEST_DATA );
        serversGroupData.setNote( TEST_DATA );
        serversGroupData.setEnabled( TEST_DATA_ENABLED );
        serversGroupData.setServerDatas( Arrays.asList( serverData ) );

        // add servers group
        long serversGroupId = database.addServersGroup( serversGroupData );
        assertTrue( ERROR_SERVERS_GROUP_ADD, serversGroupId != ServersGroupData.UNDEFINED_SERVERS_GROUP );

        // add application with servers group
        applicationData.setServersGroupDatas( Arrays.asList( serversGroupData ) );
        applicationId = database.addApplication( applicationData );
        assertTrue( ERROR_APPLICATION_ADD, applicationId != ApplicationData.UNDEFINED_APPLICATION );
        assertTrue( applicationData.equals( new ApplicationData( applicationData.getId(), applicationData.getName(), applicationData.getCommand(), applicationData.getCommandOptions(),
                                                                 applicationData.getNote(), applicationData.isServerAutoSelect(), applicationData.getOneInstance(), applicationData.isSameInstance(),
                                                                 applicationData.isEnabled(), applicationData.getIconData(), applicationData.getServersGroupDatas() ) ) );
        // add again same application
        applicationId = database.addApplication( applicationData );
        assertFalse( ERROR_APPLICATION_ADD, applicationId != ApplicationData.UNDEFINED_APPLICATION );

        // remove application
        assertTrue( ERROR_APPLICATION_REMOVE, database.removeApplication( applicationData ) );

        // remove servers group
        assertTrue( ERROR_SERVERS_GROUP_REMOVE, database.removeServersGroup( serversGroupData ) );

        // remove server
        assertTrue( ERROR_SERVER_REMOVE, database.removeServer( serverData ) );
    }

    @Test
    public void applicationsGroupTest()
    {
        ApplicationsGroupData applicationsGroupData = new ApplicationsGroupData();
        applicationsGroupData.setName( TEST_DATA );
        applicationsGroupData.setNote( TEST_DATA );
        applicationsGroupData.setEnabled( TEST_DATA_ENABLED );

        // add applications group without application
        long applicationsGroupId = database.addApplicationsGroup( applicationsGroupData );
        assertTrue( ERROR_APPLICATIONS_GROUP_ADD, applicationsGroupId == ApplicationsGroupData.UNDEFINED_APPLICATIONS_GROUP );

        // server data
        ServerData serverData = new ServerData();
        serverData.setName( TEST_DATA );
        serverData.setNote( TEST_DATA );
        serverData.setAddress( TEST_DATA_IP );
        serverData.setSshPort( ServerData.DEFAULT_SSH_PORT );
        serverData.setXpraMinPort( ServerData.DEFAULT_XPRA_MIN_PORT );
        serverData.setXpraMaxPort( ServerData.DEFAULT_XPRA_MAX_PORT );
        serverData.setMaxUsers( ServerData.DEFAULT_MAX_USERS );
        serverData.setEnabled( TEST_DATA_ENABLED );

        // add server
        long serverId = database.addServer( serverData );
        assertTrue( ERROR_SERVER_ADD, serverId != ServerData.UNDEFINED_SERVER );

        // servers group data
        ServersGroupData serversGroupData = new ServersGroupData();
        serversGroupData.setName( TEST_DATA );
        serversGroupData.setNote( TEST_DATA );
        serversGroupData.setEnabled( TEST_DATA_ENABLED );
        serversGroupData.setServerDatas( Arrays.asList( serverData ) );

        // add servers group
        long serversGroupId = database.addServersGroup( serversGroupData );
        assertTrue( ERROR_SERVERS_GROUP_ADD, serversGroupId != ServersGroupData.UNDEFINED_SERVERS_GROUP );

        // application data
        ApplicationData applicationData = new ApplicationData();
        applicationData.setName( TEST_DATA );
        applicationData.setCommand( TEST_DATA );
        applicationData.setNote( TEST_DATA );
        applicationData.setOneInstance( TEST_DATA_ONE_INSTANCE );
        applicationData.setServerAutoSelect( TEST_DATA_ENABLED );
        applicationData.setEnabled( TEST_DATA_ENABLED );
        applicationData.setServersGroupDatas( Arrays.asList( serversGroupData ) );

        // add application
        long applicationId = database.addApplication( applicationData );
        assertTrue( ERROR_APPLICATION_ADD, applicationId != ApplicationData.UNDEFINED_APPLICATION );

        // add applications group with applications
        applicationsGroupData.setApplicationDatas( Arrays.asList( applicationData ) );
        applicationsGroupId = database.addApplicationsGroup( applicationsGroupData );
        assertTrue( ERROR_APPLICATIONS_GROUP_ADD, applicationsGroupId != ApplicationsGroupData.UNDEFINED_APPLICATIONS_GROUP );
        assertTrue( applicationsGroupData.equals( new ApplicationsGroupData( applicationsGroupData.getId(), applicationsGroupData.getName(), applicationsGroupData.getNote(),
                                                                             applicationsGroupData.isSameInstance(), applicationsGroupData.isEnabled(),
                                                                             applicationsGroupData.getApplicationDatas() ) ) );
        // add again same applications group
        applicationsGroupId = database.addApplicationsGroup( applicationsGroupData );
        assertFalse( ERROR_APPLICATIONS_GROUP_ADD, applicationsGroupId != ApplicationsGroupData.UNDEFINED_APPLICATIONS_GROUP );

        // remove applications group
        assertTrue( ERROR_APPLICATIONS_GROUP_REMOVE, database.removeApplicationsGroup( applicationsGroupData ) );

        // remove application
        assertTrue( ERROR_APPLICATION_REMOVE, database.removeApplication( applicationData ) );

        // remove servers group
        assertTrue( ERROR_SERVERS_GROUP_REMOVE, database.removeServersGroup( serversGroupData ) );

        // remove server
        assertTrue( ERROR_SERVER_REMOVE, database.removeServer( serverData ) );
    }

    @Test
    public void userTest()
    {
        UserData userData = new UserData();
        userData.setUserName( TEST_DATA );
        userData.setFullName( TEST_DATA );
        userData.setMaxApplications( UserData.DEFAULT_MAX_APPLICATIONS );
        userData.setAdministrator( TEST_DATA_ENABLED );
        userData.setUpload( TEST_DATA_ENABLED );
        userData.setDownload( TEST_DATA_ENABLED );
        userData.setEnabled( TEST_DATA_ENABLED );

        // add user without applications group
        long userId = database.addUser( userData );
        assertTrue( ERROR_USER_ADD, userId != UserData.UNDEFINED_USER );

        // server data
        ServerData serverData = new ServerData();
        serverData.setName( TEST_DATA );
        serverData.setNote( TEST_DATA );
        serverData.setAddress( TEST_DATA_IP );
        serverData.setSshPort( ServerData.DEFAULT_SSH_PORT );
        serverData.setXpraMinPort( ServerData.DEFAULT_XPRA_MIN_PORT );
        serverData.setXpraMaxPort( ServerData.DEFAULT_XPRA_MAX_PORT );
        serverData.setMaxUsers( ServerData.DEFAULT_MAX_USERS );
        serverData.setEnabled( TEST_DATA_ENABLED );

        // add server
        long serverId = database.addServer( serverData );
        assertTrue( ERROR_SERVER_ADD, serverId != ServerData.UNDEFINED_SERVER );

        // servers group data
        ServersGroupData serversGroupData = new ServersGroupData();
        serversGroupData.setName( TEST_DATA );
        serversGroupData.setNote( TEST_DATA );
        serversGroupData.setEnabled( TEST_DATA_ENABLED );
        serversGroupData.setServerDatas( Arrays.asList( serverData ) );

        // add servers group
        long serversGroupId = database.addServersGroup( serversGroupData );
        assertTrue( ERROR_SERVERS_GROUP_ADD, serversGroupId != ServersGroupData.UNDEFINED_SERVERS_GROUP );

        // application data
        ApplicationData applicationData = new ApplicationData();
        applicationData.setName( TEST_DATA );
        applicationData.setCommand( TEST_DATA );
        applicationData.setCommandOptions( TEST_DATA );
        applicationData.setNote( TEST_DATA );
        applicationData.setServerAutoSelect( TEST_DATA_ENABLED );
        applicationData.setOneInstance( TEST_DATA_ONE_INSTANCE );
        applicationData.setSameInstance( TEST_DATA_ENABLED );
        applicationData.setEnabled( TEST_DATA_ENABLED );
        applicationData.setServersGroupDatas( Arrays.asList( serversGroupData ) );

        // add application
        long applicationId = database.addApplication( applicationData );
        assertTrue( ERROR_APPLICATION_ADD, applicationId != ApplicationData.UNDEFINED_APPLICATION );

        // applications group data
        ApplicationsGroupData applicationsGroupData = new ApplicationsGroupData();
        applicationsGroupData.setName( TEST_DATA );
        applicationsGroupData.setNote( TEST_DATA );
        applicationsGroupData.setEnabled( TEST_DATA_ENABLED );
        applicationsGroupData.setApplicationDatas( Arrays.asList( applicationData ) );

        // add applications group
        long applicationsGroupId = database.addApplicationsGroup( applicationsGroupData );
        assertTrue( ERROR_APPLICATIONS_GROUP_ADD, applicationsGroupId != ApplicationsGroupData.UNDEFINED_APPLICATIONS_GROUP );

        // add again same user
        userId = database.addUser( userData );
        assertFalse( ERROR_USER_ADD, userId != UserData.UNDEFINED_USER );

        // remove user
        assertTrue( ERROR_USER_REMOVE, database.removeUser( userData ) );

        // remove applications group
        assertTrue( ERROR_APPLICATIONS_GROUP_REMOVE, database.removeApplicationsGroup( applicationsGroupData ) );

        // remove application
        assertTrue( ERROR_APPLICATION_REMOVE, database.removeApplication( applicationData ) );

        // remove servers group
        assertTrue( ERROR_SERVERS_GROUP_REMOVE, database.removeServersGroup( serversGroupData ) );

        // remove server
        assertTrue( ERROR_SERVER_REMOVE, database.removeServer( serverData ) );
    }

}
